; $Id: dm_calc_deteff.pro,v 1.7.6.1 2013/05/29 15:09:24 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_calc_deteff
;
; PURPOSE:
;   calculate detector efficiency
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################


;this function calculates the intensity ratio of two macs detectors
;parameters:
;    a4_1:  [na4_1]
;    a4_2:  [na4_2]
;    int_1: [2,na4_1]
;    int_2: [2,na4_2]
;keywords:
;   ignore_a4: [2], the A4 range to be ignored in the calculation
;   intavg:  if set, average the intensity first, then calculate the ratio, otherwise calculate the average of the ratio
function dm_macs_int_ratio,a4_1,a4_2,int_1,int_2,ignore_a4=ignore_a4,intavg=intavg
    if n_elements(ignore_a4) eq 2 then begin       
       indx = where((a4_1 lt ignore_a4[0]) or (a4_1 gt ignore_a4[1]),cnt)
       if cnt eq 0 then return,[!values.f_nan,!values.f_nan] $
       else begin
          a4_1  = a4_1[indx]
          int_1 = int_1[*,indx] 
       endelse
       indx = where((a4_2 lt ignore_a4[0]) or (a4_2 gt ignore_a4[1]),cnt)
       if cnt eq 0 then return,[!values.f_nan,!values.f_nan] $
       else begin
          a4_2  = a4_2[indx]
          int_2 = int_2[*,indx] 
       endelse
    endif
    range = [min(a4_2),max(a4_1)]
    indx0 = where((a4_1 ge range[0]) and (a4_1 le range[1]),cnt0) 
    indx1 = where((a4_2 ge range[0]) and (a4_2 le range[1]),cnt1)
    cnt   = (cnt0<cnt1)
    if keyword_set(intavg) then begin
       int1  = total(int_1[*,indx0[0:cnt-1]],2)/cnt
       int2  = total(int_2[*,indx1[0:cnt-1]],2)/cnt
       ratio = int1/int2
    endif else $
       ratio = int_1[*,indx0[0:cnt-1]]/int_2[*,indx1[0:cnt-1]]
    if n_elements(ratio) gt 2 then ratio = total(ratio,2)/n_elements(ratio[0,*])
    return,ratio
end

; parameter:
;   qty:  [ne,ndet], value will be changed if maskdet exists
; keyword:
;   kidney: if present, this is for MACS file, maskdet will be the a4 range to be ignored
;   fit:  if set, fit by an gaussian and use the data in the gaussian fucntion range
;   maskdet: detector indices to be masked, those detectors will be given a value of 1.0; for macs, this is the a4 range to be ignored
;   intavg:  for macs, if set, average the intensity first, then calculate the ratio, otherwise calculate the average of the intensity ratios
function dm_calc_deteff,qty,fit=fit,kidney=kidney,intavg=intavg,maskdet=maskdet,debug=debug
    nkid = n_elements(kidney)
    if nkid ne 0 then begin
       ;first sort the kidney angles
       index = sort(kidney)
       tmpkid = kidney[index]
       tmpqty = qty[*,*,index]
       a4 = fltarr(20,nkid)
       for i=0,19 do a4[i,*] = tmpkid-76.0+8.0*i
       eff = fltarr(2,20)+1 & eff[*,1:19] = !values.f_nan
       for i=1,19 do begin
           ratio = dm_macs_int_ratio(reform(a4[i-1,*]),reform(a4[i,*]),reform(tmpqty[*,i-1,*]),reform(tmpqty[*,i,*]),ignore_a4=maskdet,intavg=intavg)
           if total(finite(ratio)) eq 2 then begin
              eff[*,i] = ratio
              for j=0,1 do tmpqty[j,i,*] = tmpqty[j,i,*]*eff[j,i]
           endif
       endfor
       for i=0,1 do eff[i,*] = eff[i,*]*total(finite(eff[i,*]))/total(eff[i,*],/nan) 
    endif else begin
       if n_elements(maskdet) ne 0 then begin
          qty[*,maskdet] = 0.0
       endif
       if keyword_set(fit) then begin
          ;first fit the det-sum and find out the elastic range
          y = total(qty,2,/double)
          dm_gaussfit,y,ny=ny,params=params,fitnotgood=fitnotgood,debug=debug
          if fitnotgood then begin
             eff  = total(qty,1,/double)
          endif else begin
             tmp  = shift(qty,ny/2-params[1],0)
             istart = (0>(ny/2-fix(2.5*params[2])))
             iend = ((ny-1)<(ny/2+fix(2.5*params[2])))
             tmp  = tmp[istart:iend,*] 
             eff  = total(tmp,1,/double)
          endelse
       endif else begin
          eff = total(qty,1,/double)
       endelse
       index0 = where(eff eq 0,count,complement=index1)
       if count ne 0 then begin
          minv = min(eff[index1])
          eff[index0] = minv
       endif
       eff = float(eff/mean(eff,/double))
       if n_elements(maskdet) ne 0 then eff[maskdet] = 1.0
    endelse
    return, eff
end