; $Id: dm_stepgrid_bin.pro,v 1.12 2013/03/15 21:57:46 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_stepgrid_bin
;
; PURPOSE:
;  bin data according to give xstep and ystep
;
; CATEGORY:
;  general
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  March, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameters:
;   in_xstep:   xstep
;   in_ystep:   ystep
;   xdat:       input xdat[ne,ndet]    output xdat[xsize]
;   ydat:       input ydat[ne,ndet]    output ydat[ysize]
;   zdat:       input zdat[ne,ndet]    output zdat[xsize,ysize]
; keyword:
;   xstart:   binning starting point, if absent, use the min(xdat) as the starting point
;   ystart:   binning starting point, if absent, use the min(ydat) as the starting point
;   checkfinite: flag for checking finite values of zdat
;   uniq_xval:  if set, then use the given unique values instead of the steps to determine xbin
;   uniq_yval:  if set, then use the given unique values instead of the steps to determine ybin
;   avgsum:     0: average 1:sum
;   bintr:      binning threshold, bins with fewer data points than bintr*(max number of points in a bin) will be discarded
;   conststep:  0: use the average as the resulted binning values(default) 1: constant binning intervals
;   leave_blank: if set, leave empty column or rows as they are
;   ewid:       energy width, used when e-integrated intensity option is chose
;               input  dat=sum(I*ewid)
;               output dat=sum(I*ewid)/sum(ewid)
;   extlib:     0:not using external library 1:use external library
;   zerr:       input error bar of zdat[ne,ndet], ouput zerr[xsize,ysize]
pro dm_stepgrid_bin,in_xstep,in_ystep,xdat,ydat,zdat,xstart=xstart,ystart=ystart,checkfinite=checkfinite,uniq_xval=uniq_xval,uniq_yval=uniq_yval,$
    debug=debug,avgsum=avgsum,bintr=bintr,conststep=conststep,ewid=ewid,leave_blank=leave_blank,zerr=zerr,extlib=extlib
    if n_elements(avgsum) eq 0 then avgsum=0          ;default using average in binning
    if n_elements(bintr)  eq 0 then bintr=0.0
    if n_elements(conststep) eq 0 then conststep = 0  ;default is using average as the binned value
    if keyword_set(conststep) then leave_blank=1
    if n_elements(extlib) eq 0 then extlib = 1b       ;default using external binning library if available
    if keyword_set(debug) then current=systime(/sec)
    nxdat   = n_elements(xdat)
    ok_eint = n_elements(ewid) eq nxdat
    ok_zerr = n_elements(zerr) eq nxdat
    if keyword_set(checkfinite) then begin
       ind = where(finite(zdat),count) 
       if (count ne nxdat) and (count ne 0) then begin
          xdat = xdat[ind]
          ydat = ydat[ind]
          zdat = zdat[ind]
          if ok_zerr then zerr = zerr[ind]
          if ok_eint then ewid = ewid[ind]
          ind  = 0 & nxdat = count
       endif
    endif
    hstep = abs(in_xstep)
    vstep = abs(in_ystep)
    xmax  = max(xdat,min=xmin)
    ymax  = max(ydat,min=ymin)
    if n_elements(xstart) eq 1 then begin
       if finite(xstart) then xmin = xstart
    endif
    if n_elements(ystart) eq 1 then begin
       if finite(ystart) then ymin = ystart
    endif
    if (hstep eq 0) then hstep = ((xmax eq xmin)?0.1:(xmax-xmin)/100.)
    if (vstep eq 0) then vstep = ((ymax eq ymin)?0.1:(ymax-ymin)/100.)
    xmax  = xmax+hstep/2.
    xmin  = xmin-hstep/2.
    ymax  = ymax+vstep/2.
    ymin  = ymin-vstep/2.
    xsize = (xmax-xmin)/hstep
    ysize = (ymax-ymin)/vstep
    if xsize gt fix(xsize,type=3) then xsize=(2.>(xsize+0.5))
    if ysize gt fix(ysize,type=3) then ysize=(2.>(ysize+0.5))
    xsize   = fix(xsize,type=3) & ysize=fix(ysize,type=3)
    xlookup = xmin+findgen(xsize+1)*hstep
    ylookup = ymin+findgen(ysize+1)*vstep
    if n_elements(uniq_xval) ne 0  then begin
       xlookup = uniq_xval[uniq(uniq_xval,sort(uniq_xval))]
       xlookup = [xlookup,xmax]
    endif
    if n_elements(uniq_yval) ne 0 then begin
       ylookup = uniq_yval[uniq(uniq_yval,sort(uniq_yval))]
       ylookup = [ylookup,ymax]
    endif
    xsize = n_elements(xlookup)
    ysize = n_elements(ylookup)
    tmp_x = fltarr(xsize) & num_x = lonarr(xsize)
    tmp_y = fltarr(ysize) & num_y=lonarr(ysize)
    tmp_z = fltarr(xsize,ysize)& num_z = lonarr(xsize,ysize)
    if ok_eint then tmp_w = fltarr(xsize,ysize)
    if ok_zerr then tmp_e = fltarr(xsize,ysize)
    
    defsysv,'!DAVE_AUXILIARY_DIR',exists=exists
    if exists and extlib then begin
       filename = 'rebinnd'+'_'+!version.os+'_'+strtrim(string(!VERSION.MEMORY_BITS),2)
       shlib = (FILE_SEARCH(!DAVE_AUXILIARY_DIR+'rebin_nd'+path_sep()+filename+'*'))[0]
    endif else shlib = ''
    
    if ((strlen(shlib) gt 0) and (n_elements(uniq_xval) eq 0) and (n_elements(uniq_yval) eq 0))then begin
      if (ok_eint and ok_zerr) then begin
        r = CALL_EXTERNAL(shlib, 'rebin2d_zew', long(nxdat),float(xdat),float(ydat),$
        double(zdat),double(zerr),double(ewid),long(xsize),long(ysize),float(xmin),float(hstep),$
        float(ymin),float(vstep), num_x, tmp_x, num_y, tmp_y, num_z, tmp_z, tmp_e, tmp_w, /CDECL) 
      end       
      if (ok_eint and ~ok_zerr) then begin
        r = CALL_EXTERNAL(shlib, 'rebin2d_zw', long(nxdat),float(xdat),float(ydat),$
        double(zdat),double(ewid),long(xsize),long(ysize),float(xmin),float(hstep),$
        float(ymin),float(vstep), num_x, tmp_x, num_y, tmp_y, num_z, tmp_z, tmp_w, /CDECL) 
      end       
      if (~ok_eint and ok_zerr) then begin
        r = CALL_EXTERNAL(shlib, 'rebin2d_ze', long(nxdat),float(xdat),float(ydat),$
        double(zdat),double(zerr),long(xsize),long(ysize),float(xmin),float(hstep),$
        float(ymin),float(vstep), num_x, tmp_x, num_y, tmp_y, num_z, tmp_z, tmp_e, /CDECL) 
      end       
      if (~ok_eint and ~ok_zerr) then begin
        r = CALL_EXTERNAL(shlib, 'rebin2d_z', long(nxdat),float(xdat),float(ydat),$
        double(zdat),long(xsize),long(ysize),float(xmin),float(hstep),$
        float(ymin),float(vstep), num_x, tmp_x, num_y, tmp_y, num_z, tmp_z,  /CDECL) 
      end 
    end else begin
      for i=0L,nxdat-1L do begin
        ix = value_locate(xlookup,xdat[i])
        iy = value_locate(ylookup,ydat[i])
        tmp_x[ix] = tmp_x[ix]+xdat[i] & num_x[ix] = num_x[ix]+1L
        tmp_y[iy] = tmp_y[iy]+ydat[i] & num_y[iy] = num_y[iy]+1L
        tmp_z[ix,iy] = tmp_z[ix,iy]+zdat[i] & num_z[ix,iy] = num_z[ix,iy]+1L
        if ok_eint then tmp_w[ix,iy] = tmp_w[ix,iy]+ewid[i]
        if ok_zerr then tmp_e[ix,iy] = tmp_e[ix,iy]+zerr[i]*zerr[i]
      endfor
    endelse
    
    indx = where(num_x ne 0,countx,complement=indx1,ncomplement=countx1)
    indy = where(num_y ne 0,county,complement=indy1,ncomplement=county1)
    if keyword_set(leave_blank) then begin
       if countx1 ne 0 then begin
          num_x[indx1]=1.0 & tmp_x[indx1]=xlookup[indx1]
       endif
       if county1 ne 0 then begin
          num_y[indy1]=1.0 & tmp_y[indy1]=ylookup[indy1]
       endif
    endif else begin
       if (countx ne 0) and (countx1 ne 0) then begin
          tmp_x = tmp_x[indx]   & num_x = num_x[indx]
          tmp_z = tmp_z[indx,*] & num_z = num_z[indx,*]
          if ok_eint then tmp_w = tmp_w[indx,*]
          if ok_zerr then tmp_e = tmp_e[indx,*]
       endif
       if (county ne 0) and (county1 ne 0) then begin
          tmp_y = tmp_y[indy]   & num_y = num_y[indy]
          tmp_z = tmp_z[*,indy] & num_z = num_z[*,indy]
          if ok_eint then tmp_w = tmp_w[*,indy]
          if ok_zerr then tmp_e = tmp_e[*,indy]
       endif
    endelse
    indx=-1 & indx1=-1 & indy=-1 & indy1=-1
    if keyword_set(conststep) and n_elements(uniq_xval) eq 0 then begin
       xdat  = xmin+(findgen(xsize-1)+0.5)*hstep
       tmp_z = tmp_z[0:xsize-2,*]
       num_z = num_z[0:xsize-2,*]
       if ok_zerr then tmp_e = tmp_e[0:xsize-2,*]
       if ok_eint then tmp_w = tmp_w[0:xsize-2,*]
    endif else $
       xdat = temporary(tmp_x)/temporary(num_x)
    if keyword_set(conststep) and n_elements(uniq_yval) eq 0 then begin
       ydat = ymin+(findgen(ysize-1)+0.5)*vstep
       tmp_z = tmp_z[*,0:ysize-2]
       num_z = num_z[*,0:ysize-2]
       if ok_zerr then tmp_e = tmp_e[*,0:ysize-2]
       if ok_eint then tmp_w = tmp_w[*,0:ysize-2]
    endif else $
       ydat = temporary(tmp_y)/temporary(num_y)
    zdat = temporary(tmp_z)
    if ok_zerr then zerr = sqrt(temporary(tmp_e))
    index1 = where(num_z ne 0,count1,complement=index0,ncomplement=count0)
    if (count1 ne 0) then begin
       if ok_eint then begin
          zdat[index1] = zdat[index1]/tmp_w[index1]
          if ok_zerr then zerr[index1] = zerr[index1]/tmp_w[index1]
       endif else if avgsum eq 0 then begin
          zdat[index1] = zdat[index1]/num_z[index1]    ;average
          if ok_zerr then zerr[index1] = zerr[index1]/num_z[index1]
       endif
       if (bintr gt 0.0) and (bintr lt 1.0) then begin
          min_num_z = bintr*(max(num_z)>1)
          ind = where(num_z lt min_num_z,ct1)
          if ct1 gt 0 then begin
             zdat[ind] = !values.f_nan
             if ok_zerr then zerr[ind] = !values.f_nan
          endif          
       endif
    endif
    if (count0 ne 0) then begin
       zdat[index0] = !values.f_nan
       if ok_zerr then zerr[index0] = !values.f_nan
    endif
    if keyword_set(debug) then print,'stepgrid_bin finished in ',systime(/sec)-current,' secs.'
end