; $Id: dm_to_string.pro,v 1.12.2.1 2013/05/29 15:09:26 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_to_string
;
; PURPOSE:
;  convert numbers to string
;  if number is !values.f_nan then return ''
;
; CATEGORY:
;  general
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;keywords:
;   int:        if set, the string contains no decimal digits
;   separator:  if present, use separator string to combine the string arrays into one string
;   resolution: a natural number indicating the number of decimal places for resolution in the string
;   strlength:  prescribed string length, if the returned string length is less than strlen, blank spaces are filled in
;   exponent:   if set, use **e** format
function dm_to_string,number,int=int,separator=separator,resolution=resolution,strlength=strlength,exponent=exponent
    tmp = size(number,/type)
    if tmp eq 7 then return,strtrim(number,2)  ;a string
    n_num = n_elements(number)
    n_end = 0   ;for smart '.0' ending
    if n_num eq 0 then return, ''
    if n_num eq 1 then str = '' else str = strarr(size(number,/dimension))
    tmpnumber = number
    if tmp eq 1 then tmpnumber = fix(number)   ;number is byte
    if n_elements(resolution) ne 0 then begin
       myresolution = 0>(fix(resolution[0]))
       if (~keyword_set(int)) then begin
          ind = where(finite(tmpnumber),count)
          if count gt 0 then tmpnumber[ind] = round(tmpnumber[ind]*((10.0d)^myresolution)+(0.0001d),/L64)/((10.0d)^myresolution)
       end
    endif else myresolution = 1 
    for i=0L,n_num-1 do begin
        if finite(tmpnumber[i]) then begin
           if keyword_set(int) then str[i]=strtrim(string(fix(tmpnumber[i],type=14)),2) $
           else begin
              if keyword_set(exponent) then $
                 str[i] = strtrim(string(tmpnumber[i],format='(e)'),2) $
              else $
                 str[i] = strtrim(string(tmpnumber[i]),2)
              ;get rid of leading 0's and '+'
              tmp  = strmid(str[i],0,1)
              head = (tmp eq '-')?'-':''
              if tmp eq '+' or tmp eq '-' then $
                 str[i] = strmid(str[i],1,strlen(str[i])-1)
              while (strmid(str[i],0,1) eq '0') and (strlen(str[i]) ne 1)  do $
                 str[i] = strmid(str[i],1,strlen(str[i])-1)
              if strmid(str[i],0,1) eq '.' then str[i] = '0'+str[i]
              str[i] = head+str[i]
              ;get rid of trailing 0's for floats
              if strpos(str[i],'e') ne -1 then begin
                 tmp  = strsplit(str[i],'e',/extract,/preserve)
                 tmp1 = dm_to_string(fix(tmp[0],type=5))
                 while (strmid(tmp1,1,2,/reverse) eq '10') and ~stregex(tmp1,'\.',/boolean) do begin ;'10' or '-10'
                    tmp1 = strmid(tmp1,0,strlen(tmp1)-1)
                    tmp[1] = tmp[1]+1
                 endwhile
                 str[i] = tmp1+'e'+dm_to_string(fix(tmp[1],type=2))
              endif else begin
                 tmp1 = strpos(str[i],'.')
                 if tmp1 ne -1 then $
                    str[i] = str[i]+' ' ;add a trailing empty space
                 tmp = strsplit(str[i],'[0]+ ',/reg,/extract)   ;one or more 0 and one blank space
                 str[i] = strtrim(tmp[0],2)
                 if strmid(str[i],0,1,/reverse_offset) eq '.' then n_end = n_end+1 
                 if tmp1 ne -1 then begin
                    tmp2 = strlen(str[i])
                    if (n_num gt 1) or (tmp1 eq tmp2-1) then $ 
                       for j=0,myresolution-(tmp2-tmp1-1)-1 do str[i] = str[i]+'0'
                 endif 
              endelse
           endelse
        endif
    endfor
    if n_end eq n_num then begin
       for i=0L,n_num-1 do $
           str[i] = strmid(str[i],0,strlen(str[i])-myresolution-1)  ;get rid of '.0' ending
    endif
    if n_elements(strlength) eq 1 then begin
       if strlength lt 0 then strlength = 0
       for i=0L,n_num-1 do begin
           tmp2 = strlen(str[i])
           for j=0,strlength-tmp2-1 do str[i] = str[i]+' '
       endfor
    endif
    if n_num eq 1 then str=str[0] else begin
       if n_elements(separator) eq 1 then begin
          tmpstr = str[0]
          for i=1L,n_num-1 do begin
              ;separator will be inserted before the trailing blanks
              tmp = stregex(tmpstr,' *$')
              tmpstr = strmid(tmpstr,0,tmp)+separator+strmid(tmpstr,tmp,strlen(tmpstr)-tmp)+str[i]
          endfor
          str = tmpstr
       endif 
    endelse
    return,str
end