; $Id: dcs_mslice.pro,v 1.151.2.6 2013/05/29 15:09:24 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dcs_mslice
;
; PURPOSE:
;  Viewing and analyzing neutron time-of-flight data.
;
; CATEGORY:
;  dcs time-of-flight
;
; HISTORY:
;  06/2003: IDL5.6 compatible, include all functionality of matlab mslice, and windows .hlp help file
;  10/2003: IDL6.0 compatible, add diffuse scattering type, and dcs_mslice.pdf help file
;  09/2004: Make IDL dcs_mslice extendable to other TOF instruments, documentation in dcs_mslice_addftype.pdf
;  06/2005: IDL6.1 compatible, add diffuse scattering multiple energy type, rearange sample types
;  11/2005: Remove some items in the droplists, and add extra HKL range limitation
;  03/2006: Add INX file type
;  03/2007: Add grouped data fitting in powder mode, i.e. add QENS data reduction
;           modifications in background subtraction, especially the treatment of dark count files
;  07/2007: Add empty can subtraction from vanadium file in the first two sample types
;           Add automatic Bragg peak masking
;  02/2008: Compatible with inverse geometry TOF instruments
;  11/2008: more flexible fold x-axis operation, add energy resolution convolution tool
;  05/2009: revamp viewing axes, allow more flexbility in folding and viewing
;  06/2009: add stand-alone interface for general volume data
;  07/2009: add volume plot, and change the display window to tab widgets
;  11/2009: allow two sets of empty can backgrounds, rebuild the context sensitive menu 
;  12/2009: more robust automatic masking in SPE files
;  02/2010: added NXSPE support - Andrei Savici (saviciat@ornl.gov)
;  04/2010: possible to discard bins that have few pixels mapped into them 
;           keep track of correct ki/kf powers for NXSPE files - Andrei Savici (saviciat@ornl.gov)
;  05/2010: modify the way to handle the masking in SPE and NXSPE files      
;  05/2010: add user function option 
;  09/2010: IDL 8.0 compatible
;  11/2010: add dark angle trajectory
;  03/2011: add HFIR WAND instrument, allow user function to be executed before projection, 
;  04/2011: add script file execution
;  05/2011: Move "Option->Allow Extra Viewing Axis" to "Option->Viewing Axis->Allow Extra Viewing Axis".
;  11/2011: Allow WAND user to change Ei and detector offset angle; revamp the popup status menu
;  11/2011: Proper flat plate absorption calculation in diffuse scattering mode.
;  12/2011: Add cuts and slices in dm_plot for IDL 8.0 and after
;  06/2012: IDL 8.2 compatible
;  08/2012: Add PSUTXT keyword in save_slice
;  09/2012: Add more LaTex commands in dm_plot text input, and more font choices
;  11/2012: Add DCS Q&E-resolution convolution tool
;  12/2012: Add multiple slice output in saving volume data
;  02/2013: Add MACS, fully functional
;  03/2013: Add ftp file server to file list in the file selection window
;  04/2013: Add Temperature as extra viewing axis for MACS and WAND
;  05/2013: Add Calculator Tab
;  
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;files need to be included
@dm_calc_projection
@dm_choose_cutdata
@dm_choose_slicedata
@dm_choose_volumedata
@dm_choose_trajdata
@dm_det_spurion
@dm_filemenu
@dm_filetools
@dm_load_parameter
@dm_mask_detbanks
@dm_plot_cut
@dm_plot_slice
@dm_plot_volume
@dm_plot_trajectory
@dm_save_parameter
@dm_to_fit
@dcs_plotdetmask    ;for plotting masks

;kill the object
pro dcs_mslice_Exit,tlb
    widget_control,tlb,get_uvalue=self
    if obj_valid(self) then obj_destroy,self
end

;widget event handler
pro dcs_mslice_event,event
    widget_control,event.handler,get_uvalue=self
    if obj_valid(self) then self->event,event
end

;absorption correction
;varible:
;   type: -1-none 0-cylinder 1-annulus 2-sphere 3-flat plate
pro dcs_mslice::absorption,type
    old = -1
    if type lt -1 or type gt 3 then return
    default = [0,0,!values.f_nan,!values.f_nan,!values.f_nan,!values.f_nan,!values.f_nan]
    if ptr_valid(self.absp_info) then begin
       old   = (*(self.absp_info))[0]
       ninfo = n_elements(*(self.absp_info))
       default[0:(ninfo-2)] = (*(self.absp_info))[1:(ninfo-1)]
       ptr_free,self.absp_info
    endif
    case type of
          0 :   begin   ;cylinder
                names  = ['L0(cm):','L1(cm):','R(cm):','Sampling Points:','Interpolate:']
                title  = 'Cylindrical geometry'
                end
          1 :   begin   ;annulus
                names  = ['L0(cm):','L1(cm):','Rout(cm):','Rin(cm):','Sampling Points:','Interpolate:']
                title  = 'Annular geometry'
                end
          2 :   begin   ;sphere
                names  = ['L0(cm):','L1(cm):','R(cm):','Sampling Points:','Interpolate:']
                title  = 'Spherical geometry'
                end
          3 :   begin   ;flat plate
                names  = ['L0(cm):','L1(cm):','Width(cm):','Thickness(cm):','Angle('+string('B0'XB)+'):','Sampling Points:','Interpolate:']
                title  = 'Flat plate geometry'
                end
        else:           ;none
    endcase
    nitem = n_elements(names)
    if nitem ne 0 then begin
       npts  = *self.absp_npts[type]
       nptst = *self.absp_nptst[type]
       if self.samp_typ ge 2 then default[1]=0 ;no need to interpolate for diffuse scattering type
       default = [default[2:(nitem-1)],default[0:1]]
       is_droplist = intarr(nitem) & is_droplist[nitem-2:nitem-1] = 1
       mesg = ['L0 is the energy independent mean free path;',$
               'L1 is the energy dependent mean free path for',$
               '2200m/s or 25.3 meV neutron.',$
               'The effective mean free path is:',$
               'L=1/{1/L0+1/(L1*sqrt[E(meV)/25.3])}' ]
       if type eq 3 then mesg = [mesg,'','The angle is from ki to the normal of the plate'+([(['.',' for the first A3 angle.'])[self.instrname eq 'macs'],$
               ' for the first file.'])[self.samp_typ ge 2],'Counterclockwise rotation is positive.']
       info = dm_dialog_input(names,default=default,/float,info=mesg,is_droplist=is_droplist,droplist_content=[ptr_new(nptst),ptr_new(['No','Yes'])],$
                /return_number,title=title,xsize=140,dialog_parent=self.tlb,cancel=cancel)
       if (total(finite(info)) eq nitem) and (~keyword_set(cancel)) then begin
          self.absp_info = ptr_new([type,info[nitem-2:nitem-1],info[0:(nitem-3)]])
       endif else begin
          type = -1
          if old ne -1 then ok = dialog_message((keyword_set(cancel)?(''):('Invalid inputs. '))+'Absorption correction is disabled.',dialog_parent=self.tlb,/error,/center)
       endelse
    endif
    self->my_menu_toggle,'absp'+['None','Cylinder','Annulus','Sphere','Flat'],type+1
    if ((type ne -1) or (old ne -1)) and ptr_valid(self.projStrPtr) and ptr_valid(self.dataStrPtr) then self->set_display_button,/clearprojection ;need to recalculate projections
end

;adjust the viewing axis label according to hkl value
pro dcs_mslice::adjust_viewlabel,uid
    for i=0,n_elements(uid)-1 do begin
        tmp = ([[self.view_u1],[self.view_u2],[self.view_u3]])[*,uid[i]]
        if tmp[0] ne 0 then $
           label = 'H'      $
        else if tmp[1] ne 0 then $
           label = 'K'      $
        else if tmp[2] ne 0 then $
           label = 'L'      
        if n_elements(label) ne 0 then begin
           if uid[i] eq 0 then begin
              self->script,'set_parameter',name='viewU1lab',value=label
           endif else if uid[i] eq 1 then begin
              self->script,'set_parameter',name='viewU2lab',value=label
           endif else begin
              self->script,'set_parameter',name='viewU3lab',value=label
           endelse
        endif
    endfor
end

;change show powder line option
pro dcs_mslice::change_powderline,option,menuonly=menuonly
    if n_elements(option) eq 0 then return
    if (self.powd_typ ne 0) and (self.powd_typ ne option) and (option ne 0)then begin
       self.powd_latta = !values.f_nan ;reset saved value
       self.powd_bcc   = 0b
    endif 
    self.powd_typ = 0>(option)<4
    self->my_menu_toggle,'powder'+['None','Al','Cu','St','Sp'],self.powd_typ
    dm_set_button,widget_info(self.tlb,find_by_uname='powdershowall'),self.powd_showall,onstring='Show Closed Powder Line',offstring='Show Actual Powder Line',sensitive=(self.powd_typ ne 0)
    if keyword_set(menuonly) then return
    default = dm_to_string([0,4.05,3.6148,3.594,!values.f_nan]) ; _, Al, Cu, SS, specify
    if ptr_valid(self.powd_qvalue) then default[4]=dm_to_string(*self.powd_qvalue,separator=', ')
    default = default[self.powd_typ]
    ptr_free,self.powd_qvalue,self.powd_line    
    if self.powd_typ gt 0 and self.powd_typ lt 4 then begin ;ask for the lattice parameter
       if (self.powd_typ eq 3) and self.powd_bcc then begin
          str_str = 'BCC'
          default = '2.87'
       endif else $
          str_str = 'FCC'
       if finite(self.powd_latta) then begin
          if self.powd_latta ne dm_to_number(default) then begin
             info = 'Try a = '+default+' '+string('c5'XB)+' for '+str_str+([' aluminum',' copper',' stainless steel'])[self.powd_typ-1]+'.'
             default = dm_to_string(self.powd_latta)
          endif
       endif 
       if n_elements(info) eq 0 then info = 'Press OK to accept the default '+str_str+' lattice parameter. 
       name   = 'a ('+string('c5'XB)+'):'
       title  = (['Al','Cu','Stainless Steel'])[self.powd_typ-1]+' Lattice Parameter'
       latt_a = !values.f_nan
       if self.powd_typ eq 3 then begin
          info    = [info,'Try a = '+(['2.87 ','3.594 '])[self.powd_bcc]+string('c5'XB)+' for '+(['BCC','FCC'])[self.powd_bcc]+' stainless steel.']
          default = [default,dm_to_string(self.powd_bcc,/int)]
          name    = [name,'structure:']
       endif  
       while(finite(latt_a,/nan)) do begin
          if self.powd_typ ne 3 then begin
             latt_a = dm_dialog_input(name,xsize=80,dialog_parent=self.tlb,info=info,default=default,/float,title=title) 
          endif else begin
             latt_a = dm_dialog_input(name,xsize=80,dialog_parent=self.tlb,info=info,default=default,/float,title=title,droplist_content=ptr_new(['FCC','BCC']),is_droplist=[0,1],/return_number) 
             is_bcc = (latt_a[1] eq 1)    ;BCC structure
             latt_a = latt_a[0]
             self.powd_bcc = is_bcc
          endelse
       endwhile
       self.powd_latta = latt_a
       if keyword_set(is_bcc) then $  ;BCC structure, enough for 2518 meV for BCC SS  
          qsqrt = sqrt([2.,4,6,8,10,12,14,16,18,20,22,24,26,30,32,34,36,38,40,42,44,46,48,50,52,54,56,58,62,64,66,68,70,$
                   72,74,76,78,80,82,84,86,88,90,94,96,98,100,102,104,106,108,110,114,116,118,120,122,126,128,130,$
                   132,134,136,138,140,142,144,146,148,150,152,154,158,160,162,164,166,168,170,172,174,176,178,180,$
                   182,184,186,190,192,194,196,198,200,202,204,206,208,210,212,214,216,218,222,224,226,228,230,232,$
                   234,236,238,242,244,246,248,250,254,256,258,260,262,264,266,268,270,272,274,276,278,280,282,286,$
                   288,290,292,294,296,298,300,302,304,306,308,310,312,314,318,320,322,324,326,328,330,332,334,336,$
                   338,340,342,344,346,350,352,354,356,358,360,362,364,366,370,372,374,376,378,382,384,386,388,390,$
                   392,394,396,398,400,402,404,406,408,410,414,416,418,420,422,424,426,428,430,432,434,436,438,440,$
                   442,446,450,452,454,456,458,460,462,464,466,468,470,472,474,478,480,482,484,486,488,490,492,494,$
                   498,500,502,504,506,510,512,514,516,518,520,522,524,526,528,530,532,534,536,538,542,544,546,548,$
                   550,552,554,556,558,560,562,564,566,568,570,574,576,578,580,582,584,586,588,590,592,594,596,598,$
                   600,602,606,608,610,612,614,616,618,620,622,626,628,630,632,634,638,640,642,644,646,648,650,652,$
                   654,656,658,660,662,664,666,670,672,674,676,678,680,682,684,686,688,690,692,694,696,698,702,704,$
                   706,708,710,712,714,716,718,720,722,724,726,728,730,734,736,738,740,742,744,746,748,750,754,756,$
                   758,760,762,766,768,770,772,774,776,778,780,782,784,786,788,790,792,794,798,800,802,804,806,808,$
                   810,812,814,816,818,820,822,824,826,830,832,834,836,838,840,842,844,846,848,850,852,854,856,858,$
                   862,864,866,868,870,872,874,876,878,882,884,886,888,890,894,896,898,900,902,904,906,908,910,912,$
                   914,916,918,920,922,926,928,930,932,934,936,938,940,942,944,946,948,950,952,954,958,962,964,966,$
                   968,970,972,974,976,978,980,982,984,986,990,992,994,996,998,1000,1002,1004,1006,1010,1012,1014]) $      
       else $  ;FCC structure, enough for 2005 meV for FCC Al 
          qsqrt = sqrt([3.,4,8,11,12,16,19,20,24,27,32,35,36,40,43,44,48,51,52,56,59,64,67,68,72,75,76,80,83,84,88,91,$
                   96,99,100,104,107,108,115,116,120,123,128,131,132,136,139,140,144,147,148,152,155,160,163,164,$
                   168,171,172,176,179,180,184,187,192,195,196,200,203,204,208,211,212,216,219,224,227,228,232,235,$
                   236,243,244,248,251,256,259,260,264,267,268,272,275,276,280,283,288,291,292,296,299,300,304,307,$
                   308,312,315,320,323,324,328,331,332,336,339,340,344,347,352,355,356,360,363,364,371,372,376,379,$
                   384,387,388,392,395,396,400,403,404,408,411,416,419,420,424,427,428,432,435,436,440,443,451,452,$
                   456,459,460,464,467,468,472,475,480,483,484,488,491,492,499,500,504,507,512,515,516,520,523,524,$
                   528,531,532,536,539,544,547,548,552,555,556,560,563,564,568,571,576,579,580,584,587,588,592,595,$
                   596,600,603,608,611,612,616,619,620,627,628,632,635,640,643,644,648,651,652,656,659,660,664,667,$
                   672,675,676,680,683,684,688,691,692,696,699,704,707,708,712,715,716,720,723,724,728,731,736,739,$
                   740,744,747,748,755,756,760,763,768,771,772,776,779,780,784,787,788,792,795,800,803,804,808,811,$
                   812,816,819,820,824,827,832,835,836,840,843,844,848,851,852,856,859,864,867,868,872,875,876,883,$
                   884,888,891,896,899,900,904,907,908,912,915,916,920,923,928,931,932,936,939,940,944,947,948,952,$
                   955,963,964,968,971,972,976,979,980,984,987,992,995,996,1000,1003,1004,1011,1012,1016,1019,1024,$
                   1027,1028,1032,1035,1036,1040,1043,1044,1048,1051,1056,1059,1060,1064,1067,1068,1072,1075,1076,$
                   1080,1083,1088,1091,1092,1096,1099,1100,1104,1107,1108,1112,1115,1120,1123,1124,1128,1131,1132,$
                   1139,1140,1144,1147,1152,1155,1156,1160,1163,1164,1168,1171,1172,1176,1179,1184,1187,1188,1192,$
                   1195,1196,1200,1203,1204,1208,1211,1216,1219,1220,1224,1227,1228,1232,1235,1236,1240,1243,1248,$
                   1251,1252,1256,1259,1260,1267,1268,1272,1275,1280,1283,1284,1288,1291,1292,1296,1299,1300,1304,$
                   1307,1312,1315,1316,1320,1323,1324,1328,1331,1332,1336,1339,1344,1347,1348,1352,1355,1356,1360,$
                   1363,1364,1368,1371,1376,1379,1380,1384,1387,1388,1395,1396,1400,1403,1408,1411,1412,1416,1419,$
                   1420,1424,1427,1428,1432,1435,1440,1443,1444,1448,1451,1452,1456,1459,1460,1464,1467,1475,1476,$
                   1480,1483,1484,1488,1491,1492,1496,1499,1504,1507,1508,1512,1515,1516,1523,1524,1528,1531,1536,$
                   1539,1540,1544,1547,1548,1552,1555,1556,1560,1563,1568,1571,1572,1576,1579,1580,1584,1587,1588,$
                   1592,1595,1600,1603,1604,1608])
    endif        
    case self.powd_typ of
          1: qvalue = qsqrt*2*!pi/latt_a     ;Al a=4.05    FCC
          2: qvalue = qsqrt*2*!pi/latt_a     ;Cu a=3.6148  FCC
          3: qvalue = qsqrt*2*!pi/latt_a     ;SS a=3.594   FCC, a=2.87 BCC
          4: begin
             info   = 'Numbers should be separated by a space or comma.'
             qvalue = dm_dialog_input('Q ('+string('c5'XB)+'^-1):',xsize=180,dialog_parent=self.tlb,info=info,default=default)
             if (~ finite(dm_to_number(qvalue))) then begin
                if (strlen(default) ne 0) or (strlen(qvalue) ne 0) then $
                   ok = dialog_message((['Invalid inputs.','Empty input.'])[strlen(qvalue) eq 0]+' No powder line is specified.',dialog_parent=self.tlb,/error,/center)
                self.powd_typ = 0
                self->my_menu_toggle,'powder'+['None','Al','Cu','St','Sp'],self.powd_typ
                dm_set_button,widget_info(self.tlb,find_by_uname='powdershowall'),self.powd_showall,onstring='Show Closed Powder Line',offstring='Show Actual Powder Line',sensitive=(self.powd_typ ne 0)
             endif else begin
                qvalue = dm_to_number(strsplit(qvalue,', '+string(9b),/extract))   
                qvalue = qvalue[sort(qvalue)]
             endelse
             end 
          else:
    endcase
    if n_elements(qvalue) ne 0 and self.powd_typ gt 0 then self.powd_qvalue=ptr_new(qvalue)
    if ptr_valid(self.projStrPtr) then self->set_display_button,/clearprojection ;need to recalculate projections
end

;keyword:
;   init: if not set, free some data pointers
pro dcs_mslice::change_samptyp,new_typ,init=init,pdt_reset=pdt_reset
    old_typ = self.samp_typ
    self.samp_typ = new_typ
    self->my_widget_control,['bgtolBut','empttolMenu'],sensitive=((self.samp_typ ge 2) or (self.instrname eq 'macs'))
    self->my_widget_control,'darkTraj',sensitive=((self.samp_typ ge 2) or ((self.instrname eq 'macs') and (self.samp_typ ne 0)))
    self->my_widget_control,'darkCorr',sensitive=((self.samp_typ ge 2) and (self.instrname eq 'dcs'))
    self->my_widget_control,'vfoldMenu',sensitive=(self.samp_typ ne 0)
    self->my_widget_control,'vaxisMenu',sensitive=((self.samp_typ ne 0) or (self.instrname eq 'macs'))
    self->my_widget_control,'bgtcBut',sensitive=((self.samp_typ ne 2) and (self.instrname ne 'wand') and (self.instrname ne 'macs'))
    if old_typ eq new_typ then return
    self->my_menu_toggle,['powdTyp','crystTyp1','crystTyp2','crystTyp3'],self.samp_typ,base=self.fileBas
    if (self.idl_version ge 5.6) then self->my_widget_control,'crystTyp',set_button=(self.samp_typ ge 1)  
    sptypes = ['Powder','Single Crystal','Diffuse Scattering Single E','Diffuse Scattering Multiple E']
    self->my_widget_control,'sampTyp',set_value=sptypes[self.samp_typ]
    if keyword_set(init) then return
    if (self.samp_typ ge 2) or (old_typ ge 2) then begin    ;if changed from/to diffuse scattering type
       ;clear data
       ptr_free,self.kfactor,self.dataStrPtr,self.powd_line
       self->my_widget_control,['addBut','clearDat'],sensitive=0          ;disallow adding data and clear data
       self->my_widget_control,'titlLab',set_value='' & self.titl_lab=''  ;clear title label
       self->my_widget_control,'legdLab',set_value='' & self.legd_lab=''  ;clear legend label
       ;clear background
       ptr_free,self.bgdata,self.bgdata2,self.bgratePtr,self.bgtchnPtr,self.bgdetsPtr
       self->my_widget_control,'bgclBut',sensitive=0
       if self.extraempcan_yn then self->my_widget_control,'emptBut2',sensitive=0
    endif
    if (self.instrname eq 'macs') and (self.samp_typ eq 1) and (self.macsintn_typ ne 0) then begin  ;switch to SPEC detector automatically in crystal mode
       self.macsintn_typ = 0
       self->my_menu_toggle,'macsint'+['SPEC','DIFF','SPDF'],self.macsintn_typ
       ok = dialog_message('Detectors have been switched to SPEC.',dialog_parent=self.tlb,/center,title='Please note:')  
    endif
    ptr_free,self.projStrPtr
    self->reset_file
    self->reset_pdt
    pdt_reset = 1b
end

;clear mask pointer and set the menu items accordingly
pro dcs_mslice::clear_mask,message=message,a3=a3
    if keyword_set(a3) then begin
       ptr_free,self.a3mask
       self->my_widget_control,'mkcla3But',sensitive=0
       if keyword_set(message) then ok = dialog_message('A3 mask has been cleared.',dialog_parent=self.tlb,/center)  
    endif else if ptr_valid(self.mask) then begin
       ptr_free,self.mask
       if self.instrname eq 'dcs' then begin
          self.dcsdetmask[*] = 0
          self->dm_mask_detbanks
       endif else $
          self->my_widget_control,'mkclBut',sensitive=0
       if keyword_set(message) then ok = dialog_message('Detector mask has been cleared.',dialog_parent=self.tlb,/center)  
       if self.instrname eq 'macs' then self.ftypecheckmask = 0b 
    endif
    self->my_widget_control,'mksvBut',sensitive=(ptr_valid(self.mask) or ptr_valid(self.a3mask))
    if ptr_valid(self.projStrPtr) then self->set_display_button,/clearprojection ;need to recalculate projections
end

; adds an object to the list and shows its title to the view menu or deletes an object from the list
; and removes the corresponding item in the view menu
; parameter:
;   opcode:   0: add plotwindow , this is the default operation
;             1: make plotwindow current, object keyword must be given
;             2: keep the plotwindow, object keyword must be given
;             3: delete plotwindow, object keyword must be given
;             4: close all plot windows
;             5: change title, object keyword must be given
; keyword:
;   type:     0:slice 1:cut 2:trajectory 3:volume
;   title:    string of the button name to be added or changed to the view menu
;   object:   opcode=1,2--object chosen; opcode=3--object to be deleted
pro dcs_mslice::dm_add_plotwindow,opcode,type=type,title=title,object=object
    if n_elements(opcode) eq 0 then opcode=0   ;default add plot window
    viewMenu = widget_info(self.tlb,find_by_uname='viewMenu')
    if ~widget_info(viewMenu,/valid_id) then return
    case opcode of
       0:begin
         if (~ obj_valid(self.current)) then return
         ;create the unique id for the view menu item
         id  = self->dm_get_id(self.current)
         ;check if the current object is already in the list, if not add it to the list
         all = self.gr_list->get(/all,count=count)
         in  = 0b
         for i=0,count-1 do begin
             if all[i] eq self.current then begin
                in = 1b
                break
             endif
         endfor
         if in eq 0 then begin    ;not in the list
            self.gr_list->add,self.current            ;add the object to the list
            id = dm_widget_button(viewMenu,value='',uname=id,separator=(count eq 0)) ;add the item to the view menu
         endif else begin     ;in the list, find the menu item id
            id = widget_info(viewMenu,find_by_uname=id)
         endelse
         ;generate proper string for the menu item
         if n_elements(type) eq 0 then type=0
         case type of
            0:   value = 'Slice: '
            1:   value = 'Cut: '
            2:   value = 'Trajectory: '
            3:   value = 'Volume: '
            else:value = 'Plot: '
         endcase
         if n_elements(title) ne 0 then value=value+strtrim(title,2)
         if widget_info(id,/valid_id) then begin
            if self.idl_version ge 5.6 then $
               widget_control,id,set_value=value,set_button=1 $
            else $
               widget_control,id,set_value=value
         endif
         self->my_widget_control,['killAll','arrPlot'],viewMenu,/sensitive
         if self.idl_version ge 6.1 then $
            self->my_widget_control,'viewCurr',viewMenu,/sensitive 
         end
       1:begin 
         if obj_valid(self.current) and (self.idl_version ge 5.6) then begin   ;remove the current check mark
            id = self->dm_get_id(self.current) ;locate the corresponding view menu item id
            self->my_widget_control,id,viewMenu,set_button=0
         endif
         all = self.gr_list->get(/all,count=count)
         for i=0,count-1 do begin
            if  all[i] ne object then all[i]->setproperty,keep=1 $
            else current=all[i]
         endfor
         if n_elements(current) ne 0 then self.current = current
         if self.idl_version ge 5.6 then begin ;add check mark to the current plot window
            id = self->dm_get_id(self.current)
            self->my_widget_control,id,viewMenu,set_button=1
         endif
         if self.idl_version ge 6.1 then $
            self->my_widget_control,'viewCurr',viewMenu,/sensitive 
         end
       2:begin          
         self.current = obj_new()
         if self.idl_version ge 5.6 then begin ;remove check mark from the view menu
            id = self->dm_get_id(object)  ;locate the corresponding view menu item id
            self->my_widget_control,id,viewMenu,set_button=0
         endif
         if self.idl_version ge 6.1 then $
            self->my_widget_control,'viewCurr',viewMenu,sensitive=0 
         end
       3:begin       
         all = self.gr_list->get(/all,count=count)
         if count eq 0 then return
         for i=0,count-1 do begin   ;destroy all the plot menu items
             id = self->dm_get_id(all[i])
             self->my_widget_control,id,viewMenu,get_value=value,/destroy  ;remove the item in the view menu
             if n_elements(info) eq 0 then info = [id,value] $  ;save the names
             else info = [[info],[[id,value]]]
         endfor      
         self.gr_list->remove,object       ;remove the object from the list
         all = self.gr_list->get(/all,count=count)
         for i=0,count-1 do begin   ;reconstruct the plot menu items
             id  = self->dm_get_id(all[i])
             all[i]->getproperty,keep=keep,title=title
             value = title ;change the menu name to the title of the plot
             id1 = where(info[0,*] eq id,existed)
             if existed ne 0 then begin
                tmp = info[1,id1[0]]
                tmp = strsplit(tmp,':',/extract,/preserve_null)
                if n_elements(tmp) ne 0 then value=tmp[0]+': '+title ;include the type string
             endif
             id = dm_widget_button(viewMenu,uname=id,value=value,separator=(i eq 0),set_button=(1-keep))
         endfor
         self->my_widget_control,['killAll','arrPlot'],viewMenu,sensitive=(count gt 0)
         if self.idl_version ge 6.1 then $
            self->my_widget_control,'viewCurr',viewMenu,sensitive=(self.current ne object)
         end
       4:begin
         all = self.gr_list->get(/all,count=count)
         self.gr_list->remove,/all
         if count gt 0 then begin
            obj_destroy,all
             ;destroy all the plot menu items
            for i=0,count-1 do begin  
                id = self->dm_get_id(all[i])
                self->my_widget_control,id,viewMenu,get_value=value,/destroy  ;remove the item in the view menu
            endfor
            self->my_widget_control,['killAll','arrPlot'],viewMenu,sensitive=0
            if self.idl_version ge 6.1 then $
               self->my_widget_control,'viewCurr',viewMenu,sensitive=0 
         endif
         end
       5:begin          
         if n_elements(object) ne 0 and n_elements(title) ne 0 then begin
            id = self->dm_get_id(object)  ;locate the corresponding view menu item id
            self->my_widget_control,id,viewMenu,get_value=oldtitle
            oldtitle = (strsplit(oldtitle,':',/extract))[0]
            if strlen(oldtitle) ne 0 then oldtitle = oldtitle+': '
            self->my_widget_control,id,viewMenu,set_value=oldtitle+strtrim(title,2)
         endif
         end
       else:
    endcase
end

;arrange the plot windows
pro dcs_mslice::dm_arrange_plot,oplots,cascade=cascade
    count = n_elements(oplots)
    if count eq 0 then oplots = self.gr_list->get(/all,count=count)
    if count eq 0 then return
    for i=0,count-1 do oplots[i]->totop
    if count eq 1 then return
    if self.idl_version ge 5.6 then scr_size = get_screen_size() else device,get_screen_size=scr_size
    if keyword_set(cascade) then begin
       oplots[count-1]->getproperty,xsize=xsize,ysize=ysize & ysize=ysize+20
       xstep = (40<((scr_size[0]-xsize)/count))
       ystep = (40<((scr_size[1]-ysize)/count))
       xoff  = 5>((scr_size[0]-xstep*count-xsize)/2)
       yoff  = 5>((scr_size[1]-ystep*count-ysize)/2)
       for i=0,count-1 do begin
           oplots[i]->getproperty,tlb=tlb
           widget_control,tlb,xoffset=((xoff+xstep*i)<(scr_size[0]-10)),yoffset=((yoff+ystep*i)<(scr_size[1]-10))
       endfor
    endif else begin
       if count le 4 then ncol=2 $
       else if count le 9 then ncol=3 $
       else ncol=4
       xsize = intarr(count) & ysize = intarr(count) & wid = lonarr(count)
       for i=0,count-1 do begin
           oplots[i]->getproperty,tlb=tlb
           tmp = widget_info(tlb,/geometry)
           wid[i] = tlb & xsize[i] = tmp.scr_xsize+2 & ysize[i] = tmp.scr_ysize+2
       endfor
       nrow = (count+ncol-1)/ncol
       roww = intarr(nrow) & rowh = intarr(nrow) & xoff = intarr(nrow)
       for i=0,nrow-1 do begin
           roww[i] = total(xsize[ncol*i:((ncol*(i+1)-1)<(count-1))])
           rowh[i] = max(ysize[ncol*i:((ncol*(i+1)-1)<(count-1))])
           xoff[i] = 0>((scr_size[0]-roww[i])/2)
       endfor
       yoff = 0>((scr_size[1]-total(rowh))/2)
       for i=0,count-1 do begin
           row = i/ncol
           col = (i mod ncol)
           if xoff[row] eq 0 then xoffset=scr_size[0]/ncol*col $
           else begin
              xoffset = xoff[row]
              if col ne 0 then xoffset = (xoffset+total(xsize[row*ncol:row*ncol+col-1]))<(scr_size[0]-20)
           endelse
           if yoff eq 0 then yoffset=scr_size[1]/nrow*row $
           else begin
              yoffset = yoff
              if row ne 0 then yoffset = (yoffset+total(rowh[0:row-1]))<(scr_size[1]-20)
           endelse
           widget_control,wid[i],xoffset=xoffset,yoffset=yoffset
       endfor
    endelse
end

;create a unique id for an object
function dcs_mslice::dm_get_id,object
    help,object,output=id & id=strsplit(id,' ',/extract) &  id=id[n_elements(id)-1]
    tmp = strsplit(id,'()',/extract) & if n_elements(tmp) eq 3 then id=tmp[0]+tmp[2]
    return,id
end

;object event handler
pro dcs_mslice::event,event
    compile_opt IDL2    ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       errmesg = !error_state.msg 
       if obj_valid(self) then begin
          if (self.samp_typ eq 3) and (!error_state.name eq 'IDL_M_CNTGETMEM') then $
             errmesg = [errmesg,'','Please reduce the energy range, or use single E option.']
          if (lmgr(/runtime) eq 0) and (!version.os_family eq 'Windows') then begin
             if strpos(errmesg[0],'rebinnd') ne -1 then begin
                errmesg = 'Please copy msvcr100'+(['','_64'])[!VERSION.MEMORY_BITS eq 64]+'.dll from \dave\programs\auxiliary\rebin_nd\ to your Windows system directory.'
                errmesg = [errmesg,'','Or you can disable the external binning library in the menu Option->Binning Method->Use External Binning Library->No.']
             endif
          endif  
       endif
       ok = dialog_message(errmesg,/error,dialog_parent=self.tlb,/center)
       if obj_valid(self) then self.error = 1b
       return
    endif

    widget_type = widget_info(event.id,/type)
    case widget_type of
       1:tmp_value = event.select                               ;button
       3:begin                                                  ;text box
         widget_control,event.id,get_value=tmp_value
         tmp_value = strtrim(tmp_value,2)
         end
       8:tmp_value = widget_info(event.id,/droplist_select)     ;drop list
       12:begin                                                 ;combobox
         tmp_value = event.index
         widget_control,event.id,get_value=tmp_list
         if tmp_list[tmp_value] ne event.str then begin         ;to avoid events from set_value in unix operating system
            return
         endif
         end
       else:
    endcase
    if n_elements(tmp_value) eq 1 then tmp_value=tmp_value[0]

    resetu1 = 0b & resetu2 = 0b & resetu3 = 0b & recalc_proj = 0b

    uname = widget_info(event.id,/uname)
    type  = tag_names(event,/structure)
    if self.samp_typ eq -1 then nitem = n_elements((*self.dataPtr).label)
    case strlowcase(type) of
        'dm_plot_make_current': uname = type
        'dm_plot_keep':         uname = type
        'dm_plot_kill':         uname = type
        'dm_plot_changetitle':  uname = type
        'widget_context':       uname = 'status'
        'widget_kbrd_focus':    if widget_info(event.id,/name) eq 'TEXT' then uname = 'text_focus'
        else:
    endcase
    tmp = strmid(strlowcase(uname),0,8)
    ind_sel = dm_to_number(strmid(strlowcase(uname),0,1,/reverse),/int)
    case tmp of
        'cutsel_t': uname = 'cutsel_t'
        'cutselfr': uname = 'cutsel_from'
        'cutselto': uname = 'cutsel_to'
        'dispsel_': uname = 'dispsel_t'
        'dispself': uname = 'dispsel_from'
        'dispselt': uname = 'dispsel_to'
        'volsel_t': uname = 'volsel_t'
        'volselfr': uname = 'volsel_from'
        'volselto': uname = 'volsel_to'
        'pdchoice': uname = 'pdim_choice'
        else:
    endcase
    if strmid(strlowcase(uname),0,4) eq 'user' then begin
       if !version.os_family eq 'Windows' then begin
          sep  = string(9b)+': ' 
          tail = '             '
       endif else begin 
          sep  = ': '
          tail = ''
       endelse
       if self.instrname eq 'macs' then begin
          if self.extravaxis_yn and ptr_valid(self.dataStrPtr) then $
             tmp_info = 'en, Ei, Ef, psi, kidney, data_temperature, det_tt.' $
          else $
             tmp_info = 'en, Ei, Ef, psi, kidney, det_tt.'
       endif else begin
          if (self.instrname eq 'wand') and self.extravaxis_yn and ptr_valid(self.dataStrPtr) then $
             tmp_info = 'eief, en, psi, data_temperature, det_tt.' $
          else $
             tmp_info = 'eief, en, '+(['','psi, '])[self.samp_typ gt 0]+'det_tt.'
       endelse
       userf_info = 'Available variables after projection:'
       if self.samp_typ eq 0 then begin ;powder
          userf_info = userf_info+' qty, err, Q, '+tmp_info
          userf_info = [userf_info,'Available variables before projection: qty, err, '+tmp_info]
          if ptr_valid(self.dataStrPtr) then begin
             help,(*self.dataStrPtr).qty,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ') 
             userf_info = [userf_info,'qty,err'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]        
             if self.instrname eq 'macs' then begin
                help,(*self.dataStrPtr).Ei,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                userf_info = [userf_info,'en,Ei,Ef,psi,kidney'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
                if self.extravaxis_yn then begin
                   help,(*self.dataStrPtr).temperature,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                   userf_info = [userf_info,'data_temperature'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail] 
                endif
             endif else begin
                help,self.eief,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                userf_info = [userf_info,'eief'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
                help,(*self.dataStrPtr).e,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                userf_info = [userf_info,'en'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             endelse
          endif else if ptr_valid(self.projStrPtr) then begin
             if self.instrname ne 'macs' then begin
             help,self.eief,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
             userf_info = [userf_info,'eief'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             endif
             help,(*self.projStrPtr).en,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
             if self.instrname eq 'macs' then begin
             userf_info = [userf_info,'en,Ei,Ef,psi,kidney'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail] 
             endif else begin
             userf_info = [userf_info,'en'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             endelse
          endif
          if ptr_valid(self.projStrPtr) then begin
             help,(*self.projStrPtr).Q,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
             userf_info = [userf_info,'Q'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail] 
          endif
          if ptr_valid(self.detPosPtr) then begin 
             help,(*self.detPosPtr).two_theta,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ') 
             userf_info = [userf_info,'det_tt'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
          endif
       endif else begin                 ;single crystal
          userf_info = userf_info+' qty, err, Qv, '+tmp_info
          userf_info = [userf_info,'Available variables before projection: qty, err, '+tmp_info]
          if ptr_valid(self.dataStrPtr) then begin
             help,(*self.dataStrPtr).qty,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ') 
             userf_info = [userf_info,'qty,err'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             if self.instrname eq 'macs' then begin
                help,(*self.dataStrPtr).Ei,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                userf_info = [userf_info,'en,Ei,Ef,psi,kidney'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
                if self.extravaxis_yn then begin
                   help,(*self.dataStrPtr).temperature,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                   userf_info = [userf_info,'data_temperature'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
                endif    
             endif else begin
                help,self.eief,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                userf_info = [userf_info,'eief'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
                help,(*self.dataStrPtr).e,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                userf_info = [userf_info,'en'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
                help,(*self.dataStrPtr).rotation,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
                if (self.instrname eq 'wand') and self.extravaxis_yn then $
                   userf_info = [userf_info,'psi,data_temperature'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail] $
                else $
                   userf_info = [userf_info,'psi'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             endelse
          endif else if ptr_valid(self.projStrPtr) then begin
             if self.instrname ne 'macs' then begin
             help,self.eief,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
             userf_info = [userf_info,'eief'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             endif
             help,(*self.projStrPtr).en,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
             if self.instrname eq 'macs' then begin
             userf_info = [userf_info,'en,Ei,Ef,psi,kidney'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail] 
             endif else begin
             userf_info = [userf_info,'en'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             help,reform((*self.projStrPtr).V[0,0,0,*]),output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ')
             userf_info = [userf_info,'psi'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
             endelse
          endif
          if ptr_valid(self.projStrPtr) then begin
             help,(*self.projStrPtr).V,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ') & pos1 = strpos(strlowcase(tmp),'array[')
             userf_info = [userf_info,'Qv'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
          endif
          if ptr_valid(self.detPosPtr) then begin 
             help,(*self.detPosPtr).two_theta,output=tmp & tmp = strtrim(tmp[0],2) & pos = strpos(tmp,' ') 
             userf_info = [userf_info,'det_tt'+sep+strmid(tmp,pos+1,strlen(tmp)-1-pos)+tail]
          endif
       endelse
       if (self.instrname eq 'macs') and (self.macsintn_typ gt 0) then userf_info = [userf_info,'Ef = Ei when detector choice is "'+(['DIFF','SPEC+DIFF'])[self.macsintn_typ-1]+'".'] 
    endif
    if uname ne 'text_focus' then WIDGET_CONTROL,/HOURGLASS
    case strlowcase(uname) of
        'dm_plot_make_current': self->dm_add_plotwindow,1,object=event.object   ;this event is sent from dm_plot window
        'dm_plot_keep':         self->dm_add_plotwindow,2,object=event.object   ;this event is sent from dm_plot window
        'dm_plot_kill':         self->dm_add_plotwindow,3,object=event.object   ;this event is sent from dm_plot window
        'dm_plot_changetitle':  self->dm_add_plotwindow,5,object=event.object,title=event.title ;this event is sent from dm_plot_window
        'text_focus': begin
            if event.enter then begin
               if !version.os_family eq 'Windows' then begin ;only for Windows OP
                  widget_control,event.id,get_value=tmp
                  widget_control,event.id,set_text_select=[0,strlen(tmp[0])] 
               endif
            endif else $
               widget_control,event.id,set_text_select=[0,0]
          end
        'tlb':begin
            if self.idl_version ge 5.5 then $
               widget_control,self.tlb,xsize=self.geom[0],ysize=self.geom[1] $
            else $
               widget_control,self.tlb,scr_xsize=self.geom[0],scr_ysize=self.geom[1]
          end
        'savevolumetxt':      self->save_volume
        'savevolxslicespe':   self->save_volume,slice=0,/spe
        'savevolxsliceiexy':  self->save_volume,slice=0,/iexy
        'savevolxslicexyie':  self->save_volume,slice=0,/xyie
        'savevolyslicespe':   self->save_volume,slice=1,/spe
        'savevolysliceiexy':  self->save_volume,slice=1,/iexy
        'savevolyslicexyie':  self->save_volume,slice=1,/xyie
        'savevolzslicespe':   self->save_volume,slice=2,/spe
        'savevolzsliceiexy':  self->save_volume,slice=2,/iexy
        'savevolzslicexyie':  self->save_volume,slice=2,/xyie
        'saveslicedave':      self->save_slice,/dave
        'saveslicespe':       self->save_slice,/spe
        'savesliceiexy':      self->save_slice,/iexy
        'saveslicexyie':      self->save_slice,/xyie
        'sendslicedave': begin
            self->save_slice,dPtr=dPtr
            (self.DAVETool)->AddDavePtrToDataManager, dPtr, 'slice from mslice'  
            heap_free,dPtr  
          end              
        'savecutxye':    self->save_cut,/xye
        'savecutdave':   self->save_cut,/dave
        'sendcutdave': begin
            self->save_cut,dPtr=dPtr    
            (self.DAVETool)->AddDavePtrToDataManager, dPtr, 'cut from mslice'
            heap_free,dPtr
          end              
        'saveprojtxt':   self->save_projection,/txt
        'runscript': begin
            path = self.dirs[1]
            scriptfile = dm_choose_file('*',dialog_parent=self.tlb,/read,path=path,title='Please choose a script file')
            if strlen(scriptfile) eq 0 then return
            self->set_dir,workDir=path
            WIDGET_CONTROL,/HOURGLASS
            tmp = ''
            script = ''
            openr,unit,scriptfile,/get_lun
            while(~ eof(unit)) do begin
                readf,unit,tmp
                tmp = strtrim(tmp,2)
                ;remove comment starting with ';'
                if strmid(tmp,0,1) eq ';' then tmp = '' else begin
                   if strpos(tmp,';') gt 0 then begin
                      flag = 0
                      for i=0,strlen(tmp)-1 do begin
                          letter = strmid(tmp,i,1)
                          if (letter eq ';') and (flag eq 0) then begin
                             tmp = strmid(tmp,0,i)
                             break
                          endif
                          if letter eq '"' then flag = 2-flag
                          if (letter eq "'") and (flag ne 2)  then flag = 1-flag
                      endfor
                   endif
                endelse
                if (strlen(tmp) ne 0) then  script=script+'&'+tmp
            endwhile
            free_lun,unit
            if strlen(script) ne 0 then begin
               script = strmid(script,1,strlen(script)-1)     ;remove the first '&' character
               while((index=strpos(script,'$&')) ge 0) do strput,script,'  ',index ;remove possible '$&'
               ok = execute(script,1,1)
               if ~ok then ok = dialog_message(!error_state.msg,/error,dialog_parent=self.tlb,/center)
            endif
          end
        'hist_0':  self->set_filehistory,choose=0
        'hist_1':  self->set_filehistory,choose=1
        'hist_2':  self->set_filehistory,choose=2
        'hist_3':  self->set_filehistory,choose=3
        'hist_4':  self->set_filehistory,choose=4
        'hist_5':  self->set_filehistory,choose=5
        'hist_6':  self->set_filehistory,choose=6
        'hist_7':  self->set_filehistory,choose=7
        'hist_8':  self->set_filehistory,choose=8
        'hist_9':  self->set_filehistory,choose=9
        'clearhist': begin
            ptr_free,self.file_hist
            self.n_history = 0
            self->set_filehistory
           end
        'exitbut':begin
            widget_control,self.tlb,/destroy
            return
          end
        'ldpmbut':begin  ;load parameter file
            path = self.dirs[1]
            parmfile = dm_choose_file('prm',dialog_parent=self.tlb,/read,path=path)
            if strlen(parmfile) ne 0 then self->dm_load_parameter,parmfile
            self->set_dir,workDir=path
          end
        'svpmbut':begin ;save parameters into a file
            path = self.dirs[1]
            parmfile = dm_choose_file('prm',dialog_parent=self.tlb,/write,path=path)
            if strlen(parmfile) ne 0 then self->dm_save_parameter,parmfile
            self->set_dir,workDir=path
          end
        'knpmbut':begin  ;change smoothing kernel
            old = dm_to_string(float(self.slice_sm_kernel))
            old = '[['+old[0]+' ,'+old[1]+' ,'+old[2]+'],['+old[3]+' ,'+old[4]+' ,'+old[5]+'],['+old[6]+' ,'+old[7]+' ,'+old[8]+']]'
            new = dm_dialog_input('kernel matrix:',title='Change smoothing kernel',default=old,xsize=250,dialog_parent=self.tlb,info='The smoothing kernel must be a 3x3 matrix.',cancel=cancel)
            if keyword_set(cancel) then return
            new = dm_to_number(strsplit(new,'[], '+string(9b),/extract),/float)
            if n_elements(new) ne 9 then begin
               ok = dialog_message('Please input a 3x3 matrix.',/error,dialog_parent=self.tlb,/center)
               break
            endif
            if max(abs(new[indgen(3,3)]-new[transpose(indgen(3,3))])) ne 0.0 then begin
               ok = dialog_message('Please input a symmetric matrix.',/error,dialog_parent=self.tlb,/center)
               break
            endif
            if total(new[0:8]) ne 0 then self.slice_sm_kernel=new[indgen(3,3)]
          end
        'bnthbut':begin  ;change binning threshold
            old   = self.binthreshold
            info  = ['Bins with fewer number of data points than (filling factor)*(NMAX)','will be discarded, where NMAX is the maximum number of points in a bin.']
            if self.samp_typ ne -1 then info=[info,'This factor will be reset to 0 every time a file is loaded.']
            repeat begin
              new = dm_dialog_input('filling factor:',title='Change binning threshold',dialog_parent=self.tlb,default=old,info=info,/float,cancel=cancel)
              if keyword_set(cancel) then return
              if finite(new) then begin
                 if (new ge 0) and (new lt 1) then begin
                     ok2 = 1b
                 endif else begin
                     old  = new
                     info = '0.0<= filling factor <1.0'
                 endelse
              endif
            endrep until keyword_set(ok2)
            self.binthreshold = new           
          end
        'ssfcbut':begin  ;change self-shielding factor
            old = self.ssfac
            new = dm_dialog_input('self-shielding factor:',title='Change self-shielding factor',default=old,/float,info='I=I_samp-ssfac*I_can',dialog_parent=self.tlb,cancel=cancel)
            if keyword_set(cancel) then return
            if finite(new) then begin
               if new ne self.ssfac then begin
                  self.ssfac = new
                  if ptr_valid(self.projStrPtr) and ptr_valid(self.bgdata) then recalc_proj = 1b    ;need to recalculate projections
               endif
            endif else $
               ok = dialog_message('Invalid input.',/error,dialog_parent=self.tlb,/center)
          end
        'detoffsetbut':begin
            case self.instrname of 
                 'wand':  old = -((*self.detPosPtr).two_theta)[0]
                 else:    old = ((*self.detPosPtr).two_theta)[0]
            endcase
            new = dm_dialog_input('Offset Angle:',title='Change detector offset angle',default=old,/float,dialog_parent=self.tlb,cancel=cancel)
            if keyword_set(cancel) then return 
            if finite(new) then begin
               case self.instrname of 
                    'wand':   (*self.detPosPtr).two_theta = (*self.detPosPtr).two_theta+old-new 
                    else:     (*self.detPosPtr).two_theta = (*self.detPosPtr).two_theta+new-old
               endcase
            endif else $
               ok = dialog_message('Invalid input.',/error,dialog_parent=self.tlb,/center)  
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'macsdeteffbut':begin
            if ~ptr_valid(self.eff) then begin
               widget_control,event.id,sensitive=0
               return
            endif
            new = dm_dialog_listinput('det '+dm_to_string(1+indgen(n_elements((*self.detPosPtr).two_theta))),title='Please input new efficiencies:',default=(*self.eff),$
                 label=['SPEC','DIFF'],info=['Intensity will be multiplied by these values.'],dialog_parent=self.tlb)
            if total(finite(new,/nan)) gt 0 then return
            if total(abs((*self.eff)-new)) eq 0 then return
            (*self.eff) = new     
            if ~ptr_valid(self.eff1) then self.ftoolflag[1] = 1b
          end 
        'bgtolbut':begin
            if self.instrname eq 'macs' then begin
               name = (['energy tolerance (meV)',['kidney','a3']+' tolerance ('+string('b0'xb)+')'])[0:(1+self.samp_typ)]
               name = [name,'matching order']+':' & n_name = n_elements(name)
               default = [self.macs_tolerance[0:(1+self.samp_typ)],self.macs_bgefirst]
               is_droplist = intarr(n_name) & is_droplist[n_name-1] = 1
               new = dm_dialog_input(name,/float,default=default,dialog_parent=self.tlb,title='Tolerance value',cancel=cancel,is_droplist=is_droplist,/return_number,xsize=80,$
                     droplist_content=ptr_new(['energy last','energy first']),info='The tolerance values are used in choosing corresponding empty can data for subtraction.')
            endif else begin
               default = self.psi_tolerance
               new = dm_dialog_input(['psi tolerance('+string('b0'xb)+')'],/float,default=default,dialog_parent=self.tlb,title='Tolerance value',cancel=cancel,$
                     info='The tolerance value is used in choosing corresponding empty can data for subtraction.')
            endelse
            if keyword_set(cancel) then return 
            if total(finite(new,/nan)) gt 0 then begin
               ok = dialog_message('Invalid input.',/error,dialog_parent=self.tlb,/center)
               return
            endif
            if total(abs(new-default)) eq 0 then return ;no change
            if self.instrname eq 'macs' then begin
               self.macs_tolerance[0:(1+self.samp_typ)] = abs(new[0:n_name-2])
               self.macs_bgefirst = new[n_name-1]
            endif else $
               self.psi_tolerance = abs(new)
            if ptr_valid(self.dataStrPtr) and ptr_valid(self.bgdata) and ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections   
          end 
        'wandeiefbut':begin
            old = sqrt(81.8042/self.eief)
            new = dm_dialog_input('wavelength ('+string('c5'XB)+'):',/float,default=finite(old)?round(old*1000.0,/L64)/1000.0:old,dialog_parent=self.tlb,/lambda2e,title='Change wavelength',cancel=cancel)
            if keyword_set(cancel) then return 
            if finite(new) then begin
               self.eief = 81.8042/(new^2)
               if ptr_valid(self.dataStrPtr) then (*self.dataStrPtr).eief = self.eief
            endif else $
               ok = dialog_message('Invalid input.',/error,dialog_parent=self.tlb,/center)  
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end  
        'emfcbut':begin  ;change 2-set empty can factors
            repeat begin
                tmp = dm_dialog_input(['empty can 1 factor:','empty can 2 factor:'],/float,default=self.empcanfactor,$
                      info=['I=I_samp-ssfac*(factor1*I_can1+factor2*I_can2)','ssfac: self-shielding factor'],dialog_parent=self.tlb,cancel=cancel)
                if keyword_set(cancel) then return
                if total(finite(tmp)) eq 2 then begin
                   self.empcanfactor = tmp
                   ok2 = 1b
                endif
            endrep until keyword_set(ok2)
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end  
        'ddirbut':begin    ;set data directory
            old = self.dirs[0]
            new = dialog_pickfile(/directory,path=old,dialog_parent=self.tlb)
            if (strlen(new) ne 0) then begin
               self.fileSel->set_path,new
               self->set_dir,dataDir=new
            endif
          end
        'wdirbut':begin  ;set working directory
            old = self.dirs[1]
            new = dialog_pickfile(/directory,path=old,dialog_parent=self.tlb)
            if (strlen(new) ne 0) then self->set_dir,workDir=new
          end
        'bgovbut':begin  ;overall background rate
            value = !values.f_nan & nvalue = 1
            if ptr_valid(self.bgratePtr) then begin
               value  = mean((*self.bgratePtr)[*,0])
               nvalue = n_elements(*self.bgratePtr)   ;dark count file existed
            endif
            tmp_value = dm_dialog_input(dialog_parent=self.tlb,'background rate:',info='in (cts/hr/det)',/float,default=value,cancel=cancel)
            if keyword_set(cancel) then return
            if finite(tmp_value) then begin
               if tmp_value ge 0 then begin
                  if nvalue gt 1 then begin
                     ok = dialog_message('Do you want to replace the existing dark count file data?',/question,/default_no,dialog_parent=self.tlb,/center)
                     if strupcase(ok) eq 'NO' then break
                  endif
                  ptr_free,self.bgratePtr
                  self.bgratePtr = ptr_new(tmp_value)
                  self->my_widget_control,'bgclBut',/sensitive
                  if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
               endif else $
                  ok = dialog_message('Negative number is entered. The overall background rate has not been changed.',dialog_parent=self.tlb,/center)
            endif else $
               ok = dialog_message('Invalid input. The overall background rate has not been changed.',dialog_parent=self.tlb,/center)
          end
        'bgclbut':begin
            ptr_free,self.bgdata,self.bgdata2,self.bgratePtr,self.bgtchnPtr,self.bgdetsPtr
            widget_control,event.id,sensitive=0
            if self.extraempcan_yn then self->my_widget_control,['emfcBut','emptBut2'],sensitive=0
            if (self.idl_version ge 5.6) and (strlowcase(!version.os_family) eq 'windows') then self->my_widget_control,'clearDat',tooltip=''
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'vdclbut':begin
            widget_control,event.id,sensitive=0
            if self.instrname eq 'macs' then begin
               (*self.eff) = (*self.eff1)
               ptr_free,self.eff1 
               self.ftoolflag[1] = 0b
            endif else $
               ptr_free,self.eff
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'maskfiller':begin   
            self.mask_filler = dm_dialog_input('mask value:',/float,default=self.mask_filler,$
                 info=['The data with intensity less than or equal to this value will be masked.','Clear the value to disable it.'],dialog_parent=self.tlb,cancel=cancel)
            if keyword_set(cancel) then return
            if finite(self.mask_filler) then begin
               if ptr_valid(self.dataStrPtr) then $
                  index = where((*self.dataStrPtr).data[*,0] gt self.mask_filler,count) $
               else $
                  index = where((*self.dataPtr).data[*,0] gt self.mask_filler,count)
               if count ne 0 then begin
                  if (~ ptr_valid(self.dataStrPtr)) then self.dataStrPtr = ptr_new((*self.dataPtr))
                  (*self.dataPtr) = {data:(*self.dataStrPtr).data[index,*],label:(*self.dataStrPtr).label,unit:(*self.dataStrPtr).unit,is_uniq:(*self.dataStrPtr).is_uniq,instrument:(*self.dataStrPtr).instrument}
               endif else begin
                  ok = dialog_message('All data have been masked by the given value. Mask is disabled.',dialog_parent=self.tlb,/center)
                  self.mask_filler = !values.f_nan
               endelse
            endif 
            if (~ finite(self.mask_filler)) and ptr_valid(self.dataStrPtr) then begin
               (*self.dataPtr) = (*self.dataStrPtr)
               ptr_free,self.dataStrPtr
            endif
            widget_control,event.id,set_button=finite(self.mask_filler)   
          end
        'mkngbut': self->dm_mask_detbanks, 'negative'   ;mask negative angle detectors
        'mkulbut': self->dm_mask_detbanks, 'upperlower' ;mask upper and lower banks
        'mkcnbut': self->dm_mask_detbanks, 'center'     ;mask central bank
        'mklobut': self->dm_mask_detbanks, 'lower'      ;mask lower bank
        'mkupbut': self->dm_mask_detbanks, 'upper'      ;mask upper bank
        'mkstdnum':begin ;set mask by detector number
            if ptr_valid(self.detPosPtr) then ndet=n_elements((*self.detPosPtr).two_theta) else ndet=0 
            default = ''
            ;if the number of exisiting masked detectors is less than 20, save it to the default
            ;if the existing mask is of incompatible type, clear the mask
            if ptr_valid(self.mask) then begin
               exist = 1b
               if (self.masktype eq 0) or ((self.masktype eq 1) and (self.instrname ne 'macs')) then begin
                  n_mask = n_elements(*self.mask)
                  maxmsk = max(*self.mask,min=minmsk)
                  if (n_mask eq (maxmsk-minmsk+1)) and (n_mask gt 3) then $
                      default = dm_to_string(minmsk)+'-'+dm_to_string(maxmsk) $
                  else if n_mask le 20 then $
                      default = dm_to_string(*self.mask,separator=',')
               endif else self->clear_mask    ;clear the mask
            endif
            info = 'Detector number starts from 0'+(['',' to '+dm_to_string(ndet-1)+' for '+(*self.ftypename)[self.ftype]])[ndet gt 0]+'.'
            case self.instrname of 
               'dcs':   info = [info,'eg: 370,373 or 370-374c']
               'wand':  info = [info,'eg: 0-8,633-639']
               'macs':  info = [info,'eg: 0-3,6,10']
               else:    info = [info,'eg: 20-28,30,32']
            endcase
            tmp = dm_dialog_input(dialog_parent=self.tlb,'Det num:',title='Masking detectors',default=default,info=info,xsize=200)
            if strlen(tmp) eq 0 then begin
               if keyword_set(exist) then begin
                  self->clear_mask
                  ok = dialog_message(['No detector is specified.','Previously set detector mask has been cleared.'],dialog_parent=self.tlb,/center)
               endif
               break
            endif else begin
               ;search for :- range detectors
               tmpmask = -1L
               repeat begin
                   tmp1 = stregex(tmp,'([0-9]+)[ '+string(9b)+']*[:-][ '+string(9b)+']*([0-9]+)[ '+string(9b)+']*([cCuUlL]*)',/subexpr,length=len)
                   if len[0] gt 0 then begin
                      tmp2 = dm_to_number(strmid(tmp,tmp1[1],len[1]),/long)
                      tmp3 = dm_to_number(strmid(tmp,tmp1[2],len[2]),/long)
                      if len[3] gt 0 then status = strlowcase(strmid(tmp,tmp1[3],1)) else status=''
                      tmp4 = min([tmp2,tmp3])+lindgen(abs(tmp3-tmp2)+1)
                      if (self.instrname eq 'dcs') and (status ne '') then begin
                         tmp4 = dm_common(tmp4,dm_mask_detbanks((where(['l','c','u'] eq status))[0]-1),/no_copy)
                      endif
                      tmpmask = [tmpmask,tmp4]
                      tmp = strmid(tmp,0,tmp1[0])+strmid(tmp,tmp1[0]+len[0],strlen(tmp)-tmp1[0]-len[0])
                   endif
               endrep until len[0] le 0 
               ;individual detectors
               tmp1 = strsplit(tmp,'[a-zA-Z '+string(9b)+',:-]',/regex,/extract)
               tmp2 = dm_to_number(tmp1)
               ind2 = where(finite(tmp2),count2)
               if count2 gt 0 then tmpmask = [tmpmask,dm_to_number(tmp1[ind2],/long)]
               index = where(tmpmask ge 0,count)
               if count gt 0 then mask = tmpmask[index]
            endelse
            if n_elements(mask) gt 0 then begin
               mask = mask[uniq(mask,sort(mask))]
               if ndet gt 0 then ind  = where((mask lt ndet) and (mask ge 0),count) $
               else ind  = where(mask ge 0,count)
               if count ne 0 then begin
                  if ptr_valid(self.mask) then begin
                     if (n_elements(*self.mask) eq count) and (total(abs((*self.mask)-mask[ind])) eq 0) then break
                  endif
                  self->clear_mask
                  self.mask = ptr_new(mask[ind])
                  self->my_widget_control,['mksvBut','mkclBut'],/sensitive
               endif 
            endif
            self.masktype = 0b
          end
        'mkstangl':begin ;set mask by detector angle
            default = ''
            ;clear the incompatible mask, save the existing mask to the default
            if ptr_valid(self.mask) then begin
               exist = 1b
               if self.masktype eq 2 then default = (*self.mask) $
               else self->clear_mask    ;clear the mask
            endif
            if ptr_valid(self.detPosPtr) then det_tt = (*self.detPosPtr).two_theta
            info = 'Use ":" to denote range, "-" only as the negative sign.'
            case self.instrname of 
               'dcs':   info = [info,'eg: <0, 10:20, >120']
               'wand':  info = [info,'eg: <-100, -60:-50,>-20']
               'macs':  begin
                        det_tt = det_tt-76.0  ;kidney=0.0
                        if ptr_valid(self.dataStrPtr) then begin
                           min_kid = min((*self.dataStrPtr).kidney,max=max_kid)
                           det_tt = [det_tt+min_kid,det_tt+max_kid]
                        endif
                        info = [info,'eg: <-10, 20:30,>100']
                        end
               else:    info = [info,'eg: <20, 20:28,>100']
            endcase
            if n_elements(det_tt) ne 0 then begin
               min_tt = min(det_tt,max=max_tt)
               if self.instrname eq 'macs' and ~ptr_valid(self.dataStrPtr) then $
                  info = [info,'Detector angle starts from '+dm_to_string(min_tt)+string('b0'XB)+' to '+dm_to_string(max_tt)+string('b0'XB)+' with kidney=0'+string('b0'XB)+'.'] $
               else $
                  info = [info,'Detector angle starts from '+dm_to_string(min_tt)+string('b0'XB)+' to '+dm_to_string(max_tt)+string('b0'XB)+'.']
            endif
            tmp = dm_dialog_input(dialog_parent=self.tlb,'masked angular range:',title='Masking detectors',default=default,info=info,xsize=200)
            if strlen(tmp) eq 0 then begin
               if keyword_set(exist) then begin
                  self->clear_mask
                  ok = dialog_message(['No angular range is specified.','Previously set detector mask has been cleared.'],dialog_parent=self.tlb,/center)
               endif
               break
            endif else begin
               if ptr_valid(self.mask) then begin
                  if (*self.mask) eq tmp then break
               endif
               self->clear_mask
               self.mask = ptr_new(tmp)
            endelse
            self.masktype = 2b
            if self.instrname eq 'macs' then self.ftypecheckmask = ptr_valid(self.mask)
            self->my_widget_control,['mksvBut','mkclBut'],/sensitive
          end
        'mka3but':begin ;mask a3 by angle, macs only
            default = ''
            if ptr_valid(self.a3mask) then begin
               exist = 1b
               default = (*self.a3mask)
            endif
            info = ['Use ":" to denote range, "-" only as the negative sign.','eg: <-10, 20:30,>100']
            tmp = dm_dialog_input(dialog_parent=self.tlb,'masked angular range:',title='Masking A3',default=default,info=info,xsize=200)
            if strlen(tmp) eq 0 then begin
               if keyword_set(exist) then begin
                  self->clear_mask,/a3
                  ok = dialog_message(['No angular range is specified.','Previously set a3 mask has been cleared.'],dialog_parent=self.tlb,/center)
               endif
               break
            endif else begin
               if ptr_valid(self.a3mask) then begin
                  if (*self.a3mask) eq tmp then break
               endif
               self->clear_mask,/a3
               self.a3mask = ptr_new(tmp)
            endelse
            self->my_widget_control,['mksvBut','mkcla3But'],/sensitive
          end  
        'mkpldnum':begin ;plot and set mask by detector number
            if ptr_valid(self.mask) and ((self.masktype eq 2) or ((self.masktype eq 1) and (self.instrname eq 'macs'))) then begin
               self->clear_mask
               exist = 1b
            endif 
            ;must have data
            if (~ ptr_valid(self.dataStrPtr)) then begin
               ok = dialog_message('Please load data files first.',dialog_parent=self.tlb,/center)
               widget_control,event.id,sensitive=0
               break
            endif
            if self.instrname eq 'macs' then begin
               if self.macsintn_typ eq 2 then $
                  qty = reform(transpose((*self.dataStrPtr).qty[0,*,*]+(*self.dataStrPtr).qty[1,*,*])) $
               else $
                  qty = reform(transpose((*self.dataStrPtr).qty[self.macsintn_typ,*,*]))
            endif else $
               qty = (*self.dataStrPtr).qty
            case self.samp_typ of
                 0: detsum = total(qty,1,/nan)
                 1: detsum = total(qty,1,/nan)
                 2: detsum = qty[*,0]
                 3: detsum = total(qty[*,*,0],1,/nan)
            endcase
            ndet  = n_elements(detsum)
            blist = lonarr(ndet)
            if ptr_valid(self.mask) then begin
               ind = where(*self.mask lt ndet,count)
               if count gt 0 then blist[0:count-1] = (*self.mask)[ind]+1
            endif
            self->plot_mask,detsum,blist,exist=exist
            self.masktype = 0b
          end
        'mkplangl':begin ;plot and set mask by angle, only available for DCS, MACS, and WAND
            if ptr_valid(self.mask) and ((self.masktype eq 2) or ((self.masktype eq 0) and (self.instrname eq 'macs'))) then begin
               self->clear_mask
               exist = 1b
            endif 
            ;must have data
            if (~ ptr_valid(self.dataStrPtr)) then begin
               ok = dialog_message('Please load data files first.',dialog_parent=self.tlb,/center)
               widget_control,event.id,sensitive=0
               break
            endif
            case self.samp_typ of
                 0: begin
                    if self.instrname eq 'macs' then begin
                       if self.macsintn_typ eq 2 then $
                          detsum = (*self.dataStrPtr).qty[0,*,*]+(*self.dataStrPtr).qty[1,*,*] $
                       else $
                          detsum = (*self.dataStrPtr).qty[self.macsintn_typ,*,*]
                    endif else begin
                       detsum = total((*self.dataStrPtr).qty,1,/nan)
                    endelse
                    end
                 1: begin
                    if self.instrname eq 'macs' then begin
                       if self.macsintn_typ eq 2 then $
                          detsum = (*self.dataStrPtr).qty[0,*,*]+(*self.dataStrPtr).qty[1,*,*] $
                       else $
                          detsum = (*self.dataStrPtr).qty[self.macsintn_typ,*,*]
                    endif else begin
                       detsum = total((*self.dataStrPtr).qty,1,/nan)
                    endelse
                    end
                 2: detsum = (*self.dataStrPtr).qty[*,0]
                 3: detsum = total((*self.dataStrPtr).qty[*,*,0],1,/nan)
            endcase
            ndet  = n_elements(detsum)
            blist = lonarr(ndet)
            if ptr_valid(self.mask) then begin
               ind = where(*self.mask lt ndet,count)
               if count gt 0 then blist[0:count-1] = (*self.mask)[ind]+1
            endif
            if self.instrname eq 'dcs' then begin
               self->plot_mask,detsum,blist,/angle,exist=exist
            endif else begin
               case self.instrname of
                   'macs':begin
                          n_kid = n_elements((*self.dataStrPtr).kidney)
                          n_det = n_elements((*self.detPosPtr).two_theta)
                          det_tt = ((*self.detPosPtr).two_theta)#(fltarr(n_kid)+1)
                          for i=0L,n_det-1 do det_tt[i,*] = (*self.dataStrPtr).kidney-76.0+(*self.detPosPtr).two_theta[i]
                          detsum = reform(detsum,n_det*n_kid)
                          det_tt = reform(det_tt,n_det*n_kid)
                          end 
                   else:  det_tt = (*self.detPosPtr).two_theta
               endcase
               self->plot_mask,detsum,blist,/angle,det_tt=det_tt,exist=exist
            endelse
            self.masktype = 1b
            if self.instrname eq 'macs' then self.ftypecheckmask = ptr_valid(self.mask)
          end
        'mkbpbut':begin ;automatically mask bragg peaks, or in macs, mask by angular range
            if (~ ptr_valid(self.dataStrPtr)) then begin
               ok = dialog_message('Please load data files first.',dialog_parent=self.tlb,/center)
               widget_control,event.id,sensitive=0
               break
            endif
            if ptr_valid(self.mask) then begin
               ok = dialog_message('This operation will remove the existing masks. Do you want to continue?',/question,dialog_parent=self.tlb,/center)
               if strupcase(ok) eq 'NO' then break
               self->clear_mask     ;clear the mask
               exist = 1b
            endif
            case self.samp_typ of
                 0: detsum = total((*self.dataStrPtr).qty,1,/nan)
                 1: detsum = total((*self.dataStrPtr).qty,1,/nan)
                 2: detsum = (*self.dataStrPtr).qty[*,0]
                 3: detsum = total((*self.dataStrPtr).qty[*,*,0],1,/nan)
            endcase
            case self.samp_typ of
                 0: errsum = sqrt(total((*self.dataStrPtr).err^2,1,/nan))
                 1: errsum = sqrt(total((*self.dataStrPtr).err^2,1,/nan))
                 2: errsum = (*self.dataStrPtr).err[*,0]
                 3: errsum = sqrt(total((*self.dataStrPtr).err[*,*,0]^2,1,/nan))
            endcase
            ;sort detectors by 2theta angles
            if (self.instrname ne 'dcs') and ptr_valid(self.detPosPtr) then begin
               tmpind = sort((*self.detPosPtr).two_theta)
               oldsum = detsum
               detsum = detsum[tmpind]
               errsum = errsum[tmpind]
            endif
            ;masking
            self->maskbraggpeak,detsum,errsum,mask
            if (self.instrname ne 'dcs') and ptr_valid(self.detPosPtr) then begin
               detsum = temporary(oldsum)
               if n_elements(mask) ne 0 then  mask = tmpind[mask] 
               tmpind = 0
            endif
            if n_elements(mask) eq 0  then begin
               mask = [0]     ; no mask
            endif else begin
               mask = mask+1l ;+1 because the mask starts from 1 in plot_mask
            endelse
            if n_elements(mask) eq n_elements(detsum) then mask = [0] ; no mask
            self->plot_mask,detsum,mask,exist=exist
            self.masktype = 0b
          end
        'mkrdbut':begin ;read mask file
            path = self.dirs[1]
            file = dm_choose_file('',/read,dialog_parent=self.tlb,title='Read mask file:',path=path)
            if strlen(file) eq 0 then return
            self->set_dir,workDir=path
            if ptr_valid(self.mask) then exist = 1b
            if ptr_valid(self.a3mask) then a3exist = 1b
            self->clear_mask      ;clear the detector mask
            self->clear_mask,/a3  ;clear the a3 mask
            openr,unit,file,/get_lun
            tmp = ''
            self.masktype = 255b  
            masktype = 0
            while(~ eof(unit)) do begin
                readf,unit,tmp
                tmp = strtrim(tmp,2)
                if strlen(tmp) ne 0 then begin
                   if stregex(tmp,'#type',/boolean,/fold_case) then begin
                      masktype = dm_to_number(strmid(tmp,0,1,/reverse),/int)
                      if masktype ne 3 then self.masktype = 0>(masktype)<2 
                      continue
                   endif
                   if masktype eq 3 then begin
                      if n_elements(a3mask) eq 0 then a3mask=tmp else a3mask=a3mask+' '+tmp
                   endif else begin
                      if ~stregex(tmp,'^[0-9><.,-]+',/boolean) then continue  ;make sure it's not a comment line
                      if n_elements(mask) eq 0 then mask=tmp else mask=[mask,tmp]
                   endelse         
                endif
            endwhile
            free_lun,unit
            if n_elements(mask) ne 0 then begin
               if (self.masktype eq 255) then begin  ;ask for the mask type
                  self.masktype = dm_dialog_input('mask type:',default=0,droplist_content=ptr_new(['detector number','detector angle index','detector angular range']),is_droplist=[1],$
                     /return_number,xsize=200,dialog_parent=self.tlb,info=['Detector angle index type is from Mask->Plot Mask->By Detector Angle.',$
                     'Detector angular range type is from Mask->Set Mask->By Detector Angle.'])
               endif
               if (self.masktype ne 2) then begin  
                  tmp = temporary(mask)
                  for j=0L,n_elements(tmp)-1 do begin
                      tmp1 = strsplit(tmp[j],' ,;'+string(9b),/extract,count=count1)
                      if finite(dm_to_number(tmp1[0])) then begin 
                         for i=0L,count1-1L do begin
                             tmp2 = strsplit(tmp1[i],'-:',/extract,count=count2)
                             tmp2 = dm_to_number(tmp2,/long)
                             if count2 eq 2 then $
                                tmp2 = min(tmp2)+lindgen(abs(tmp2[1]-tmp2[0])+1)
                             if n_elements(mask) eq 0 then mask=tmp2 else mask=[mask,tmp2]
                         endfor
                      endif 
                  endfor
                  if n_elements(mask) ne 0 then begin
                     ok = dm_dialog_input('det number starts from:',default=0,droplist_content=ptr_new(['0','1']),is_droplist=[1],/return_number,dialog_parent=self.tlb)
                     mask = mask-ok
                  endif
               endif else begin
                  for i=1L,n_elements(mask)-1 do mask[0] = mask[0]+' '+mask[i]
                  mask = mask[0]
               endelse
            endif 
            if n_elements(mask) ne 0 then begin
               self.mask = ptr_new(mask)
            endif else if keyword_set(exist) then begin
               ok = dialog_message('Previously set detector mask has been cleared.',dialog_parent=self.tlb,/center)
            endif
            if n_elements(a3mask) ne 0 then begin
               self.a3mask = ptr_new(a3mask)
            endif else if keyword_set(a3exist) then begin
               ok = dialog_message('Previously set a3 mask has been cleared.',dialog_parent=self.tlb,/center)
            endif  
            self->my_widget_control,'mksvBut',sensitive=(ptr_valid(self.mask) or ptr_valid(self.a3mask))
            self->my_widget_control,'mkclBut',sensitive=ptr_valid(self.mask)
            self->my_widget_control,'mkcla3But',sensitive=ptr_valid(self.a3mask)
          end
        'mksvbut':begin ;save mask in a file, including a3 mask
            if ptr_valid(self.mask) or ptr_valid(self.a3mask) then begin
               path = self.dirs[1]
               file = dm_choose_file('msk',/write,dialog_parent=self.tlb,title='Save mask in an ASCII file.',path=path)
               if strlen(file) eq 0 then break
               self->set_dir,workDir=path
               openw,unit,file,/get_lun,error=openerr
               if openerr ne 0 then begin
                  ok = dialog_message("Can't write in "+file[0],/error,dialog_parent=self.tlb,/center)
                  break
               endif else begin
                  widget_control,/hourglass  ;busy signal
               endelse
            endif else break
            if ptr_valid(self.mask) then begin
               printf,unit,'#type '+dm_to_string(self.masktype)
               printf,unit,*(self.mask)   
            endif
            if ptr_valid(self.a3mask) then begin
               printf,unit,'#type 3'  ;3 to denote a3 mask
               printf,unit,*(self.a3mask)   
            endif
            free_lun,unit
          end
        'mkclbut':    self->clear_mask         ;clear the mask
        'mkcla3but':  self->clear_mask,/a3     ;clear the a3 mask
        'directgeom': if self.instrgeom ne 0 then self->set_instrgeom,geom=0
        'inversegeom':if self.instrgeom ne 1 then self->set_instrgeom,geom=1
        'intntyp0':if self.intn_typ ne 0 then self->set_intntype,0
        'intntyp1':if self.intn_typ ne 1 then self->set_intntype,1
        'intntyp2':if self.intn_typ ne 2 then self->set_intntype,2
        'intntyp3':if self.intn_typ ne 3 then self->set_intntype,3
        'intntyp4':if self.intn_typ ne 4 then self->set_intntype,4
        'intntyp5':if self.intn_typ ne 5 then self->set_intntype,5
        'intntyp6':if self.intn_typ ne 6 then self->set_intntype,6
        'intneint': begin
            self.eint_yn = ~self.eint_yn
            dm_set_button,event.id,self.eint_yn,onstring='E-Integrated Intensity Enabled',offstring='E-Integrated Intensity Disabled'
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'macsintspec': if self.macsintn_typ ne 0 then begin
            self.macsintn_typ = 0
            self->my_menu_toggle,'macsint'+['SPEC','DIFF','SPDF'],self.macsintn_typ
            self->set_display_button,clearprojection=(self.samp_typ eq 1)
          end  
        'macsintdiff': if self.macsintn_typ ne 1 then begin
            self.macsintn_typ = 1
            self->my_menu_toggle,'macsint'+['SPEC','DIFF','SPDF'],self.macsintn_typ
            self->set_display_button,clearprojection=(self.samp_typ eq 1)
          end  
        'macsintspdf': if self.macsintn_typ ne 2 then begin
            self.macsintn_typ = 2
            self->my_menu_toggle,'macsint'+['SPEC','DIFF','SPDF'],self.macsintn_typ
            self->set_display_button,clearprojection=(self.samp_typ eq 1)
          end      
        'binavg':if self.bin_avgsum ne 0 then begin
            self.bin_avgsum = 0
            self->my_menu_toggle,'bin'+['Avg','Sum'],self.bin_avgsum
          endif
        'binsum':if self.bin_avgsum ne 1 then begin
            self.bin_avgsum = 1
            self->my_menu_toggle,'bin'+['Avg','Sum'],self.bin_avgsum
          endif
        'bincsyes': if self.bin_conststep ne 1 then begin
            self.bin_conststep = 1
            self->my_menu_toggle,'bincs'+['No','Yes'],self.bin_conststep
          endif
        'bincsno': if self.bin_conststep ne 0 then begin
            self.bin_conststep = 0
            self->my_menu_toggle,'bincs'+['No','Yes'],self.bin_conststep
          endif
        'binexyes': if self.bin_extlib ne 1 then begin
            self.bin_extlib = 1
            self->my_menu_toggle,'binex'+['No','Yes'],self.bin_extlib
          endif
        'binexno': if self.bin_extlib ne 0 then begin
            self.bin_extlib = 0
            self->my_menu_toggle,'binex'+['No','Yes'],self.bin_extlib
          endif
        'monsum':if self.mon_sumfit ne 0 then begin
            self.mon_sumfit = 0
            self->my_menu_toggle,'mon'+['Sum','Fit'],self.mon_sumfit
            if (self.instrname eq 'macs') then begin
               if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
               self->reset_intnlab
            endif
            if (self.instrname eq 'dcs') and ptr_valid(self.dataStrPtr) then begin  ;clear data
               ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
               cid = widget_info(self.tlb,find_by_uname='clearDat') 
               if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
               else ptr_free,self.dataStrPtr
            endif
          endif
        'monfit':begin
            if (self.mon_sumfit eq 0) then begin
               if (self.instrname eq 'macs') and ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
               if (self.instrname eq 'dcs') and ptr_valid(self.dataStrPtr) then begin  ;clear data
                  ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
                  cid = widget_info(self.tlb,find_by_uname='clearDat') 
                  if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
                  else ptr_free,self.dataStrPtr
               endif
            endif
            self.mon_sumfit = 1
            self->my_menu_toggle,'mon'+['Sum','Fit'],self.mon_sumfit
            if self.instrname eq 'macs' then begin
               old = self.macs_mon[1] 
               repeat begin
                  new = dm_dialog_input('monitor count:',default=dm_to_string(old,/exponent),dialog_parent=self.tlb,info='Intensity is normalized to this monitor count.',/float)
                  if finite(new) then begin
                     if new gt 0 then ok2 = 1b
                  endif
               endrep until keyword_set(ok2)
               if new ne old then begin
                  self.macs_mon[1] = new
                  widget_control,event.id,set_value='Specify ('+dm_to_string(self.macs_mon[1],/exponent)+')'
                  if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
               endif
               self->reset_intnlab
            endif
          end
        'monfc0':if self.dcs_mon ne 0 then begin
            self.dcs_mon = 0
            self->my_menu_toggle,'mon'+['FC0','BM1','TIME'],self.dcs_mon
            if ptr_valid(self.dataStrPtr) then begin  ;clear data
               ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
               cid = widget_info(self.tlb,find_by_uname='clearDat') 
               if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
               else ptr_free,self.dataStrPtr
            endif
          endif
        'monbm1':if self.dcs_mon ne 1 then begin
            self.dcs_mon = 1
            self->my_menu_toggle,'mon'+['FC0','BM1','TIME'],self.dcs_mon
            if ptr_valid(self.dataStrPtr) then begin  ;clear data
               ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
               cid = widget_info(self.tlb,find_by_uname='clearDat') 
               if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
               else ptr_free,self.dataStrPtr
            endif
          endif
        'montime':if self.dcs_mon ne 2 then begin
            self.dcs_mon = 2
            self->my_menu_toggle,'mon'+['FC0','BM1','TIME'],self.dcs_mon
            if ptr_valid(self.dataStrPtr) then begin  ;clear data
               ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
               cid = widget_info(self.tlb,find_by_uname='clearDat') 
               if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
               else ptr_free,self.dataStrPtr
            endif
          endif
        'monlamb2': begin
            self.macs_lamb2 = ~self.macs_lamb2
            dm_set_button,event.id,self.macs_lamb2,onstring='Apply Monitor Lambda/2 Correction',offstring='No Monitor Lambda/2 Correction'
            if ptr_valid(self.dataStrPtr) and ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'abspnone':     self->absorption,-1
        'abspcylinder': self->absorption,0
        'abspannulus':  self->absorption,1
        'abspsphere':   self->absorption,2
        'abspflat':     self->absorption,3
        'vaxisarbi': if self.view_ortharbi ne 1 then begin
            self.view_ortharbi = 1
            self->my_menu_toggle,'vaxis'+['Ortho','Arbi'],self.view_ortharbi 
            self->my_widget_control,'viewAxis',set_value='  Viewing Axes:'
          endif
        'vaxisortho':if self.view_ortharbi ne 0 then begin
            self.view_ortharbi = 0
            self->my_menu_toggle,'vaxis'+['Ortho','Arbi'],self.view_ortharbi 
            self->my_widget_control,'viewAxis',set_value='  Orthogonal Viewing Axes:'
          endif
        'vfoldmirror':if self.fold_type ne 0 then begin
            self.fold_type = 0
            self->my_menu_toggle,'vfold'+['Mirror','Rotation'],self.fold_type
            self.view_u1fold = 0 & self.view_u2fold = 0 & self.view_u3fold = 0
            self->reset_projection,/reset,/foldonly
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          endif
        'vfoldrotation':if self.fold_type ne 1 then begin
            self.fold_type = 1
            self->my_menu_toggle,'vfold'+['Mirror','Rotation'],self.fold_type
            self.view_u1fold = 0 & self.view_u2fold = 0 & self.view_u3fold = 0
            self->reset_projection,/reset,/foldonly
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          endif  
        'powdernone': self->change_powderline,0
        'powderal':   self->change_powderline,1
        'powdercu':   self->change_powderline,2
        'powderst':   self->change_powderline,3
        'powdersp':   self->change_powderline,4
        'powdershowall': begin
            self.powd_showall = ~self.powd_showall
            dm_set_button,event.id,self.powd_showall,onstring='Show Closed Powder Line',offstring='Show Actual Powder Line'
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end
        'epadnone': self->eadjust,0
        'epadauto': self->eadjust,1
        'epadspec': self->eadjust,2
        'empttol_cwt': if self.empt_tol_optn ne 0 then begin
            self.empt_tol_optn = 0b
            self->my_menu_toggle,'empttol_'+['cwt','wtc'],self.empt_tol_optn
            if ptr_valid(self.bgdata) and ptr_valid(self.dataStrPtr) and ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end
        'empttol_wtc': if self.empt_tol_optn ne 1then begin
            self.empt_tol_optn = 1b
            self->my_menu_toggle,'empttol_'+['cwt','wtc'],self.empt_tol_optn
            if ptr_valid(self.bgdata) and ptr_valid(self.dataStrPtr) and ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end
        'epadcheckfirst':begin
            self.eadjust_checkfirst = ~self.eadjust_checkfirst  
            dm_set_button,event.id,self.eadjust_checkfirst,onstring='Check the First File Only',offstring='Check All Files'
          end
        'pbgblack': if self.pbgcol_bw ne 0 then begin
            self.pbgcol_bw = 0
            self->my_menu_toggle,'pbg'+['Black','White'],self.pbgcol_bw 
            (*self.colors)[0] = 'white'
            if total(self.coldefine) eq 0 then self.coldefine = [255b,255b,255b]
            dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutColor'),select=self.cut_color,set_value=(*self.colors)
            oplots = self.gr_list->get(/all,count=count)
            for i=0,count-1 do oplots[i]->setproperty,bgcolor=0
            if obj_valid(self.current) and (count gt 1) then self.current->totop
          endif    
        'pbgwhite': if self.pbgcol_bw ne 1 then begin
            self.pbgcol_bw = 1
            self->my_menu_toggle,'pbg'+['Black','White'],self.pbgcol_bw 
            (*self.colors)[0] = 'black'
            if total(255-self.coldefine) eq 0 then self.coldefine = [0b,0b,0b]
            dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutColor'),select=self.cut_color,set_value=(*self.colors)
            oplots = self.gr_list->get(/all,count=count)
            for i=0,count-1 do oplots[i]->setproperty,bgcolor=1
            if obj_valid(self.current) and (count gt 1) then self.current->totop
          endif   
        'pdim_choice':  begin
            self.pdim_choice = ind_sel
            pdimmenu = widget_info(self.tlb,find_by_uname='pdimMenu')
            self->my_menu_toggle,'pdchoice_'+dm_to_string(indgen(n_elements((*self.pdim_list)[0,*])),/int),self.pdim_choice,base=pdimmenu
          end 
        'userfunction': begin
             self.userfwarning = 0b ;clear the warning flag
             if lmgr(/runtime) then begin
                name  = ['user macro:','execution time:'] 
                title = 'Specify user macro'
                examp = ['Example: index = where(qty lt 0,count)','              if count gt 0 then qty[index]=!values.f_nan'] 
             endif else begin
                name  = ['user macro/call:','execution time:']
                title = 'Specify user macro or calls'
                examp = ['Example 1: index = where(qty lt 0,count)',$
                         '                if count gt 0 then qty[index]=!values.f_nan',$
                         'Example 2: qty=my_function(qty,err,en=en,'+((self.samp_typ gt 0)?'Qv=Qv)':'Q=Q)'),$
                         'Example 3: my_procedure,qty,err,en,det_tt=det_tt']
             endelse
             default = self.userf_str
             ;replace '&' with string('A4'xb)
             while((index=strpos(default,'&')) ge 0) do strput,default,string('A4'xb),index
             default = [default,dm_to_string(self.userf_before)]
             answer  = dm_dialog_input(name,title=title,default=default,dialog_parent=self.tlb,$
                       info=[examp,userf_info,'Clear the string to disable this option. Do not use comment lines.'],xsize=200,multiline=4,$
                       droplist_content=ptr_new([['After','Before']+' Projection to Viewing Axes','Before Empty Can Subtraction']),is_droplist=[0,1],/return_number,/scroll)
             ;replace possible string('A4'xb) with '&'
             b_a = dm_to_number(answer[1],/int)
             str = answer[0]
             while((index=strpos(str,string('A4'xb))) ge 0) do strput,str,'&',index
             ;remove possible '$&'
             while((index=strpos(str,'$&')) ge 0) do strput,str,'  ',index
             changed = (self.userf_str ne str) or (self.userf_before ne b_a)
             self.userf_str    = str
             self.userf_before = b_a
             widget_control,event.id, set_button=(strlen(str) ne 0)
             if ptr_valid(self.projStrPtr) and changed then recalc_proj = 1b    ;need to recalculate projections
          end
        'extravaxis':begin
            self.extravaxis_yn = ~self.extravaxis_yn
            onstr  = (['Allow Extra Viewing Axis','Allow Temperature as Extra Viewing Axis'])[self.instrname eq 'macs']
            offstr = (['Disallow Extra Viewing Axis','Disallow Temperature as Extra Viewing Axis'])[self.instrname eq 'macs']
            dm_set_button,event.id,self.extravaxis_yn,onstring=onstr,offstring=offstr
            self->my_widget_control,'taxisMenu',sensitive=self.extravaxis_yn
            if self.extravaxis_yn then ptr_free,self.projStrPtr  ;need to recalculate projection
            if (self.instrname eq 'macs') or (self.instrname eq 'wand') then begin
               if self.extravaxis_yn and ptr_valid(self.dataStrPtr) then begin  ;need to reload data
                  ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
                  cid = widget_info(self.tlb,find_by_uname='clearDat') 
                  if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
                  else ptr_free,self.dataStrPtr
               endif
            endif
            self->reset_pdt
          end
        'taxissamp':if self.macstemp_typ ne 0 then begin
            self.macstemp_typ = 0
            self->my_menu_toggle,'taxis'+['Samp','Ctrl','Setp'],self.macstemp_typ
            if ptr_valid(self.projStrPtr) and ptr_valid(self.dataStrPtr) and ((self.intn_typ eq 1) or (self.intn_typ eq 3) or (self.intn_typ eq 4) or (self.intn_typ eq 5)) then recalc_proj = 1b
          end  
        'taxisctrl':if self.macstemp_typ ne 1 then begin
            self.macstemp_typ = 1
            self->my_menu_toggle,'taxis'+['Samp','Ctrl','Setp'],self.macstemp_typ
            if ptr_valid(self.projStrPtr) and ptr_valid(self.dataStrPtr) and ((self.intn_typ eq 1) or (self.intn_typ eq 3) or (self.intn_typ eq 4) or (self.intn_typ eq 5)) then recalc_proj = 1b
          end  
        'taxissetp':if self.macstemp_typ ne 2 then begin
            self.macstemp_typ = 2
            self->my_menu_toggle,'taxis'+['Samp','Ctrl','Setp'],self.macstemp_typ
            if ptr_valid(self.projStrPtr) and ptr_valid(self.dataStrPtr) and ((self.intn_typ eq 1) or (self.intn_typ eq 3) or (self.intn_typ eq 4) or (self.intn_typ eq 5)) then recalc_proj = 1b
          end   
        'avgtemp':begin
            self.macstempavg_yn = ~self.macstempavg_yn
            dm_set_button,event.id,self.macstempavg_yn,onstring='Average Temperature of Each File',offstring='Keep Individaul Temperature'
            if ptr_valid(self.dataStrPtr) then begin  ;need to reload data
               ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
               cid = widget_info(self.tlb,find_by_uname='clearDat') 
               if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
               else ptr_free,self.dataStrPtr
            endif
          end     
        'darkcorr':begin
            self.dcor_yn = ~self.dcor_yn
            ptr_free,self.dcor_info
            if self.dcor_yn then begin
               self.filesel->getproperty,path=open_path,file=open_file
               if n_elements(open_file) eq 0 then begin
                  ok = dialog_message(dialog_parent=self.tlb,'Please select dark angle correction files first.',/error,/center)
                  self.dcor_yn = 0b
               endif else begin
                  default = self.dcor_eqran
                  eqran   = dm_dialog_input(['E_start (meV):','E_end (meV):','2theta_start:','2theta_end:'],title='Specify E, 2theta range:',xsize=80,$
                            default=default,info=['The total intensity in the specified range will','be used to calculate the dark angle correction.'],dialog_parent=self.tlb)
                  if total(finite(eqran)) ne 4 then $
                     self.dcor_yn = 0b $
                  else if (eqran[0] ge eqran[1]) or (eqran[2] ge eqran[3]) then $
                     self.dcor_yn = 0b
                  if self.dcor_yn eq 0 then $
                     ok = dialog_message('Invalid input. Dark angle correction is disabled.',dialog_parent=self.tlb,/center) $
                  else begin
                     self.dcor_eqran = eqran
                     det_tt = (*self.detPosPtr).two_theta
                     index  = where((abs(det_tt) ge self.dcor_eqran[2]) and (abs(det_tt) le self.dcor_eqran[3]),count)
                     if count eq 0 then begin
                        ok  = dialog_message(['The specified 2theta angle is beyond the detector range.','Dark angle correction is skipped.'],dialog_parent=self.tlb,/center)
                        self.dcor_yn = 0b
                     endif else begin
                        dm_load_dcsdata,open_path,open_file,qty=tmp_qty,t_chan=2,e_range=self.dcor_eqran[0:1],ang=ang,parent=self.tlb,/raw,title='Load Dark Angle Files',eadjust = self.eadjust,$
                              epadjust=self.eadjust_spec,epadcheckfirst=self.eadjust_checkfirst,montype=self.dcs_mon
                        tmp_qty = tmp_qty[index,*]
                        tmp_qty = total(tmp_qty,1)
                        tmp_qty = max(tmp_qty)/tmp_qty
                        self.dcor_info = ptr_new({factor:tmp_qty,psi:reform(ang[0,*])})
                     endelse
                  endelse
               endelse
            endif
            dm_set_button,event.id,self.dcor_yn,onstring='Dark Angle Correction Enabled',offstring='Dark Angle Correction Disabled'
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'darktraj':begin
            self.datr_yn = ~self.datr_yn
            dm_set_button,event.id,self.datr_yn,onstring='Show Dark Angle Trajectory',offstring='Disable Dark Angle Trajectory'
            if ptr_valid(self.projStrPtr) and (~ptr_valid(self.dataStrPtr)) then recalc_proj = 1b
          end 
        'edepeff':begin
            self.edep_yn = ~self.edep_yn
            dm_set_button,event.id,self.edep_yn,onstring='E-Dependent Detector Efficiency Correction Enabled',offstring='E-Dependent Detector Efficiency Correction Disabled'
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
            if self.edep_yn then begin
               case self.instrname of 
                    'dcs':  info = 'The default values for DCS detecotors are 1 cm and 6 atm.'
                    'macs': info = 'The default values for MACS detectors are 3.773 cm and 3 atm.'
                    else:   info = '1 cm and 6 atm are the default parameters for DCS detectors.'
               endcase
               repeat begin
                  new = dm_dialog_input('He3 detector '+['thickness(cm):','pressure(atm):'],title='Please confirm detector info',dialog_parent=self.tlb,default=self.edep_det,info=info,/float)
               endrep until total(finite(new,/nan)) eq 0
               self.edep_det = new
            end
          end
        'errbar':begin
            self.errbar = ~self.errbar
            dm_set_button,event.id,1-self.errbar,onstring='Error Bar Disabled',offstring='Error Bar Enabled'
          end
        'viewspurion': self->dm_det_spurion
        'useitool': begin
            self.useitool = ~self.useitool
            dm_set_button,event.id,self.useitool,onstring='Use Classical Plotting',offstring='Use iTools Plotting'
          end
        'viewproj':begin
            self.view[0] = (~self.view[0])
            dm_set_button,event.id,self.view[0],onstring='Hide Projection',offstring='Show Projection'
            self->reset_pdt
          end
        'viewdisp':begin
            self.view[1] = (~self.view[1])
            dm_set_button,event.id,self.view[1],onstring='Hide Display',offstring='Show Display'
            self->reset_pdt
          end
        'viewcurr':   if obj_valid(self.current) then self.current->totop
        'killall':    self->dm_add_plotwindow,4
        'arrcascade': self->dm_arrange_plot,/cascade
        'arrtable':   self->dm_arrange_plot
        'hpctbut':if strlen(self.helpfile[0]) ne 0 then begin
               online_help,book=self.helpfile[0],/full_path
          endif
        'hpftbut':if strlen(self.helpfile[1]) ne 0 then begin
               online_help,book=self.helpfile[1],/full_path
          endif
        'hpnewbut':  ok = dialog_message(*self.whatsnew,/information,dialog_parent=self.tlb,title="What's new in Mslice",/center)
        'selfbut':if ptr_valid(self.dataPtr) then $
                     ok = dialog_message(*self.info,/information,dialog_parent=self.tlb,title='About Mslice',/center) $
                  else $
                     ok = dialog_message(*self.info,/information,dialog_parent=self.tlb,title='About Mslice for '+(*self.ftypename)[self.ftype],/center)
        'cleardat':  if event.select then begin
            self.monrate = 0d
            ptr_free,self.kfactor,self.dataStrPtr,self.projStrPtr
            self->my_widget_control,['addBut','mkplBut'],sensitive=0  ;disallow adding data & plotting mask
            widget_control,event.id,sensitive=0
            self->my_widget_control,'titlLab',set_value='' & self.titl_lab=''   ;clear title label
            self->my_widget_control,'legdLab',set_value='' & self.legd_lab=''   ;clear legend label
            if self.instrname eq 'macs' then begin
               self.macs_mon[0] = !values.f_nan
               self->my_widget_control,'monSum',set_value='Automatic'
               self->reset_intnlab
               if ptr_valid(self.mask) and (self.masktype eq 1) then self->clear_mask,/message
            endif
            self->set_display_button
          end
        'intnlab':  self.intn_lab = tmp_value   ;intensity label
        'titllab':  self.titl_lab = tmp_value   ;title label
        'legdlab':  self.legd_lab = tmp_value   ;legend label
        'powdtyp':  self->change_samptyp,0
        'crysttyp1':self->change_samptyp,1
        'crysttyp2':self->change_samptyp,2
        'crysttyp3':self->change_samptyp,3
        'ebin':  begin    ;ebin size /energy range
            changed  = 0b
            old_ebin = [self.e_bin[self.samp_typ],self.e_bin[4]]
            old_eran = self.e_range 
            if self.samp_typ ge 2 then begin   ;diffuse scattering
               self.e_bin[self.samp_typ] = tmp_value
               if tmp_value eq 2 then begin     ;specify energy range
                  default = dm_to_string(self.e_range)
                  items   = ['from:','to:']
                  info    = ['Energy range is in units of meV.','Data files need to be reloaded.']
                  if self.samp_typ eq 3 then begin
                     default = [default,dm_to_string(self.e_bin[4])]
                     items   = [items,'E rebin size:']
                  endif
                  new = dm_dialog_input(items,title='Energy range',default=default,/float,dialog_parent=self.tlb,info=info)
                  self.e_bin[self.samp_typ] = 1
                  self.e_range[*] = !values.f_nan
                  if total(finite(new[0:1])) eq 2 then $
                     if new[1] gt new[0] then begin
                        self.e_range = new[0:1]
                        self.e_bin[self.samp_typ] = 2
                     endif
                  if self.e_bin[self.samp_typ] eq 1 then begin
                     ok = dialog_message('Invalid energy range. Elastic peak option is used.',dialog_parent=self.tlb,/center)
                     dm_set_droplist,event.id,select=1
                  endif
                  if self.samp_typ eq 3 then begin  ;rebin size
                     new = fix(new[2])
                     if new le 0 then begin
                        new = 1
                        ok = dialog_message('Invalid rebin size. 1 is used instead.',dialog_parent=self.tlb,/center)
                     endif
                     self.e_bin[4] = new
                  end
               endif else if self.samp_typ eq 3 then begin  ;need to ask for rebin size for multiple E type
                  new = dm_to_string(self.e_bin[4])
                  new = dm_dialog_input('E rebin size:', title='Rebin size',default=new,/int,dialog_parent=self.tlb)
                  if new le 0 then begin
                     new = 1
                     ok = dialog_message('Invalid rebin size. 1 is used instead.',dialog_parent=self.tlb,/center)
                  endif
                  self.e_bin[4] = new   
               endif
            endif else begin    ;single crystal or powder
               self.e_bin[self.samp_typ] = (*self.rebinsize)[tmp_value]
            endelse
            if self.e_bin[self.samp_typ] ne old_ebin[0] then changed = 1b else begin
               if self.samp_typ ge 2 then begin
                  if self.e_bin[self.samp_typ] eq 2 then begin
                     if (total(finite(self.e_range)) ne 2) or (total(finite(old_eran)) ne 2) then changed=1b else begin
                        if total(abs(self.e_range-old_eran)) gt 1e-7 then changed = 1b
                     endelse
                  endif
                  if self.samp_typ eq 3 then begin
                     if self.e_bin[4] ne old_ebin[1] then changed = 1b
                  endif
               endif
            endelse
            if ptr_valid(self.bgdata) and changed then begin
               ptr_free,self.bgdata,self.bgdata2
               if self.extraempcan_yn then self->my_widget_control,'emptBut2',sensitive=0
               self->my_widget_control,'bgclBut',sensitive=(ptr_valid(self.bgratePtr) or ptr_valid(self.bgtchnPtr) or ptr_valid(self.bgdetsPtr))
               ok = dialog_message('Please reload empty can files.',dialog_parent=self.tlb,/center)
            endif
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
          end
        'latta': self.latt_parm[0] = dm_to_number(tmp_value,/float)   ;a
        'lattb': self.latt_parm[1] = dm_to_number(tmp_value,/float)   ;b
        'lattc': self.latt_parm[2] = dm_to_number(tmp_value,/float)   ;c
        'lattaa':self.latt_parm[3] = dm_to_number(tmp_value,/float)   ;alpha
        'lattbb':self.latt_parm[4] = dm_to_number(tmp_value,/float)   ;beta
        'lattcc':self.latt_parm[5] = dm_to_number(tmp_value,/float)   ;gama
        'orieux':self.orie_u[0]    = dm_to_number(tmp_value,/float)   ;ux
        'orieuy':self.orie_u[1]    = dm_to_number(tmp_value,/float)   ;uy
        'orieuz':self.orie_u[2]    = dm_to_number(tmp_value,/float)   ;uz
        'orievx':self.orie_v[0]    = dm_to_number(tmp_value,/float)   ;vx
        'orievy':self.orie_v[1]    = dm_to_number(tmp_value,/float)   ;vy
        'orievz':self.orie_v[2]    = dm_to_number(tmp_value,/float)   ;vz
        'oriepsi': begin  ;psi
            tmp_value = dm_to_number(tmp_value,/float)
            if self.orie_psi ne tmp_value then begin
               self.orie_psi    = tmp_value   
               self.orie_offset = !values.f_nan
            endif
          end
        'viewu1typ':if self.view_u1typ ne tmp_value then begin
            self.view_u1typ = tmp_value
            self->script,'set_parameter',name='viewU1lab',value=self.view_label[tmp_value]
            resetu1 = 1 & islabel = 1
          endif
        'viewu1h': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu1 = finite(tmp)?(finite(self.view_u1[0])?((self.view_u1[0] eq tmp)?0:1):(1)):(0)
            self.view_u1[0] = tmp
          end
        'viewu1k': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu1 = finite(tmp)?(finite(self.view_u1[1])?((self.view_u1[1] eq tmp)?0:1):(1)):(0)
            self.view_u1[1] = tmp
          end
        'viewu1l': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu1 = finite(tmp)?(finite(self.view_u1[2])?((self.view_u1[2] eq tmp)?0:1):(1)):(0)
            self.view_u1[2] = tmp
          end
        'viewu1lab':if self.view_u1lab[(self.samp_typ ge 1)] ne tmp_value then begin
            self.view_u1lab[(self.samp_typ ge 1)] = (strlen(tmp_value) eq 0)?' ':tmp_value
            if self.samp_typ eq 0 then $
               self.view_label[self.view_u1typ] = self.view_u1lab[0]
            resetu1 = 1 & islabel = 1
          endif
        'viewu1fold':  begin
            self.view_u1fold = ~self.view_u1fold
            self->reset_projection,/reset,/foldonly
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end 
        'viewu1foldcenter': begin
            old = dm_to_string(*(self.view_u1foldcenter),separator=',')
            *(self.view_u1foldcenter) = dm_to_number(strsplit(tmp_value,' ,;',/extract),/float)
            if dm_to_string(*(self.view_u1foldcenter),separator=',') ne old then begin
               if self.fold_type eq 0 then begin
                  *(self.view_u1folddir) = (*(self.view_u1folddir))[0] ;reset folding direction to the first value
                  dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU1Folddir'),select=(*self.view_u1folddir)[0]+1,set_value=['-','auto','+']
               endif
               if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
            endif
          end
        'viewu1folddir': begin
            old = dm_to_string(*(self.view_u1folddir),separator=',')
            nfc = n_elements(*(self.view_u1foldcenter))
            if nfc le 1 then begin
               *(self.view_u1folddir) = tmp_value-1
            endif else begin
               *(self.view_u1folddir) = (dm_dialog_input('center value '+dm_to_string(*(self.view_u1foldcenter))+':',title='Set u1 folding direction',$
                    default =*(self.view_u1folddir)+1,is_droplist=1+intarr(nfc),droplist_content=ptr_new(['-','auto','+']),xsize=100,dialog_parent=self.tlb,/return_number)-1)
               value = (['-','auto','+'])[(*self.view_u1folddir)[0]+1]
               for i=1,nfc-1 do value = value+','+(['-','auto','+'])[(*self.view_u1folddir)[i]+1]
               dm_set_droplist,event.id,select=0,set_value=[value,'auto','+']
            endelse
            if (dm_to_string(*(self.view_u1folddir),separator=',') ne old) and ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end
        'viewu2typ':if self.view_u2typ ne tmp_value then begin
            self.view_u2typ = tmp_value
            self->script,'set_parameter',name='viewU2lab',value=self.view_label[tmp_value]
            resetu2 = 1 & islabel = 1
          endif
        'viewu2h': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu2 = finite(tmp)?(finite(self.view_u2[0])?((self.view_u2[0] eq tmp)?0:1):(1)):(0)
            self.view_u2[0] = tmp
          end
        'viewu2k': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu2 = finite(tmp)?(finite(self.view_u2[1])?((self.view_u2[1] eq tmp)?0:1):(1)):(0)
            self.view_u2[1] = tmp
          end
        'viewu2l': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu2 = finite(tmp)?(finite(self.view_u2[2])?((self.view_u2[2] eq tmp)?0:1):(1)):(0)
            self.view_u2[2] = tmp
          end
        'viewu2lab':if self.view_u2lab[(self.samp_typ ge 1)] ne tmp_value then begin
            self.view_u2lab[(self.samp_typ ge 1)] = (strlen(tmp_value) eq 0)?' ':tmp_value
            if self.samp_typ eq 0 then $
               self.view_label[self.view_u2typ] = self.view_u2lab[0]
            resetu2 = 1 & islabel = 1
          endif
        'viewu2fold':  begin
            self.view_u2fold = ~self.view_u2fold
            self->reset_projection,/reset,/foldonly
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end 
        'viewu2foldcenter': begin
            old = dm_to_string(*(self.view_u2foldcenter),separator=',')
            *(self.view_u2foldcenter) = dm_to_number(strsplit(tmp_value,' ,;',/extract),/float)
            if dm_to_string(*(self.view_u2foldcenter),separator=',') ne old then begin
               *(self.view_u2folddir) = (*(self.view_u2folddir))[0] ;reset folding direction to the first value
               dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU2Folddir'),select=(*self.view_u2folddir)[0]+1,set_value=['-','auto','+']
               if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
            endif
          end
        'viewu2folddir': begin
            old = dm_to_string(*(self.view_u2folddir),separator=',')
            nfc = n_elements(*(self.view_u2foldcenter))
            if nfc le 1 then begin
               *(self.view_u2folddir) = tmp_value-1
            endif else begin
               *(self.view_u2folddir) = (dm_dialog_input('center value '+dm_to_string(*(self.view_u2foldcenter))+':',title='Set u2 folding direction',$
                    default =*(self.view_u2folddir)+1,is_droplist=1+intarr(nfc),droplist_content=ptr_new(['-','auto','+']),xsize=100,dialog_parent=self.tlb,/return_number)-1)
               value = (['-','auto','+'])[(*self.view_u2folddir)[0]+1]
               for i=1,nfc-1 do value = value+','+(['-','auto','+'])[(*self.view_u2folddir)[i]+1]
               dm_set_droplist,event.id,select=0,set_value=[value,'auto','+']
            endelse
            if (dm_to_string(*(self.view_u2folddir),separator=',') ne old) and ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end
        'viewu3h': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu3 = finite(tmp)?(finite(self.view_u3[0])?((self.view_u3[0] eq tmp)?0:1):(1)):(0)
            self.view_u3[0] = tmp
          end
        'viewu3k': begin
            tmp     = dm_to_number(tmp_value,/float)
            resetu3 = finite(tmp)?(finite(self.view_u3[1])?((self.view_u3[1] eq tmp)?0:1):(1)):(0)
            self.view_u3[1] = tmp
          end
        'viewu3l': begin
            tmp     =  dm_to_number(tmp_value,/float)
            resetu3 = finite(tmp)?(finite(self.view_u3[2])?((self.view_u3[2] eq tmp)?0:1):(1)):(0)
            self.view_u3[2] = tmp
          end
        'viewu3lab':if self.view_u3lab ne tmp_value then begin
            self.view_u3lab = (strlen(tmp_value) eq 0)?' ':tmp_value
            resetu3 = 1 & islabel = 1
          endif
        'viewu3fold':  begin
            self.view_u3fold = ~self.view_u3fold
            self->reset_projection,/reset,/foldonly
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end 
        'viewu3foldcenter': begin
            old = dm_to_string(*(self.view_u3foldcenter),separator=',')
            *(self.view_u3foldcenter) = dm_to_number(strsplit(tmp_value,' ,;',/extract),/float)
            if dm_to_string(*(self.view_u3foldcenter),separator=',') ne old then begin
               *(self.view_u3folddir) = (*(self.view_u3folddir))[0] ;reset folding direction to the first value
               dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU3Folddir'),select=(*self.view_u3folddir)[0]+1,set_value=['-','auto','+']
               if ptr_valid(self.projStrPtr) then recalc_proj = 1b 
            endif
          end
        'viewu3folddir': begin
            old = dm_to_string(*(self.view_u3folddir),separator=',')
            nfc = n_elements(*(self.view_u3foldcenter))
            if nfc le 1 then begin
               *(self.view_u3folddir) = tmp_value-1
            endif else begin
               *(self.view_u3folddir) = (dm_dialog_input('center value '+dm_to_string(*(self.view_u3foldcenter))+':',title='Set u3 folding direction',$
                    default =*(self.view_u3folddir)+1,is_droplist=1+intarr(nfc),droplist_content=ptr_new(['-','auto','+']),xsize=100,dialog_parent=self.tlb,/return_number)-1)
               value = (['-','auto','+'])[(*self.view_u3folddir)[0]+1]
               for i=1,nfc-1 do value = value+','+(['-','auto','+'])[(*self.view_u3folddir)[i]+1]
               dm_set_droplist,event.id,select=0,set_value=[value,'auto','+']
            endelse
            if (dm_to_string(*(self.view_u3folddir),separator=',') ne old) and ptr_valid(self.projStrPtr) then recalc_proj = 1b 
          end
        'calcprj':      if event.select then self->dm_calc_projection    ;calculate projection
        'volselx':      if self.volsel_x[0>(self.samp_typ)] ne tmp_value then begin
                           old = self.volsel_x[0>(self.samp_typ)]
                           self.volsel_x[0>(self.samp_typ)] = tmp_value
                           ;swap y,z or any thickness axis that has the same axis number
                           fts = ['from','to','step']
                           if self.volsel_y[0>(self.samp_typ)] eq tmp_value then begin
                              self.volsel_y[0>(self.samp_typ)] = old
                              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselY'),select=old
                              self->script,'set_parameter',name='vol'+['x'+fts,'y'+fts],value=[self.vol_yran,self.vol_ystep,self.vol_xran,self.vol_xstep]
                           endif else if self.volsel_z[0>(self.samp_typ)] eq tmp_value then begin
                              self.volsel_z[0>(self.samp_typ)] = old
                              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselZ'),select=old
                              self->script,'set_parameter',name='vol'+['x'+fts,'z'+fts],value=[self.vol_zran,self.vol_zstep,self.vol_xran,self.vol_xstep]
                           endif else begin
                              if self.samp_typ ge 0 then begin
                                 if (self.volsel_t1 eq tmp_value) and (((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn) then begin
                                    self.volsel_t1 = old
                                    dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T0'),select=old  
                                 endif
                              endif else if nitem ge 6 then begin
                                 ind = where((*self.volsel_t) eq tmp_value,count)
                                 for i=0,count-1 do begin
                                     (*self.volsel_t)[ind[i]] = old
                                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T'+dm_to_string(ind[i])),select=old
                                 endfor
                              endif
                           endelse
                        endif
        'volxfrom':     self.vol_xran[0]  = dm_to_number(tmp_value,/float)
        'volxto':       self.vol_xran[1]  = dm_to_number(tmp_value,/float)
        'volxstep':     self.vol_xstep    = dm_to_number(tmp_value,/float)
        'volsely':      if self.volsel_y[0>(self.samp_typ)] ne tmp_value then begin
                           old = self.volsel_y[0>(self.samp_typ)]
                           self.volsel_y[0>(self.samp_typ)] = tmp_value
                           ;swap x,z or any thickness axis that has the same axis number
                           fts = ['from','to','step']
                           if self.volsel_x[0>(self.samp_typ)] eq tmp_value then begin
                              self.volsel_x[0>(self.samp_typ)] = old
                              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselX'),select=old
                              self->script,'set_parameter',name='vol'+['x'+fts,'y'+fts],value=[self.vol_yran,self.vol_ystep,self.vol_xran,self.vol_xstep]
                           endif else if self.volsel_z[0>(self.samp_typ)] eq tmp_value then begin
                              self.volsel_z[0>(self.samp_typ)] = old
                              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselZ'),select=old
                              self->script,'set_parameter',name='vol'+['y'+fts,'z'+fts],value=[self.vol_zran,self.vol_zstep,self.vol_yran,self.vol_ystep]
                           endif else begin
                              if self.samp_typ ge 0 then begin
                                 if (self.volsel_t1 eq tmp_value) and (((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn) then begin
                                    self.volsel_t1 = old
                                    dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T0'),select=old  
                                 endif
                              endif else if nitem ge 6 then begin
                                 ind = where((*self.volsel_t) eq tmp_value,count)
                                 for i=0,count-1 do begin
                                     (*self.volsel_t)[ind[i]] = old
                                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T'+dm_to_string(ind[i])),select=old
                                 endfor
                              endif
                           endelse
                        endif
        'volyfrom':     self.vol_yran[0]  = dm_to_number(tmp_value,/float)
        'volyto':       self.vol_yran[1]  = dm_to_number(tmp_value,/float)
        'volystep':     self.vol_ystep    = dm_to_number(tmp_value,/float)
        'volselz':      if self.volsel_z[0>(self.samp_typ)] ne tmp_value then begin
                           old = self.volsel_z[0>(self.samp_typ)]
                           self.volsel_z[0>(self.samp_typ)] = tmp_value
                           ;swap x,y or any thickness axis that has the same axis number
                           fts = ['from','to','step']
                           if self.volsel_x[0>(self.samp_typ)] eq tmp_value then begin
                              self.volsel_x[0>(self.samp_typ)] = old
                              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselX'),select=old
                              self->script,'set_parameter',name='vol'+['x'+fts,'z'+fts],value=[self.vol_zran,self.vol_zstep,self.vol_xran,self.vol_xstep]
                           endif else if self.volsel_y[0>(self.samp_typ)] eq tmp_value then begin
                              self.volsel_y[0>(self.samp_typ)] = old
                              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselY'),select=old
                              self->script,'set_parameter',name='vol'+['y'+fts,'z'+fts],value=[self.vol_zran,self.vol_zstep,self.vol_yran,self.vol_ystep]
                           endif else begin
                              if self.samp_typ ge 0 then begin
                                 if (self.volsel_t1 eq tmp_value) and (((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn) then begin
                                    self.volsel_t1 = old
                                    dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T0'),select=old  
                                 endif
                              endif else if nitem ge 6 then begin
                                 ind = where((*self.volsel_t) eq tmp_value,count)
                                 for i=0,count-1 do begin
                                     (*self.volsel_t)[ind[i]] = old
                                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T'+dm_to_string(ind[i])),select=old
                                 endfor
                              endif
                           endelse
                        endif
        'volzfrom':     self.vol_zran[0]  = dm_to_number(tmp_value,/float)
        'volzto':       self.vol_zran[1]  = dm_to_number(tmp_value,/float)
        'volzstep':     self.vol_zstep    = dm_to_number(tmp_value,/float)
        'volsel_t':     if self.samp_typ eq -1 then $
                           (*self.volsel_t)[ind_sel] = tmp_value $
                        else $
                           self.volsel_t1 = tmp_value
        'volsel_from':  if self.samp_typ eq -1 then $
                           (*self.volsel_tft)[ind_sel,0] = dm_to_number(tmp_value,/float) $
                        else $
                           self.volsel_tft1[0] = dm_to_number(tmp_value,/float)
        'volsel_to':    if self.samp_typ eq -1 then $
                           (*self.volsel_tft)[ind_sel,1] = dm_to_number(tmp_value,/float) $
                        else $
                           self.volsel_tft1[1] = dm_to_number(tmp_value,/float)
        'volifrom':     self.vol_iran[0]    = dm_to_number(tmp_value,/float)
        'volito':       self.vol_iran[1]    = dm_to_number(tmp_value,/float)
        'volsmooth':    self.vol_smooth     = dm_to_number(tmp_value,/int)
        'volinterp':    self.vol_interp     = tmp_value
        'volcolor':     self.vol_color      = tmp_value
        'volplot':      if event.select then self->dm_plot_volume             ;plot volume
        'voloplot':     if event.select then self->dm_plot_volume,/overplot   ;oplot volume        
        'slicesel_x': if self.slice_x[0>(self.samp_typ)] ne tmp_value then begin
            old = self.slice_x[0>(self.samp_typ)]
            self.slice_x[0>(self.samp_typ)] = tmp_value
            ;swap y axis and any thickness axis that has the same axis number
            fts = ['from','to','step']
            if self.slice_y[0>(self.samp_typ)] eq tmp_value then begin
               self.slice_y[0>(self.samp_typ)] = old
               dm_set_droplist,widget_info(self.dispBas,find_by_uname='slicesel_y'),select=old
               self->script,'set_parameter',name='disp'+['x'+fts,'y'+fts],value=[self.slice_yran,self.slice_ystep,self.slice_xran,self.slice_xstep]
            endif else begin
               if self.samp_typ ge 0 then begin
                  if (self.slice_t1[self.samp_typ] eq tmp_value) and (widget_info(self.dispBas,find_by_uname='dispsel_T0') ne 0) then begin
                     self.slice_t1[self.samp_typ] = old
                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T0'),select=old
                  endif else if (self.slice_t2[self.samp_typ] eq tmp_value) and (widget_info(self.dispBas,find_by_uname='dispsel_T1') ne 0) then begin
                     self.slice_t2[self.samp_typ] = old
                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T1'),select=old
                  endif
               endif else if nitem ge 5 then begin
                  ind = where((*self.slice_t) eq tmp_value,count)
                  for i=0,count-1 do begin
                     (*self.slice_t)[ind[i]] = old
                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T'+dm_to_string(ind[i])),select=old
                  endfor
               endif
            endelse
          end
        'slicesel_y': if self.slice_y[0>(self.samp_typ)] ne tmp_value then begin
            old = self.slice_y[0>(self.samp_typ)]
            self.slice_y[0>(self.samp_typ)] = tmp_value
            ;swap x axis and any thickness axis that has the same axis number
            fts = ['from','to','step']
            if self.slice_x[0>(self.samp_typ)] eq tmp_value then begin
               self.slice_x[0>(self.samp_typ)] = old
               dm_set_droplist,widget_info(self.dispBas,find_by_uname='slicesel_x'),select=old
               self->script,'set_parameter',name='disp'+['x'+fts,'y'+fts],value=[self.slice_yran,self.slice_ystep,self.slice_xran,self.slice_xstep]
            endif else begin
               if self.samp_typ ge 0 then begin
                  if (self.slice_t1[self.samp_typ] eq tmp_value) and (widget_info(self.dispBas,find_by_uname='dispsel_T0') ne 0) then begin
                     self.slice_t1[self.samp_typ] = old
                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T0'),select=old
                  endif else if (self.slice_t2[self.samp_typ] eq tmp_value) and (widget_info(self.dispBas,find_by_uname='dispsel_T1') ne 0) then begin
                     self.slice_t2[self.samp_typ] = old
                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T1'),select=old
                  endif
               endif else if nitem ge 5 then begin
                  ind = where((*self.slice_t) eq tmp_value,count)
                  for i=0,count-1 do begin
                     (*self.slice_t)[ind[i]] = old
                     dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T'+dm_to_string(ind[i])),select=old
                  endfor
               endif
            endelse
          end
        'dispxfrom':    self.slice_xran[0]  = dm_to_number(tmp_value,/float)
        'dispxto':      self.slice_xran[1]  = dm_to_number(tmp_value,/float)
        'dispxstep':    self.slice_xstep    = dm_to_number(tmp_value,/float)
        'dispyfrom':    self.slice_yran[0]  = dm_to_number(tmp_value,/float)
        'dispyto':      self.slice_yran[1]  = dm_to_number(tmp_value,/float)
        'dispystep':    self.slice_ystep    = dm_to_number(tmp_value,/float)
        'dispifrom':    self.slice_iran[0]  = dm_to_number(tmp_value,/float)
        'dispito':      self.slice_iran[1]  = dm_to_number(tmp_value,/float)
        'dispsmooth':   self.slice_smooth   = dm_to_number(tmp_value,/int)
        'dispshading':  self.slice_shading  = tmp_value
        'dispcolor':    self.slice_color    = tmp_value
        'dispsel_t': begin
            if self.samp_typ ge 0 then begin
               case ind_sel of 
                 0: self.slice_t1[self.samp_typ] = tmp_value
                 1: self.slice_t2[self.samp_typ] = tmp_value
                 else: 
               endcase
            endif else begin
              (*self.slice_t)[ind_sel] = tmp_value 
            endelse
          end
        'dispsel_from': begin
            if self.samp_typ ge 0 then begin
               case ind_sel of 
                 0: self.slice_t1ran[0] = dm_to_number(tmp_value,/float)
                 1: self.slice_t2ran[0] = dm_to_number(tmp_value,/float)
                 else: 
               endcase
            endif else begin
               (*self.slice_tft)[ind_sel,0] = dm_to_number(tmp_value,/float)
            endelse
          end
        'dispsel_to': begin
            if self.samp_typ ge 0 then begin
               case ind_sel of 
                 0: self.slice_t1ran[1] = dm_to_number(tmp_value,/float)
                 1: self.slice_t2ran[1] = dm_to_number(tmp_value,/float)
                 else: 
               endcase
            endif else begin  
               (*self.slice_tft)[ind_sel,1] = dm_to_number(tmp_value,/float)
            endelse
          end
        'dispfitslice': if event.select then self->dm_to_fit,/slice
        'dispplotslice':if event.select then self->dm_plot_slice             ;plot slice
        'dispopltslice':if event.select then self->dm_plot_slice,/overplot   ;oplot slice
        'dispsurfslice':if event.select then self->dm_plot_slice,/surfplot   ;surf slice
        'cutsel_t': begin
            if self.samp_typ eq -1 then begin
               (*self.cutsel_t)[ind_sel] = tmp_value 
            endif else begin
               case ind_sel of
                 0: self.cut_t1[self.samp_typ] = tmp_value
                 1: self.cut_t2[self.samp_typ] = tmp_value
                 2: self.cut_t3[self.samp_typ] = tmp_value
                 else:
               endcase
            endelse
          end
        'cutsel_from': begin
            if self.samp_typ eq -1 then begin
               (*self.cutsel_tft)[ind_sel,0] = dm_to_number(tmp_value,/float)  
            endif else begin
               case ind_sel of
                  0: self.cut_t1ran[0] = dm_to_number(tmp_value,/float)
                  1: self.cut_t2ran[0] = dm_to_number(tmp_value,/float)
                  2: self.cut_t3ran[0] = dm_to_number(tmp_value,/float)
                  else:
               endcase
            endelse
          end
        'cutsel_to': begin
            if self.samp_typ eq -1 then begin
               (*self.cutsel_tft)[ind_sel,1] = dm_to_number(tmp_value,/float)
            endif else begin
               case ind_sel of
                  0: self.cut_t1ran[1] = dm_to_number(tmp_value,/float)
                  1: self.cut_t2ran[1] = dm_to_number(tmp_value,/float)
                  2: self.cut_t3ran[1] = dm_to_number(tmp_value,/float)
                  else:
               endcase 
            endelse
          end
        'cutx': if self.cut_x[0>(self.samp_typ)] ne tmp_value then begin
            old = self.cut_x[0>(self.samp_typ)]
            self.cut_x[0>(self.samp_typ)] = tmp_value
            ;swap any thickness axis that has the same axis number
            if self.samp_typ ge 0 then begin
               if self.cut_t1[self.samp_typ] eq tmp_value then begin
                  self.cut_t1[self.samp_typ] = old
                  dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T0'),select=old
               endif else if (self.cut_t2[self.samp_typ] eq tmp_value) and (widget_info(self.dispBas,find_by_uname='cutsel_T1') ne 0) then begin
                  self.cut_t2[self.samp_typ] = old
                  dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T1'),select=old
               endif else if (self.cut_t3[self.samp_typ] eq tmp_value) and (widget_info(self.dispBas,find_by_uname='cutsel_T2') ne 0) then begin
                  self.cut_t3[self.samp_typ] = old
                  dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T2'),select=old
               endif
            endif else if nitem ge 4 then begin
               ind = where((*self.cutsel_t) eq tmp_value,count)
               for i=0,count-1 do begin
                  (*self.cutsel_t)[ind[i]] = old
                  dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T'+dm_to_string(ind[i])),select=old
               endfor
            endif
          endif
        'cutfrom':   self.cut_ran[0]  = dm_to_number(tmp_value,/float)
        'cutto':     self.cut_ran[1]  = dm_to_number(tmp_value,/float)
        'cutstep':   self.cut_step    = dm_to_number(tmp_value,/float)
        'cuty':      self.cut_y[0>(self.samp_typ)] = tmp_value
        'cutyfrom':  self.cut_yran[0] = dm_to_number(tmp_value,/float)
        'cutyto':    self.cut_yran[1] = dm_to_number(tmp_value,/float)
        'cutsymbol': self.cut_symbol  = tmp_value
        'cutcolor':  begin
            self.cut_color = tmp_value
            if (*self.colors)[self.cut_color] eq 'user define' then begin
               define = dm_dialog_input(['r:','g:','b:'],title='Define color',default=self.coldefine,dialog_parent=self.tlb,xsize=80,/coltable,cancel=cancel)
               if keyword_set(cancel) then return
               define = 0>(define)<255
               if total(define-[255b,255b,255b]*self.pbgcol_bw) eq 0 then begin
                  ok = dialog_message('The specified color is the same as the background color. The '+(['white','black'])[self.pbgcol_bw]+' color is used instead.',dialog_parent=self.tlb,/info,/center)
                  define = 255-define
                  self.cut_color = 0
                  dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutColor'),select=self.cut_color
               endif
               self.coldefine = define
            endif
          end
        'cutline':    self.cut_line = tmp_value
        'cutfit':     if event.select then self->dm_to_fit
        'cutplot':    if event.select then self->dm_plot_cut
        'cutoplot':   if event.select then self->dm_plot_cut,/overplot
        'trajx':      if self.traj_x[self.samp_typ] ne tmp_value then begin
                         z2_yn = (self.extravaxis_yn and (self.instrname ne 'macs') and (self.samp_typ eq 1 or self.samp_typ eq 3))  ;flag for z2 axis
                         z1_yn = ((self.extravaxis_yn and (self.instrname ne 'wand')) or (self.samp_typ ne 2))                       ;flag for z1 axis
                         old = self.traj_x[self.samp_typ]
                         self.traj_x[self.samp_typ] = tmp_value
                         ;swap y and any thickness axis that has the same axis number
                         if self.traj_y[self.samp_typ] eq tmp_value then begin
                            self.traj_y[self.samp_typ] = old
                            dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajY'),select=old
                         endif else if z1_yn and self.traj_z1[self.samp_typ] eq tmp_value then begin
                            self.traj_z1[self.samp_typ] = old
                            dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajZ1'),select=old
                         endif else if z2_yn and self.traj_z2[self.samp_typ] eq tmp_value then begin
                            self.traj_z2[self.samp_typ] = old
                            dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajZ2'),select=old
                         endif
                      endif
        'trajxfrom':  self.traj_xran[0] = dm_to_number(tmp_value,/float)
        'trajxto':    self.traj_xran[1] = dm_to_number(tmp_value,/float)
        'trajy':      if self.traj_y[self.samp_typ] ne tmp_value then begin
                         z2_yn = (self.extravaxis_yn and (self.instrname ne 'macs') and (self.samp_typ eq 1 or self.samp_typ eq 3))  ;flag for z2 axis
                         z1_yn = ((self.extravaxis_yn and (self.instrname ne 'wand')) or (self.samp_typ ne 2))                       ;flag for z1 axis
                         old = self.traj_y[self.samp_typ]
                         self.traj_y[self.samp_typ] = tmp_value
                         ;swap x and any thickness axis that has the same axis number
                         if self.traj_x[self.samp_typ] eq tmp_value then begin
                            self.traj_x[self.samp_typ] = old
                            dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajX'),select=old
                         endif else if z1_yn and self.traj_z1[self.samp_typ] eq tmp_value then begin
                            self.traj_z1[self.samp_typ] = old
                            dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajZ1'),select=old
                         endif else if z2_yn and self.traj_z2[self.samp_typ] eq tmp_value then begin
                            self.traj_z2[self.samp_typ] = old
                            dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajZ2'),select=old
                         endif
                      endif
        'trajyfrom':  self.traj_yran[0] = dm_to_number(tmp_value,/float)
        'trajyto':    self.traj_yran[1] = dm_to_number(tmp_value,/float)
        'trajz1':     self.traj_z1[self.samp_typ] = tmp_value
        'trajz1from': self.traj_z1ran[0] = dm_to_number(tmp_value,/float)
        'trajz1to':   self.traj_z1ran[1] = dm_to_number(tmp_value,/float)
        'trajz2':     self.traj_z2[self.samp_typ] = tmp_value
        'trajz2from': self.traj_z2ran[0] = dm_to_number(tmp_value,/float)
        'trajz2to':   self.traj_z2ran[1] = dm_to_number(tmp_value,/float)
        'plottra':    if event.select then self->dm_plot_trajectory               ;plot trajectory
        'oplottra':   if event.select then self->dm_plot_trajectory,/overplot     ;overplot trajectory
        'clearcal':   if event.select then self->my_widget_control,'calcInput',self.dispBas,set_value=''
        'calccal':    if event.select then begin
            self->my_widget_control,'calcInput',self.dispBas,get_value=str
            space = '[ '+string(9b)+']'
            str = strtrim(str)
            while ((strlen(strtrim(str[n_elements(str)-1],2)) eq 0) or stregex(str[n_elements(str)-1],'^'+space+'*;',/boolean)) and (n_elements(str) gt 1) do $
                str=str[0:(0>(n_elements(str)-2))]  ;remove trailing empty strings and comment strings
            line = '' & n_str = n_elements(str)
            hpPtr = ptrarr(n_str)  ;store help and print results
            for i=0,n_str-1 do begin
                ptrexp0 = 'if ptr_valid(hpPtr['+dm_to_string(i)+']) then *(hpPtr['+dm_to_string(i)+'])=[*(hpPtr['+dm_to_string(i)+']),'
                ptrexp1 = '] else hpPtr['+dm_to_string(i)+']=ptr_new('
                tmp = strtrim(str[i],2)
                if (strlen(tmp) eq 0) or (strmid(tmp,0,1) eq ';') then continue
                ;remove trailing comments
                tmp1 = strpos(tmp,';')
                if tmp1 ge 0 then tmp = strmid(tmp,0,tmp1)
                if stregex(tmp,'^help'+space+'*,',/fold_case,/boolean) then begin
                   tmp=tmp+',output=dcs_mslice_calchelp & '+ptrexp0+'dcs_mslice_calchelp'+ptrexp1+'dcs_mslice_calchelp)'
                endif else begin
                   tmp2 = stregex(tmp,'^print'+space+'*,',/fold_case,length=len)
                   if len[0] gt 0 then begin
                      tmp = 'dcs_mslice_calcprint=strjoin(string('+strmid(tmp,len[0])+'),",")& '+ptrexp0+'dcs_mslice_calcprint'+ptrexp1+'dcs_mslice_calcprint)'
                   endif else if strmid(tmp,0,1,/reverse) eq '=' then begin
                      tmp = 'dcs_mslice_ycal='+strmid(tmp,0,strlen(tmp)-1)+'&ny=n_elements(dcs_mslice_ycal)&hpPtr['+dm_to_string(i)+$
                         ']=ptr_new("="+(ny gt 1?"[":"")+dm_to_string(dcs_mslice_ycal,separator=", ")+(ny gt 1?"]":""))'
                   endif else begin
                      tmp1 = tmp
                      repeat begin
                         tmp2 = stregex(tmp1,'(\(.*[=,]+.*\))',length=len)
                         if len[0] gt 0 then tmp1 = strmid(tmp1,0,tmp2[0])+strmid(tmp1,tmp2[0]+len[0],strlen(tmp1)-tmp2[0]-len[0])
                      endrep until len[0] le 0 
                      if (strpos(tmp1,'=') eq -1) and (strpos(tmp1,',') eq -1) and (strpos(tmp1,'->') eq -1) and (~stregex(tmp1,'^if',/fold_case,/boolean)) $
                         and (~stregex(tmp1,' +begin$',/fold_case,/boolean)) and (~stregex(tmp1,'^endrep',/fold_case,/boolean)) $
                         and (~stregex(tmp1,'^endif',/fold_case,/boolean)) and (~stregex(tmp1,'^endelse',/fold_case,/boolean)) $
                         and (~stregex(tmp1,'^endfor',/fold_case,/boolean)) and (~stregex(tmp1,'^end$',/fold_case,/boolean)) then begin
                         tmp = 'dcs_mslice_ycal='+tmp+'&ny=n_elements(dcs_mslice_ycal)&hpPtr['+dm_to_string(i)+$
                               ']=ptr_new("="+(ny gt 1?"[":"")+dm_to_string(dcs_mslice_ycal,separator=", ")+(ny gt 1?"]":""))'
                      endif
                   endelse
                endelse
                if strmid(line,0,1,/reverse) eq '$' then $  ;remove trailing $
                   line = strmid(line,0,strlen(line)-1)+tmp $             
                else $
                   line = line+'&'+tmp
            endfor
            if strlen(line) eq 0 then return
            ok = execute(line,1,1)
            if ok then begin
               ind_hp = where(ptr_valid(hpPtr),n_hp)
               for i=0,n_hp-1 do begin
                   if strmid((*(hpPtr[ind_hp[i]]))[0],0,1) eq '=' then begin
                      while strmid(str[ind_hp[i]],0,1,/reverse) eq '=' do str[ind_hp[i]] = strmid(str[ind_hp[i]],0,strlen(str[ind_hp[i]])-1)
                      str[ind_hp[i]] = '; '+str[ind_hp[i]]+*(hpPtr[ind_hp[i]])
                   endif else str = [str,'; '+*(hpPtr[ind_hp[i]])]
               endfor
               if n_hp gt 0 then self->my_widget_control,'calcInput',self.dispBas,set_value=str
            endif else ok = dialog_message(!error_state.msg,/error,dialog_parent=self.tlb,/center)
            ptr_free,hpPtr
          endif
        'status':  begin  ;context menu
            self->my_widget_control,'contextMenu',event.id,/destroy
            ;context sensitive menu
            menuid = widget_base(event.id,/CONTEXT_MENU,UNAME='contextMenu')
            case self.instrname of 
               'dcs':  begin
                       status = ['eief','int','userf','bin','bincs','extb','binth','montyp','bgrate','bgdet','bgtch','bgcan','bgtol','bgtolmd','van','mask','eint','ssfac','absp','eadj','edep','dcor','datr']
                       option = ['bitf','neg2th','2thc','2emp']
                       end
               'macs': begin
                       status = ['geom','eief','int','det','temp','doff','koff','moncnt','monlm2','userf','bin','bincs','extb','binth','bgcan','bgtol','bgtolmd','van','mask','a3mask','ssfac','absp','datr']
                       option = ['neg2th','lockluv','checkluv','2emp']
                       end
               'spe':  begin
                       status = ['geom','eief','int','userf','bin','bincs','extb','binth','bgrate','bgdet','bgtch','bgcan','bgtol','bgtolmd','van','mask','eint','ssfac','absp','eadj','edep','datr']
                       option = ['neg2th','2emp']
                       end
               'nxspe':begin
                       status = ['geom','eief','int','lamcor','userf','bin','bincs','extb','binth','bgrate','bgdet','bgtch','bgcan','bgtol','bgtolmd','van','mask','eint','ssfac','absp','eadj','edep','datr']
                       option = ['neg2th','2emp']
                       end
               'inx':  begin
                       status = ['geom','eief','int','userf','bin','bincs','extb','binth','bgrate','bgdet','bgtch','bgcan','bgtol','bgtolmd','van','mask','eint','ssfac','absp','eadj','edep','datr']
                       option = ['neg2th','2emp']
                       end
               'wand': begin
                       status = ['eief','doff','userf','bin','bincs','extb','binth','bgcan','bgtol','bgtolmd','van','mask','ssfac','absp','datr']
                       option = ['2thc','2emp']
                       end
               else:
            endcase
            if n_elements(status) ne 0 then status = 'status_'+status
            if n_elements(option) ne 0 then option = 'status_'+option
            for i=0,n_elements(status)-1 do void = dm_widget_button(menuid,_extra=self->getstatus(status[i],/init),uname=status[i])
            for i=0,n_elements(option)-1 do void = dm_widget_button(menuid,_extra=self->getstatus(option[i],/init),uname=option[i],separator=((i eq 0) and (n_elements(status) ne 0)))
            self->my_widget_control,'status_dcor',menuid,sensitive=((self.samp_typ ge 2) and (self.instrname eq 'dcs'))
            self->my_widget_control,'status_datr',menuid,sensitive=((self.samp_typ ge 2) or ((self.samp_typ ne 0) and (self.instrname eq 'macs')))
            self->my_widget_control,'status_'+['bgtol','bgtolmd'],menuid,sensitive=((self.samp_typ ge 2) or (self.instrname eq 'macs'))
            self->my_widget_control,'status_'+['montyp','bitf'],menuid,sensitive=(self.instrname eq 'dcs')
            self->my_widget_control,'status_2thc',menuid,sensitive=((self.instrname eq 'dcs') or (self.instrname eq 'wand'))
            self->my_widget_control,'status_doff',menuid,sensitive=((self.instrname eq 'wand') or (self.instrname eq 'macs'))
            self->my_widget_control,'status_lamcor',menuid,sensitive=(self.instrname eq 'nxspe')
            self->my_widget_control,'status_neg2th',menuid,sensitive=(self.samp_typ eq 0)
            self->my_widget_control,'status_'+['bgrate','bgdet','bgtch','eadj','edep'],menuid,sensitive=(self.instrname ne 'wand')
            self->my_widget_control,'status_temp',menuid,sensitive=((self.instrname eq 'macs') and self.extravaxis_yn)
            widget_displaycontextmenu,event.id,event.x,event.y,menuid
          end
        'status_geom':   ok = self->getstatus(strlowcase(uname),/help)
        'status_eief':   ok = self->getstatus(strlowcase(uname),/help)
        'status_userf':  ok = self->getstatus(strlowcase(uname),/help)
        'status_montyp': ok = self->getstatus(strlowcase(uname),/help)
        'status_moncnt': ok = self->getstatus(strlowcase(uname),/help)
        'status_monlm2': ok = self->getstatus(strlowcase(uname),/help)
        'status_bgrate': ok = self->getstatus(strlowcase(uname),/help)
        'status_bgtch':  ok = self->getstatus(strlowcase(uname),/help)
        'status_bgdet':  ok = self->getstatus(strlowcase(uname),/help)
        'status_bgcan':  ok = self->getstatus(strlowcase(uname),/help)
        'status_bgtol':  ok = self->getstatus(strlowcase(uname),/help)
        'status_bgtolmd':ok = self->getstatus(strlowcase(uname),/help)
        'status_van':    ok = self->getstatus(strlowcase(uname),/help)
        'status_a3mask': ok = self->getstatus(strlowcase(uname),/help)
        'status_mask':   ok = self->getstatus(strlowcase(uname),/help)
        'status_doff':   ok = self->getstatus(strlowcase(uname),/help)
        'status_int':    ok = self->getstatus(strlowcase(uname),/help)
        'status_det':    ok = self->getstatus(strlowcase(uname),/help)
        'status_temp':   ok = self->getstatus(strlowcase(uname),/help)
        'status_ssfac':  ok = self->getstatus(strlowcase(uname),/help)
        'status_eint':   ok = self->getstatus(strlowcase(uname),/help)
        'status_lamcor': ok = self->getstatus(strlowcase(uname),/help)
        'status_bin':    ok = self->getstatus(strlowcase(uname),/help)
        'status_bincs':  ok = self->getstatus(strlowcase(uname),/help)
        'status_extb':   ok = self->getstatus(strlowcase(uname),/help)
        'status_binth':  ok = self->getstatus(strlowcase(uname),/help)
        'status_absp':   ok = self->getstatus(strlowcase(uname),/help)
        'status_eadj':   ok = self->getstatus(strlowcase(uname),/help)
        'status_edep':   ok = self->getstatus(strlowcase(uname),/help)
        'status_dcor':   ok = self->getstatus(strlowcase(uname),/help)
        'status_datr':   ok = self->getstatus(strlowcase(uname),/help) 
        'status_koff':   begin
            new = dm_dialog_input('kidney offset angle:',title='Specify kidney offset angle',default=self.kidney_offset,/float,info=$
                  ['This value is the kidney hardware angle when software angle is 0.','It is only used for checking the kidney angle limit, not for adjusting',$
                   'detector 2theta angle.'],dialog_parent=self.tlb,cancel=cancel)
            if keyword_set(cancel) then return
            if finite(new) then self.kidney_offset = new
          end
        'status_bitf':   begin
            self.bitflip = ~self.bitflip
            ok = dialog_message('Bitflip error checking is '+(['off.','on.'])[self.bitflip],dialog_parent=self.tlb,/info,/center)
          end
        'status_neg2th': begin
            self.negative2th = ~self.negative2th
            ;need to recalculate projections if negative 2theta exists
            if ptr_valid(self.detPosPtr) then begin
               if (min((*self.detPosPtr).two_theta) lt 0) and ptr_valid(self.projStrPtr) then recalc_proj = 1b
            endif
            ok = dialog_message((['Absolute 2theta angle value is used.','Negative 2theta angle value is preserved.'])[self.negative2th],dialog_parent=self.tlb,/info,/center)
          end
        'status_2thc': begin
            if (self.instrname ne 'dcs') and (self.instrname ne 'wand') then return
            self.tthcorr = ~self.tthcorr
            ptr_free,self.detPosPtr
            case self.instrname of
                 'dcs':  self.detPosPtr = dm_load_detpos(/dcs, adjust2th=self.tthcorr) 
                 'wand': self.detPosPtr = dm_load_detpos(/wand,adjust2th=self.tthcorr)
                 else:
            endcase
            if ptr_valid(self.projStrPtr) then recalc_proj = 1b    ;need to recalculate projections
            ok = dialog_message('Detector 2theta correction is '+(['disabled.','enabled.'])[self.tthcorr],dialog_parent=self.tlb,/info,/center)
          end
        'status_2emp': begin
            self.extraempcan_yn = ~self.extraempcan_yn
            ptr_free,self.bgdata2
            self->reset_menu,/backgroundmenuonly  ;reset background menu
            ok = dialog_message([(['One set of empty can files are allowed.','Two sets of empty can files are allowed.'])[self.extraempcan_yn],$
                 '','Empty can subtraction will be:',(['I = Isample - ssfactor*Iemptycan,','I = Isample - ssfactor*(factor1*Iemptycan1 + factor2*Iemptycan2),'])[self.extraempcan_yn],$
                 'where ssfactor is the self-shielding factor.','',(['ssfactor','All factors'])[self.extraempcan_yn]+' can be changed from the Parameter menu.'],dialog_parent=self.tlb,$
                 /info,/center)
          end  
        'status_lockluv':begin
            self.locklattuv = ~self.locklattuv
            ok = dialog_message(['The lattice parameters and crystal orientation axes have been '+(['unlocked.','locked.'])[self.locklattuv],$
                 'Loading MACS files will '+(['','not '])[self.locklattuv]+'reset the lattice parameters and crystal', 'orientation axes.'],dialog_parent=self.tlb,/info,/center)
          end
        'status_checkluv':begin
            self.checklattuv = ~self.checklattuv
            ok = dialog_message(['Check flag has been '+(['unset.','set.'])[self.checklattuv],'Loading files will '+(['not ',''])[self.checklattuv]+$
                 'check for the consistency of lattice parameters and crystal orientation axes among files.'],dialog_parent=self.tlb,/info,/center)
          end
        else:begin
            passed = self->dm_filetools(uname,event=event)  ;check for file type and file tools events
            if (~ passed) then begin            ;view specified plot window
               all = self.gr_list->get(/all,count=count)
               for i=0,count-1 do begin
                   id = self->dm_get_id(all[i])
                   if id eq uname then begin
                      all[i]->totop
                      break
                   endif
               endfor
            endif
          end
    endcase
    if resetu1 then begin
       if (~ keyword_set(islabel)) then self->adjust_viewlabel,0
       if self.samp_typ gt 0 then self.traj_type[0] = dm_set_viewlabel(self.view_u1lab[1],self.view_u1)
    endif
    if resetu2 then begin
       if (~ keyword_set(islabel)) then self->adjust_viewlabel,1
       if self.samp_typ gt 0 then self.traj_type[1] = dm_set_viewlabel(self.view_u2lab[1],self.view_u2)
    endif
    if resetu3 then begin
       if (~ keyword_set(islabel)) then self->adjust_viewlabel,2
       self.traj_type[2] = dm_set_viewlabel(self.view_u3lab,self.view_u3)
    endif
    if resetu1 or resetu2 or resetu3 then begin
       self->reset_droplist
       self->reset_display,/reset
    endif
    if recalc_proj then self->set_display_button,/clearprojection ;need to recalculate projections
    if strlowcase(uname) ne 'calcprj' then self.error = 0b    ;reset the error flag   
end

pro dcs_mslice::eadjust,option
    old = self.eadjust
    self.eadjust = 0>(option)<2
    if self.eadjust eq 2 then begin
       new  = dm_dialog_input('specify the apparent elastic peak position:',title='Specify elastic peak position',default=self.eadjust_spec,/float,info='in meV',dialog_parent=self.tlb)
       if finite(new) then $
          self.eadjust_spec = new $
       else begin
          self.eadjust = 0 & old = 2
          ok = dialog_message('No elastic peak adjustment is specified.',dialog_parent=self.tlb,/error,/center)
       endelse
    endif
    if old eq self.eadjust then return
    self->my_menu_toggle,'epad'+['None','Auto','Spec'],self.eadjust 
    self->my_widget_control,'epadCheckfirst',sensitive=(self.eadjust ne 0)
    ;need to reload data
    if ptr_valid(self.dataStrPtr) then begin  ;clear data
       ok = dialog_message(dialog_parent=self.tlb,'Please reload data.',/info,/center)
       cid = widget_info(self.tlb,find_by_uname='clearDat') 
       if cid ne 0 then self->event,{WIDGET_BUTTON,ID:cid,TOP:self.tlb,HANDLER:self.tlb,SELECT:1} $
       else ptr_free,self.dataStrPtr
    endif
end

function dcs_mslice::getproperty,tlb=tlb,instrgeom=instrgeom,dataPtr=dataPtr,error=error
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(!error_state.msg,/error,dialog_parent=self.tlb,/center)
       error = 1
       return,0
    endif
    error = 0
    if keyword_set(tlb)       then return,self.tlb
    if keyword_set(instrgeom) then return,self.instrgeom
    if keyword_set(dataPtr)   then begin
       if self.samp_typ eq -1 then begin
          return,self.dataPtr 
       endif else begin
          error = 1
          return,ptr_new()
       endelse
    endif
    return,0
end

pro dcs_mslice::getproperty,tlb=tlb,instrgeom=instrgeom,dataPtr=dataPtr,error=error
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(!error_state.msg,/error,dialog_parent=self.tlb,/center)
       error = 1
       return
    endif
    error = 0
    if arg_present(tlb)       then tlb=self.tlb
    if arg_present(instrgeom) then instrgeom=self.instrgeom
    if arg_present(dataPtr)   then begin
       if self.samp_typ eq -1 then begin
          dataPtr = self.dataPtr 
       endif else begin
          error = 1
          dataPtr = ptr_new()
       endelse
    endif
end

;returns the root directory for saving dcs_mslice.prm
function dcs_mslice::getrootdir
    if self.idl_version ge 6.2 then dir = sourcepath() $
    else cd,current=dir
    if lmgr(/runtime) then begin
       defsysv,'!DAVE_DIR',exists=exists
       if exists then dir = !DAVE_DIR
    endif
    return,dir
end

;get the status string or keywords for context sensitive menu
function dcs_mslice::getstatus,name,help=help,init=init
    status   = ''
    if keyword_set(help) then begin
       case strlowcase(name) of
           'status_userf':  status = ['Go to "Option->'+(lmgr(/runtime)?'User Macro':'User Macro/Function/Procedure')+'" to change this setting.']
           'status_ssfac':  status = ['Go to "Parameter->Change Self-shielding Factor..." to change this setting.']
           'status_geom':   status = ['Go to "Option->Instrument Geometry" to change this setting.']
           'status_eief':   if self.instrname eq 'wand' then $
                            status = ['Go to "Parameter->Change WAND Incident Wavelength..." to change this setting.'] $
                            else $
                            status = ['fixed '+(['Ei','Ef'])[self.instrgeom]]
           'status_montyp': status = ['Go to "Option->Monitor Count->Monitor Choice" to change this setting.']
           'status_moncnt': status = ['Go to "Option->Monitor Count" to change this value.']
           'status_monlm2': status = ['Go to "Option->Monitor Count->Apply Monitor Lambda/2 Correction" to change this value.']
           'status_bgrate': status = ['Go to "Background->Set Overall Background Rate..." to change this value',' or',$
                                      'go to "Background->Load Dark Count File(s)" to change this value',' or',$
                                      'go to "Background->Clear Background" to clear all backgrounds.']
           'status_bgtch':  status = ['Go to "Background->Load Time Channel Background File(s)" to change this value',' or',$
                                      'go to "Background->Clear Background" to clear all backgrounds.']
           'status_bgdet':  status = ['Go to "Background->Load Detector Background File(s)" to change this value',' or',$
                                      'go to "Background->Clear Background" to clear all backgrounds.']
           'status_bgtol':  status = ['Go to "Parameter->Change Empty Can Subtraction Tolerance Value..." to change the tolerance value.']
           'status_bgtolmd':status = ['Go to "Option->Empty Can Subtraction Tolerance Method" to change this option.']
           'status_bgcan':  status = ['Go to "Background->Load Empty Can File(s)" to change this value',' or',$
                                      'go to "Background->Clear Background" to clear all backgrounds.']
           'status_van':    begin
                            if self.instrname eq 'macs' then $
                            status = ['Go to "Parameter->Change MACS Detector Efficiency..." to change the detector efficiencies,',' or',$
                                      'go to "Background->Load Vanadium File(s)" to calculate the detector efficiencies from data files,',' or',$
                                      'go to "Background->Clear Vanadium" to reset back to the default values.'] $  
                            else $
                            status = ['Go to "Background->Load Vanadium File(s)" to change this value',' or',$
                                      'go to "Background->Clear Vanadium" to clear it.']
                            
                            end
           'status_mask':   status = ['Go to Mask menu to change it',' or',$
                                      'go to "Mask->Clear '+(['','Detector'])[self.instrname eq 'macs']+'Mask" to clear the existing detector mask.']
           'status_a3mask': status = ['Go to "Mask->Set Mask->Mask A3 by Angle" to set it',' or',$
                                      'go to "Mask->Clear a3 Mask" to clear the existing A3 mask.']                          
           'status_doff':   status = ['Go to "Parameter->Change '+(*self.ftypename)[self.ftype]+' Detector Offset Angle..." to change this setting.']                        
           'status_int':    status = ['Go to "Option->View Intensity As" to change this setting.']
           'status_det':    status = ['Go to "Option->View Intensity As->MACS Detector Choice" to change this setting.'] 
           'status_temp':   status = ['Go to "Option->Viewing Axis->Temperature Choice" to change this setting.'] 
           'status_eint':   status = ['Go to "Option->View Intensity As" to change this setting.']
           'status_lamcor': begin
                            status = (['0: Constant time bins with no ki/kf correction.',$
                                       '1: Constant time bins with ki/kf correction.',$
                                       '2: Constant energy bins with no ki/kf correction.',$
                                       '3: Constant energy bins with ki/kf correction.'])
                            if ptr_valid(self.dataStrPtr) and (self.lamcorrnxspe ge 0) and (self.lamcorrnxspe le 3) then $
                            status = status[self.lamcorrnxspe] 
                            end
           'status_bin':    status = ['Go to "Option->Binning Method" to change this setting.']
           'status_bincs':  status = ['Go to "Option->Binning Method->Constant Intervals" to change this setting.']
           'status_extb':   status = ['Go to "Option->Binning Method->Use External Binning Library" to change this setting.']
           'status_binth':  status = ['Go to "Parameter->Change Binning Threshold..." to change this setting.']
           'status_eadj':   status = ['Go to "Option->Elastic Peak Position Adjustment" to change this setting.']
           'status_edep':   status = ['Go to "Option->E-Dependent Detector Efficiency Correction" to change this setting.']
           'status_dcor':   status = ['Go to "Option->Dark Angle Correction" to change this setting.']
           'status_datr':   status = ['Go to "Option->Show Dark Angle Trajectory" to change this setting.']
           'status_absp':   status = ['Go to "Option->Apsorption Correction" to change this setting.']
           else:
       endcase
       ok = dialog_message(dialog_parent=self.tlb,status,/info,/center)
       return,status
    endif
    if !version.os_family eq 'Windows' then sep=string(9b)+': ' else sep=': '
    case strlowcase(name) of
        'status_userf':   status = (lmgr(/runtime)?'user macro':'user macro/function/procedure')+sep+((strlen(self.userf_str) ne 0)?'yes':'none')
        'status_ssfac':   status = 'self-shielding factor'+sep+dm_to_string(self.ssfac)
        'status_geom':    status = 'instrument geometry'+sep+(['direct','inverse'])[self.instrgeom]
        'status_eief':    begin
                          status = 'fixed '+(['Ei','Ef'])[self.instrgeom]+sep+(finite(self.eief)?(dm_to_string(self.eief)+' meV'):('unknown'))
                          if finite(self.eief) and ((self.instrname eq 'dcs') or (self.instrname eq 'wand')) then begin
                             status = status+' ('+dm_to_string(sqrt(81.8042/self.eief),resolution=3)+' '+string('c5'XB)+')'
                          endif
                          end
        'status_montyp':  status = 'monitor type'+sep+(['FC0','BM1','Duration'])[self.dcs_mon]
        'status_moncnt':  status = 'monitor count'+sep+([finite(self.macs_mon[0])?dm_to_string(self.macs_mon[0],/exponent):'n/a',dm_to_string(self.macs_mon[1],/exponent)])[self.mon_sumfit] 
        'status_monlm2':  status = 'monitor lambda/2 correction'+sep+(['no','yes'])[self.macs_lamb2]
        'status_bgrate':  begin
                          status = 'dark count'+sep
                          if ptr_valid(self.bgratePtr) then begin
                             nvalue = n_elements(*self.bgratePtr)
                             if nvalue eq 1 then $
                                status = status+dm_to_string(*self.bgratePtr)+' cts/hr/det' $
                             else $
                                status = status+'File'
                          endif else $
                             status = status+'none'
                          end
        'status_bgtch':   status = 'time channel background'+sep+(ptr_valid(self.bgtchnPtr)?'yes':'none')
        'status_bgdet':   status = 'detector background'+sep+(ptr_valid(self.bgdetsPtr)?'yes':'none')
        'status_bgtol':   status = 'empty can subtraction tolerance value'+sep+(self.instrname eq 'macs'?dm_to_string(self.macs_tolerance[0:1+self.samp_typ],separator=', ')+$
                            (['/E last','/E first'])[self.macs_bgefirst]:dm_to_string(self.psi_tolerance))
        'status_bgtolmd': status = 'empty can subtraction tolerance method'+sep+(['closest or within tolerance','within tolerance of the closest'])[self.empt_tol_optn]
        'status_bgcan':   status = 'empty can'+sep+(ptr_valid(self.bgdata)?(ptr_valid(self.bgdata2)?'2 sets':'yes'):'none')
        'status_van':     begin
                          if self.instrname eq 'macs' then $
                          status = 'detector efficiency'+sep+(ptr_valid(self.eff1)?'vanadium file':'default') $
                          else $
                          status = 'detector efficiency'+sep+(ptr_valid(self.eff)?'yes':'none')
                          end
        'status_mask':    begin
                          status = 'detector mask'+sep+(ptr_valid(self.mask)?'yes':'none')
                          if (self.instrname eq 'dcs') and (total(self.dcsdetmask) ne 0) then begin
                             status = 'dectector mask'+sep
                             if self.dcsdetmask[0] then status = status+'neg,'
                             if self.dcsdetmask[4] then status = status+'upper,lower,' else begin
                                if self.dcsdetmask[1] then status = status+'lower,'
                                if self.dcsdetmask[3] then status = status+'upper'
                             endelse
                             if self.dcsdetmask[2] then status = status+'central,'
                             if strmid(status,0,1,/reverse) eq ',' then status=strmid(status,0,strlen(status)-1)
                          endif 
                          end
        'status_a3mask':  status = 'a3 mask'+sep+(ptr_valid(self.a3mask)?'yes':'none')     
        'status_doff':    begin
                          if self.instrname eq 'wand' then doff = -(*self.detPosPtr).two_theta[0] $
                          else if self.instrname eq 'macs' then doff = (*self.detPosPtr).two_theta[0]
                          if n_elements(doff) ne 0 then $
                             status = 'detector offset angle'+sep+dm_to_string(doff,resolution=3)+string('b0'XB) $
                          else $
                             status = 'detector offset angle'+sep+'none'
                          end
        'status_koff':    status = 'kidney offset angle'+sep+dm_to_string(self.kidney_offset,resolution=3)+string('b0'XB)
        'status_int':     status = 'intensity type'+sep+self.intn_typ_str[self.intn_typ]
        'status_det':     status = 'intensity detector'+sep+(['SPEC','DIFF','SPEC+DIFF'])[self.macsintn_typ]
        'status_temp':    status = 'temperature choice'+sep+(['sample','control','setpoint'])[self.macstemp_typ]+(self.macstempavg_yn?'/avg':'')
        'status_eint':    status = 'E-integrated intensity'+sep+(['no','yes'])[self.eint_yn]
        'status_lamcor':  status = 'NXSPE ki/kf correction'+sep+(ptr_valid(self.dataStrPtr)?dm_to_string(self.lamcorrnxspe):'none')
        'status_bin':     status = 'binning'+sep+(['average','sum'])[self.bin_avgsum]
        'status_bincs':   status = 'constant binning interval'+sep+(['no','yes'])[self.bin_conststep]
        'status_extb':    status = 'use external binning library'+sep+(['no','yes'])[self.bin_extlib]
        'status_binth':   status = 'binning threshold'+sep+dm_to_string(self.binthreshold) 
        'status_eadj':    begin
                          status = 'elastic peak adjustment'+sep+(['no','automatic','specified:'+dm_to_string(self.eadjust_spec)])[self.eadjust]
                          if self.eadjust ne 0 then status = status+'('+(['Check All','Check 1st Only'])[self.eadjust_checkfirst]+')'
                          end
        'status_edep':    status = 'E-dep efficiency correction'+sep+(['no','yes'])[self.edep_yn]
        'status_dcor':    status = 'dark angle correction'+sep+(['no','yes'])[self.dcor_yn]
        'status_datr':    status = 'dark angle trajectory'+sep+(['no','yes'])[self.datr_yn]
        'status_absp':    begin
                          status = 'absorption correction'+sep
                          if ptr_valid(self.absp_info) then begin
                             if (*self.absp_info)[0] ge 0 and (*self.absp_info)[0] lt n_elements(self.absp_npts) then $
                                status = status+(['cylinder','annulus','sphere','flat plate'])[(*self.absp_info)[0]] $
                             else $
                                status = status+'none'
                          endif else $
                             status = status+'none'
                          end
        'status_bitf':    status = 'Check Bitflip Error'+sep+(['no','yes'])[self.bitflip]               
        'status_neg2th':  status = 'Preserve Negative 2theta Angle Value'+sep+(['no','yes'])[self.negative2th]
        'status_2thc':    status = 'Detector 2theta Correction'+sep+(['no','yes'])[self.tthcorr]
        'status_2emp':    status = 'Allow Two Sets of Empty Can Files'+sep+(['no','yes'])[self.extraempcan_yn]
        'status_lockluv': status = 'Lock Lattice Parameters and Crystal Oreintation Axes'+sep+(['no','yes'])[self.locklattuv]
        'status_checkluv':status = 'Check Lattice Parameters and Crystal Oreintation Axes in Files'+sep+(['no','yes'])[self.checklattuv]
        else:
    endcase
    ;change the string to keywords
    if keyword_set(init) then $
       result = {value:status} $
    else $
       result = {set_value:status}
    if self.idl_version ge 5.6 then begin
       tmp = strsplit(status,':',/extract)
       tmp = strtrim(tmp[n_elements(tmp)-1],2)
       set_button = ((tmp ne 'no') and (tmp ne 'none') and (tmp ne 'n/a'))
       if strlowcase(name) eq 'status_eief'   then set_button = finite(self.eief)
       if strlowcase(name) eq 'status_ssfac'  then set_button = (self.ssfac ne 1.0)
       if strlowcase(name) eq 'status_binth'  then set_button = (self.binthreshold ne 0.0)
       if strlowcase(name) eq 'status_doff'   then set_button = ((*self.detPosPtr).two_theta[0] ne 0)
       if strlowcase(name) eq 'status_koff'   then set_button = (self.kidney_offset ne 0)
       if strlowcase(name) eq 'status_bgtol'  then set_button = (self.instrname eq 'macs'?(max(self.macs_tolerance[0:1+self.samp_typ]) gt 0):(self.psi_tolerance gt 0))
       result = create_struct(result,'set_button',set_button)
    endif
    return,result
end

;get the device symbol font in different operating systems
function dcs_mslice::getsymbolfont,fsize
    fonts=['symbol*',$
           '-adobe-symbol-medium-r-normal--14-140-75-75-p-85-adobe-fontspecific',$
           '-adobe-symbol-medium-r-normal--14-100-100-100-p-85-adobe-fontspecific']
    fonts[0] = fonts[0]+dm_to_string(fsize+2,/int)
    catch, myerror
    if myerror ne 0 then begin
       return,fonts[0]
    end
    result = fonts[0]
    switch !version.os_family of
        'Windows':break
        'unix':
        'MacOS':begin
                spawn,'xlsfonts',result,errresult,exit_status=error
                if error ne 0 then begin
                   old_win = !d.window
                   device,get_fontnames=result,set_font='*'    ;will pop up a window, reason for using xlsfonts first
                   if !d.window ne old_win then wdelete, !d.window
                endif
                nlist = n_elements(result)
                tmp   = bytarr(nlist)
                for i=0,nlist-1 do begin
                    tmp[i] = ((strpos(strlowcase(result[i]),'symbol'))<0)+1
                endfor
                index = where(tmp,count)
                if count ne 0 then f_list=result[index]
                nlist = n_elements(f_list)
                if nlist ne 0 then begin
                   tmp = intarr(nlist)
                   for i=0,nlist-1 do tmp[i]=dm_to_number((strsplit(f_list[i],'[A-Za-z-]',/regex,/extract))[0],/int)
                   void = min(abs(tmp-fsize+2),index)
                   fonts[1] = f_list[index[0]]
                endif
                result = fonts[1]
                break
                end
        else:
    endswitch
    return,result
end

;mask Bragg peaks
;method:   1. find the highest peak, fit it with a Gaussian, width w0
;          2. replace the peak with background value
;          3. repeat 1 & 2, if w<0.25w0 or w>4w0, not a valid peak(default)
;          4. false peak if peak intensity is less than 4% of the strongest peak, and less than 3 sigma of the fit(default)
;          5. stop when two consecutive false peaks
;          6. if the strongest peak is at the left edge, use the second one as the strongest peak
;parameters:
;   detsum:  [ndet] array, detector intensity
;   errsum:  [ndet] array, error bar
;   mask:    saving returned sorted unique mask, if mask is not empty when called, result will be combined to the existing mask
;keywords:
;   fit_fwhm: passing the fitted fwhm of the strongest peak, if existed when calling, use it as the fwhm of the strongest peak
;   fit_max:  passing the fitted peak intensity of the strongest peak, if existed when calling, use it as the peak intensity
pro dcs_mslice::maskbraggpeak,detsum,errsum,mask,fit_fwhm=fit_fwhm,fit_max=fit_max
    ;smooth data and remove background
    sm1 = 25 & sm2 = 3 & smrep = 10
    ndet = n_elements(detsum) & x = lindgen(ndet) & y = detsum
    for i=0,smrep do  y = (detsum)<(smooth(y,sm1,/nan,/edge_truncate))
    y = detsum-y & y = smooth(y,sm2,/nan,/edge_truncate)
    ;search for peaks        
    wlim0 = self.mbparm[0] & wlim1 = self.mbparm[1] & plim = self.mbparm[2] & slim = self.mbparm[3]
    false_peak = 0
    if n_elements(fit_fwhm) eq 0 then fit_fwhm = !values.f_nan
    if n_elements(fit_max) eq 0 then  fit_max  = !values.f_nan
    nrep = 0L & replim = ndet/2 ;in case of a dead loop
    while (false_peak lt 2) and (nrep lt replim) do begin   ;search for peaks
          nrep = nrep+1
          ;find the maximum
          ymax = max(y,ind_ymax,min=ymin,/nan) 
          if finite(fit_fwhm) then $
             fwhm = fit_fwhm $
          else $
             fwhm = 10<(ndet/20)
          yave = mean(y)
          ;find peak boundaries    
          ip_0 = where((y[0:ind_ymax] le (ymax-1./2*(ymax-ymin))) and (y[0:ind_ymax] ge (ymin+1./3*(yave-ymin))),count0)
          ip_1 = where((y[ind_ymax:ndet-1] le (ymax-1./2*(ymax-ymin))) and (y[ind_ymax:ndet-1] ge (ymin+1./3*(yave-ymin))),count1)
          if count0 ne 0 then begin
             if count1 ne 0 then begin
                if ~finite(fit_fwhm) then fwhm = ip_1[0]+(ind_ymax-ip_0[count0-1])
                ip_1 = (ind_ymax+ip_1[0]*4)<(ndet-1)
             endif else begin
                if ~finite(fit_fwhm) then fwhm = (ind_ymax-ip_0[count0-1])
                ip_1 = (ind_ymax+(ind_ymax-ip_0[count0-1])*4)<(ndet-1)
             endelse
             ip_0 = 0>(ind_ymax-(ind_ymax-ip_0[count0-1])*4)
          endif else begin
             if count1 ne 0 then begin
                if ~finite(fit_fwhm) then fwhm = (ip_1[0])
                ip_0 = 0>(ind_ymax-ip_1[0]*4)   
                ip_1 = (ind_ymax+ip_1[0]*4)<(ndet-1) 
             endif else begin
                ip_0 = 0
                ip_1 = ndet-1
             endelse
          endelse
          if (ip_1-ip_0 ge ndet-1) or (ip_1-ip_0 lt 4) then break
          ;fit the peak with a Gaussian
          estimates = [ymax,x[ind_ymax],fwhm/sqrt(8.*alog(2)),ymin] 
          result = gaussfit(x[ip_0:ip_1],y[ip_0:ip_1],params,estimates=estimates,measure_errors=errsum[ip_0:ip_1],nterms=4,sigma=sigma) 
          if ~finite(sigma[0]) then sigma[0] = 0  
          fwhm = abs(params[2])*sqrt(8.*alog(2))
          if (~ finite(fit_max) and (params[1] gt fwhm))  then fit_max  = params[0]
          if (~ finite(fit_fwhm) and (params[1] gt fwhm)) then fit_fwhm = fwhm
          ip_0 = (0>(fix(params[1]-fwhm*1.2,type=3))<(ndet-1))
          ip_1 = (0>(fix(params[1]+fwhm*1.2,type=3))<(ndet-1))
          if (ip_0 gt ip_1) or (ip_1-ip_0 ge ndet-1) then break
          if (ip_1-ip_0 lt 1) then begin
             false_peak = false_peak+1
          endif else begin
             if ~finite(fit_fwhm) or ((fwhm gt wlim0*fit_fwhm) and (fwhm lt wlim1*fit_fwhm)) then begin
                if ~finite(fit_max) or ((params[0] gt plim*fit_max) and (params[0] gt slim*sigma[0])) then begin
                   false_peak = 0   ;clear the false peak flag
                   if n_elements(mask) eq 0 then $
                      mask = ip_0+lindgen(ip_1-ip_0+1) $
                   else $
                      mask = [mask,ip_0+lindgen(ip_1-ip_0+1)]
                endif else $
                   false_peak = false_peak+1
             endif else $
                false_peak = false_peak+1
          endelse
          ;replace the peak with the background
          y[ip_0:ip_1] = ymin
    endwhile
    ;sort the mask
    if n_elements(mask) ne 0 then mask = mask[uniq(mask,sort(mask))]
end        

;dummy calls to the following IDL library functions
;   amoeba:    used once for NCNR summer school
pro dcs_mslice::my_dummycalls
    ;amoeba
    parm = [-0.42d,-0.05,-0.32,-0.26,0.07,-0.43,-0.13,0.02,-0.26,-0.15,0.02, 0.0, 0.0, 0.0, 0.08] & scale = 0.5d
    parm = amoeba(1d-7,function_name='dm_mnwo4eigen',nmax=1e6,p0=parm,scale=scale,ncalls=ncalls,function_value=err)
end

;toggle a group of menu buttons according to uname and a number
pro dcs_mslice::my_menu_toggle,uname,flag,base=base
    nuname = n_elements(uname)
    if (nuname eq 0) or (n_elements(flag)) eq 0 then return
    if (flag lt 0) or (flag ge nuname) then return
    if n_elements(base) eq 0 then base=self.tlb
    ids = lonarr(nuname) 
    for i=0,nuname-1 do ids[i] = widget_info(base,find_by_uname=uname[i])
    check = ids[flag]
    uncheck = ids[where(indgen(nuname) ne flag)]
    ind = where(check gt 0,count)
    if count eq 0 then tmp = temporary(check) else check = check[ind]
    ind = where(uncheck gt 0,count)
    if count eq 0 then tmp = temporary(uncheck) else uncheck = uncheck[ind]
    dm_toggle_menubut,check=check,uncheck=uncheck
end

;a widget_control use uname instead of id as identifier
pro dcs_mslice::my_widget_control,uname,base,_ref_extra=extra
    if n_elements(base) eq 0 then base=self.tlb
    if ~widget_info(base,/valid_id) then base=self.tlb
    if ~widget_info(base,/valid_id) then return
    if n_elements(extra) ne 0 then begin
       for i=0,n_elements(uname)-1 do begin
           wid = widget_info(base,find_by_uname=uname[i])
           if wid ne 0 then widget_control,wid,_extra=extra
       endfor
    endif
end

;widget_tex with tab_mode=1,/editable,/all_events,/kbrd_focus_events,ysize=1
function dcs_mslice::my_widget_text,base,value=value,xsize=xsize,scr_xsize=scr_xsize,uname=uname
    return,widget_text(base,value=value,xsize=xsize,scr_xsize=scr_xsize,uname=uname,tab_mode=1,/editable,/all_events,/kbrd_focus_events,ysize=1)
end

;check if both projection and qty data are available when nomessage keyword is set;
;check if general volume data or both projection and qty data are available when nomessage keyword is not set
;      if not available, give a warning
;returns 0 if not available, 1 if available
function dcs_mslice::pd_valid,nomessage=nomessage
    pd_valid = 0b
    if ptr_valid(self.projStrPtr) then begin
       if n_elements((*self.projStrPtr).qty) gt 1 then pd_valid=1b
    endif
    if keyword_set(nomessage) then return, pd_valid
    pd_valid = (ptr_valid(self.dataPtr)) or (pd_valid and (~self.error))
    if (~pd_valid) then ok = dialog_message((['No data.','Please load data and calculate projections first.'])[self.samp_typ ge 0],dialog_parent=self.tlb,/center)
    return, pd_valid
end


;plot and set mask
;parameter:
;   detsum: [ndet], sum of the intensity at each detector
;   blist:  [ndet], non 0 elements are the index numbers of the masked detectors
;keyword:
;   angle:  if set, plot vs angle instead of vs det number
;   det_tt: angle set, det_tt is the two theta angle
;   exist:  whether the mask pointer exist before
pro dcs_mslice::plot_mask,detsum,blist,angle=angle,det_tt=det_tt,exist=exist
    pblist = ptr_new(blist,/no_copy)
    ;help string
    helptext = [    $
	  "Use the mouse left button to zoom, the right button to mask/unmask. ", $
		"If ''XZOOM'' is displayed on a button in the top row, zooming only ", $
		"occurs in x. If ''XYZOOM'' is displayed zooming occurs in x and y. ", $
		"Click on the ''XZOOM/XYZOOM'' button to toggle between these states. ", $
		"If ''MASK'' is displayed on the other button in the top row, the right ", $
		"button is used to mask one or more detectors. If ''UNMASK'' is displayed ", $
		"the right button unmasks detectors. ", $
		"Click on the ''MASK/UNMASK'' button to toggle between these states. ", $
		"The sliders may also be used to define the displayed detector range. ", $
		"The most recent zoom may be ''undone'' using the ''Undo zoom'' button. ", $
		"To return to the previous user interface click the ''PROCEED'' button."]
    if keyword_set(angle) then $
       dcs_plotdetmask_2th,self.tlb,'',detsum,pblist,helptext,angle=det_tt $
    else $
       dcs_plotdetmask,self.tlb,'',detsum,pblist,helptext
    ind = where(*pblist gt 0,count)
    if ptr_valid(self.mask) then begin
       exist = 1b
       self->clear_mask    ;clear the mask
    endif
    if count ne 0 then begin
       self.mask = ptr_new((*pblist)[ind]-1)
       self->my_widget_control,['mksvBut','mkclBut'],/sensitive
       if ptr_valid(self.projStrPtr) then self->set_display_button,/clearprojection ;need to recalculate projections
    endif else if keyword_set(exist) then $
       ok = dialog_message('Previously set detector mask has been cleared.',dialog_parent=self.tlb,/center)
    ptr_free,pblist
end

;construct a range string
function dcs_mslice::range_string,in_str,name,range,unit
    out_str = strtrim(in_str,2)
    if n_params() lt 3 then return,out_str
    if strlen(out_str) eq 0 then sep = '' else sep = ', '
    rangfin = finite(range)
    if total(rangfin) ne 0 then begin
       if total(rangfin) eq 2 then begin
          out_str = out_str+sep+name+'=['+dm_to_string(range[0])+','+dm_to_string(range[1])+']'
       endif else begin
          if rangfin[0] then $
             out_str = out_str+sep+name+'\geq'+dm_to_string(range[0]) $
          else $
             out_str = out_str+sep+name+'\leq'+dm_to_string(range[1])   
       endelse
       if n_elements(unit) ne 0 then begin
          tmp = strtrim(unit[0],2)
          if strlen(tmp) gt 0 then out_str = out_str+' '+tmp 
       endif
    endif
    return,out_str 
end

;reset droplist values according to the labels
pro dcs_mslice::reset_droplist
    if self.samp_typ gt 0 then begin
       self.traj_type[0] = dm_set_viewlabel(self.view_u1lab[1],self.view_u1)
       self.traj_type[1] = dm_set_viewlabel(self.view_u2lab[1],self.view_u2)
       self.traj_type[2] = dm_set_viewlabel(self.view_u3lab,self.view_u3)
    endif
end

;reset intensity label, for macs instrument, change the monitor count accordingly in the intensity label
pro dcs_mslice::reset_intnlab
    if self.instrname eq 'macs' then begin
       tmp = stregex(self.intn_lab,'(.+)[ '+string(9b)+']+\(.+\)',/subexpr,/extract)
       if strlen(tmp[1]) gt 0 then intn_lab = tmp[1] else intn_lab = 'Intensity'
       if self.intn_typ eq n_elements(self.intn_typ_str)-1 then begin
          self.intn_lab = intn_lab+' (counts'+(finite(self.macs_mon[self.mon_sumfit])?'/mon='+dm_to_string(self.macs_mon[self.mon_sumfit],/exponent):'')+')' 
       endif else begin
          if finite(self.macs_mon[self.mon_sumfit]) then begin
             self.intn_lab = intn_lab+' (mon='+dm_to_string(self.macs_mon[self.mon_sumfit],/exponent)+')'
          endif else begin
             self.intn_lab = intn_lab+' (arb. units)'
          endelse
       endelse
    endif
    self->my_widget_control,'intnLab',self.fileBas,set_value=self.intn_lab
end

;(re)create all the data file widgets
pro dcs_mslice::reset_file
    if (~ obj_valid(self.fileSel)) then begin ;create the widgets
       self.fileBas = widget_base(self.tlb,/row,/frame,ypad=1,xpad=0,space=0,map=0)
       widget_control,self.fileBas,update=0
       if !version.os_family eq 'Windows' then begin
          xsize = 280 & ysize = 9
       endif else begin
          xsize = 300 & ysize = 11 
       endelse
       self.fileSel = obj_new('dm_filesel',parent=self.fileBas,pobj=self,filter='.'+(*self.ftypeext)[self.ftype],xsize=xsize,ysize=ysize,ypad=0,group_leader=self.tlb,path=self.dirs[0],$
                      /ncnrftp,bufferdir=self.dirs[2])
       fileCol   = widget_base(self.fileBas,/col,/align_bottom,ypad=0,xpad=0,space=0,/context_events)
       sampleRow = widget_base(fileCol,/row,ypad=0,xpad=0,space=0)
       samplab   = widget_label(sampleRow,value='Sample Type',/align_center,uname='sampLab')
       geom      = widget_info(samplab,/geometry)
       if strlen(self.symfont) eq 0 then self.symfont=self->getsymbolfont(2*(fix(geom.scr_ysize)/2))
       sampTyp   = dm_widget_button(sampleRow,value='Powder',/menu,xsize=168,uname='sampTyp')
       if self.idl_version ge 5.6 then widget_control,sampTyp,tooltip='Press this button to select a sample type.'
       powdBut   = dm_widget_button(sampTyp,value='Powder',uname='powdTyp')
       crystTyp  = dm_widget_button(sampTyp,value='Single Crystal',uname='crystTyp',/menu)
       sct1But   = dm_widget_button(crystTyp,value='Single Crystal',uname='crystTyp1')
       sct2But   = dm_widget_button(crystTyp,value='Diffuse Scattering Single E',uname='crystTyp2')
       sct3But   = dm_widget_button(crystTyp,value='Diffuse Scattering Multiple E',uname='crystTyp3')
       ebinRow   = widget_base(fileCol,/row,ypad=0,xpad=0,space=0)
       ebinLab   = widget_label(ebinRow,value='E rebin size',/align_center,uname='ebinLab')
       eBin      = dm_widget_droplist(ebinRow,xsize=170,uname='eBin')
       labelRow  = widget_base(fileCol,/row,ypad=0,xpad=0,space=0)
       intn_lab  = widget_label(labelRow,value='Intensity Label',/align_center)
       intnLab   = self->my_widget_text(labelRow,value='',uname='intnLab')
       labelRow  = widget_base(fileCol,/row,ypad=0,xpad=0,space=0)
       titl_lab  = widget_label(labelRow,value='Title Label',/align_center)
       titlLab   = self->my_widget_text(labelRow,value='',uname='titlLab')
       labelRow  = widget_base(fileCol,/row,ypad=0,xpad=0,space=0)
       legd_lab  = widget_label(labelRow,value='Legend Label',/align_center)
       legdLab   = self->my_widget_text(labelRow,value='',uname='legdLab')
       loadRow   = widget_base(fileCol,/row,ypad=0,xpad=0,space=0,/align_right)
       loadBut   = widget_button(loadRow,value='Load Data',uname='loadBut',xsize=self.xsize[4],/pushbutton_events)
       if self.idl_version ge 5.6 then widget_control,loadBut,tooltip='Select the files first on the left then press this button.'
       addBut    = widget_button(loadRow,value='Add Data',uname='addBut',sensitive=0,xsize=self.xsize[4],/pushbutton_events)
       geom      = widget_info(intn_lab,/geometry)
       xsize     = geom.xsize+40
       widget_control,samplab,xsize=xsize
       widget_control,intn_lab,xsize=xsize
       widget_control,titl_lab,xsize=xsize
       widget_control,legd_lab,xsize=xsize
       widget_control,ebinLab,xsize=xsize
       geom      = widget_info(loadRow,/geometry)
       widget_control,sampTyp,scr_xsize=geom.xsize
       widget_control,eBin,   scr_xsize=geom.xsize
       widget_control,intnLab,scr_xsize=geom.xsize 
       widget_control,titlLab,scr_xsize=geom.xsize
       widget_control,legdLab,scr_xsize=geom.xsize
       geom      = widget_info(self.fileBas,/geometry)
       self.xsize[0] = geom.scr_xsize & self.xsize[1] = (self.xsize[0])/12-([3,2])[!version.os_family eq 'Windows'] 
       row       = widget_base(fileCol,/row,xpad=0,ypad=0,/align_right,space=0)
       clearDat  = widget_button(row,value='Clear Data',uname='clearDat',sensitive=0,xsize=self.xsize[4]*2,/pushbutton_events)
       widget_control,self.fileBas,/update,/map
       self.xsize[4] = 84-2*(!version.os_family eq 'Windows')
    endif
    ;reset all values
    tmp = self.samp_typ & self.samp_typ = -1
    self->change_samptyp,tmp,/init
    if self.samp_typ ge 2 then begin    ;diffuse scattering
       self->my_widget_control,'ebinLab',self.fileBas,set_value='Energy Range'
       self->my_widget_control,'eBin',self.fileBas,set_value=['All','Elastic Peak','Specify']
       tmp = self.e_bin[self.samp_typ]
       if tmp gt 2 then begin
          tmp = 1
          self.e_bin[self.samp_typ] = 1    ;default elastic peak
       end
    endif else begin
       self->my_widget_control,'ebinLab',self.fileBas,set_value='E Rebin Size'
       self->my_widget_control,'eBin',self.fileBas,set_value=dm_to_string(*(self.rebinsize))
       tmp = where(*(self.rebinsize) eq self.e_bin[self.samp_typ],count)
       if count eq 0 then self.e_bin[self.samp_typ]=1
       tmp = (0>(tmp[0]))
    endelse
    dm_set_droplist,widget_info(self.tlb,find_by_uname='eBin'),select=tmp
    self->reset_intnlab
    self->my_widget_control,'titlLab',self.fileBas,set_value=self.titl_lab
    self->my_widget_control,'legdLab',self.fileBas,set_value=self.legd_lab
end

;(re)create and set values of all the widgets in self.projBas
;keyword:
;   reset:    if set, only set the existing widget values, otherwise regenerate all the widgets
;   foldonly: if set, only set the existing foldding related values
pro dcs_mslice::reset_projection,reset=reset,foldonly=foldonly
    if (~ keyword_set(reset)) then begin ;(re)generate all the widgets
       if ~widget_info(self.projBas,/valid_id) then begin
          self.projBas = widget_base(self.tlb,/frame,xpad=0,ypad=0,map=0,space=0,xsize=self.xsize[0])
       endif
       widget_control,self.projBas,update=0
       if ~self.view[0] then return
       extra = widget_base(self.projBas,/col,ypad=1,xpad=0)
       switch self.samp_typ of
         0:begin ;powder
           row    = widget_base(extra,/row,ypad=0,xpad=0,space=0)
           left   = widget_base(row,/col,xpad=0,ypad=0)
           label  = widget_label(left,value='  Viewing Axes:',/align_left,uname='viewAxis',/dynamic_resize)
           row    = widget_base(left,/row,ypad=0,xpad=0,space=0)
           label  = widget_label(row,value='',xsize=self.xsize[1]+self.xsize[2])
           label2 = widget_label(row,value='Label',/align_center)
           row    = widget_base(left,/row,ypad=0,xpad=0)
           label  = widget_label(row,value='u1',xsize=self.xsize[1],/align_center)
           viewU1typ = dm_widget_droplist(row,value=self.view_type,xsize=self.xsize[2],uname='viewU1typ')
           viewU1lab = self->my_widget_text(row,value='',xsize=self.xsize[3],uname='viewU1lab')
           geom   = widget_info(viewU1lab,/geometry)
           widget_control,label2,xsize=geom.scr_xsize
           row    = widget_base(left,/row,ypad=0,xpad=0)
           label  = widget_label(row,value='u2',xsize=self.xsize[1],/align_center)
           viewU2typ = dm_widget_droplist(row,value=self.view_type,xsize=self.xsize[2],uname='viewU2typ')
           viewU2lab = self->my_widget_text(row,value='',xsize=self.xsize[3],uname='viewU2lab')
           break
           end
         1:         ;single crystal
         2:         ;diffuse scattering single E
         3:begin    ;diffuse scattering multiple E
           toprow  = widget_base(extra,/row,ypad=0,xpad=0,space=0)
           left    = widget_base(toprow,/col,ypad=0,xpad=0,tab_mode=1)
           row     = widget_base(left,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='  Unit Cell Lattice Parameters:')
           row     = widget_base(left,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='a ('+string('c5'XB)+')',xsize=self.xsize[1],/align_center)
           lattA   = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='lattA')
           label   = widget_label(row,value='b ('+string('c5'XB)+')',xsize=self.xsize[1],/align_center)
           lattB   = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='lattB')
           label   = widget_label(row,value='c ('+string('c5'XB)+')',xsize=self.xsize[1],/align_center)
           lattC   = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='lattC')
           row     = widget_base(left,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='a ('+string('B0'XB)+')',xsize=self.xsize[1],/align_center,font=self.symfont)
           lattAA  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='lattAA')
           label   = widget_label(row,value='b ('+string('B0'XB)+')',xsize=self.xsize[1],/align_center,font=self.symfont)
           lattBB  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='lattBB')
           label   = widget_label(row,value='g ('+string('B0'XB)+')',xsize=self.xsize[1],/align_center,font=self.symfont)
           lattCC  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='lattCC')
           geom    = widget_info(left,/geometry)
           xsize   = self.xsize[0]-geom.xsize
           right   = widget_base(toprow,/col,ypad=0,xpad=0,xsize=xsize,tab_mode=1)
           row     = widget_base(right,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='  Crystal Orientation:')
           row     = widget_base(right,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='ux',xsize=self.xsize[1],/align_center)
           orieUx  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='orieUx')
           label   = widget_label(row,value='uy',xsize=self.xsize[1],/align_center)
           orieUy  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='orieUy')
           label   = widget_label(row,value='uz',xsize=self.xsize[1],/align_center)
           orieUz  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='orieUz')
           row     = widget_base(right,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='vx',xsize=self.xsize[1],/align_center)
           orieVx  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='orieVx')
           label   = widget_label(row,value='vy',xsize=self.xsize[1],/align_center)
           orieVy  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='orieVy')
           label   = widget_label(row,value='vz',xsize=self.xsize[1],/align_center)
           orieVz  = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='orieVz')
           row     = widget_base(right,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='Psi',/align_center,xsize=self.xsize[1])
           oriePsi = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='oriePsi')
           oriepsiLab = widget_label(row,value='   <- angle of ki^u',uname='oriepsiLab',/dynamic_resize)   ; positive if (ki x u)||Qz (up)
           row     = widget_base(extra,/row,ypad=0,xpad=0,space=0)
           left    = widget_base(row,/col,xpad=0,ypad=0,tab_mode=1)
           label   = widget_label(left,value='  Orthogonal Viewing Axes:',/align_left,uname='viewAxis',/dynamic_resize)
           row     = widget_base(left,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='',scr_xsize=self.xsize[1],/align_center)
           label   = widget_label(row,value='a*',scr_xsize=self.xsize[1],/align_center)
           label   = widget_label(row,value='b*',scr_xsize=self.xsize[1],/align_center)
           label   = widget_label(row,value='c*',scR_xsize=self.xsize[1],/align_center)
           label   = widget_label(row,value='Label',scr_xsize=self.xsize[1]+1,/align_center)
           fold    = widget_label(row,value='Fold',scr_xsize=self.xsize[1]-1,/align_center)
           label   = widget_label(row,value='Center',scr_xsize=self.xsize[1],/align_center,uname='viewFoldcenter')
           label   = widget_label(row,value='Direction',scr_xsize=self.xsize[1]+5,/align_center,uname='viewFolddir')
           row     = widget_base(left,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='u1',scr_xsize=self.xsize[1],/align_center)
           viewU1h = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU1h')
           viewU1k = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU1k')
           viewU1l = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU1l')
           viewU1lab = self->my_widget_text(row,value='',scr_xsize=self.xsize[1]+1,uname='viewU1lab')
           tmp       = widget_label(row,value='')
           exclbas   = widget_base(row,/row,xpad=0,ypad=0,space=0,/nonexclusive)
           viewU1fd  = widget_button(exclbas,value=(!version.os_family eq 'Windows')?'':' ',uname='viewU1Fold',/align_center)
           geom      = widget_info(viewU1fd,/geom)
           if !version.os_family eq 'Windows' then $
              widget_control,tmp,scr_xsize=0>(self.xsize[1]-geom.scr_xsize-4) $
           else $
              widget_control,fold,scr_xsize=(self.xsize[1]+geom.scr_xsize)/2
           viewU1fcn = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU1Foldcenter')
           viewU1fdr = dm_widget_droplist(row,value=['-','auto','+'],xsize=self.xsize[1]+([20,5])[!version.os_family eq 'Windows'],uname='viewU1Folddir',select=(*(self.view_u1folddir))[0]+1)
           row     = widget_base(left,/row,ypad=0,xpad=0)
           label   = widget_label(row,value='u2',scr_xsize=self.xsize[1],/align_center)
           viewU2h = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU2h')
           viewU2k = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU2k')
           viewU2l = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU2l')
           viewU2lab = self->my_widget_text(row,value='',scr_xsize=self.xsize[1]+1,uname='viewU2lab')
           tmp       = widget_label(row,value='')
           exclbas   = widget_base(row,/row,xpad=0,ypad=0,space=0,/nonexclusive)
           viewU2fd  = widget_button(exclbas,value=(!version.os_family eq 'Windows')?'':' ',uname='viewU2Fold',/align_center)
           if !version.os_family eq 'Windows' then widget_control,tmp,scr_xsize=0>(self.xsize[1]-geom.scr_xsize-4)
           viewU2fcn = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU2Foldcenter')
           viewU2fdr = dm_widget_droplist(row,value=['-','auto','+'],xsize=self.xsize[1]+([20,5])[!version.os_family eq 'Windows'],uname='viewU2Folddir',select=(*(self.view_u2folddir))[0]+1)
           if (self.extravaxis_yn) and (self.instrname ne 'macs') and (self.instrname ne 'wand') then begin
              row     = widget_base(left,/row,ypad=0,xpad=0)
              label   = widget_label(row,value='u3',scr_xsize=self.xsize[1],/align_center)
              viewU3h = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU3h')
              viewU3k = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU3k')
              viewU3l = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU3l')
              viewU3lab = self->my_widget_text(row,value='',scr_xsize=self.xsize[1]+1,uname='viewU3lab')
              tmp       = widget_label(row,value='')
              exclbas   = widget_base(row,/row,xpad=0,ypad=0,space=0,/nonexclusive)
              viewU3fd  = widget_button(exclbas,value=(!version.os_family eq 'Windows')?'':' ',uname='viewU3Fold',/align_center)
              if !version.os_family eq 'Windows' then widget_control,tmp,scr_xsize=0>(self.xsize[1]-geom.scr_xsize-4)
              viewU3fcn = self->my_widget_text(row,value='',scr_xsize=self.xsize[1],uname='viewU3Foldcenter')
              viewU3fdr = dm_widget_droplist(row,value=['-','auto','+'],xsize=self.xsize[1]+([20,5])[!version.os_family eq 'Windows'],uname='viewU3Folddir',select=(*(self.view_u3folddir))[0]+1)
           endif
           break
           end
         else:
       endswitch
       right = widget_base(row,/col,xpad=0,ypad=0,/align_right)
       geom  = widget_info(left,/geom)
       widget_control,right,scr_xsize=self.xsize[0]-geom.xsize-1
       row   = widget_base(right,/row,xpad=0,ypad=0,/align_right,space=0)
       calcPrj = widget_button(row,value='Calculate Projections',uname='calcPrj',/pushbutton_events,xsize=((self.xsize[4]*2)<(self.xsize[0]-geom.xsize-1)),/align_center)
       if self.idl_version ge 5.6 then begin  ;tips for some widget buttons
          widget_control,calcPrj,tooltip='Calculate the projection before any plots can be made.'
       endif
    endif
    ;now set all the values
    if ~widget_info(self.projBas,/valid_id) then return
    extravaxis = ((self.extravaxis_yn) and (self.instrname ne 'macs') and (self.instrname ne 'wand'))
    ;folding first, single crystal only
    if self.samp_typ gt 0 then begin
       self->my_widget_control,'viewFoldcenter',self.projBas,set_value=(['Center','n-fold'])[self.fold_type],$
          sensitive=(self.view_u1fold or ((self.fold_type eq 0) and (self.view_u2fold or (extravaxis and self.view_u3fold))))
       self->my_widget_control,'viewFolddir',self.projBas,sensitive=((self.fold_type eq 0) and (self.view_u1fold or self.view_u2fold or (extravaxis and self.view_u3fold)))   
       self->my_widget_control,'viewU1Fold',self.projBas,set_button=self.view_u1fold
       self->my_widget_control,'viewU1Foldcenter',self.projBas,set_value=dm_to_string(*self.view_u1foldcenter,separator=','),sensitive=self.view_u1fold
       nfd = n_elements(*self.view_u1folddir)
       if nfd le 1 then begin
          select = (*self.view_u1folddir)[0]+1
          value  = ['-','auto','+']
       endif else begin
          select = 0
          value = (['-','auto','+'])[(*self.view_u1folddir)[0]+1]
          for i=1,nfd-1 do value = value+','+(['-','auto','+'])[(*self.view_u1folddir)[i]+1]
       endelse 
       dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU1Folddir'),select=select,set_value=value,sensitive=((self.fold_type eq 0) and self.view_u1fold) 
       self->my_widget_control,'viewU2Fold',self.projBas,set_button=self.view_u2fold,sensitive=(self.fold_type eq 0)
       self->my_widget_control,'viewU2Foldcenter',self.projBas,set_value=dm_to_string(*(self.view_u2foldcenter),separator=','),sensitive=((self.fold_type eq 0) and self.view_u2fold)
       nfd = n_elements(*self.view_u2folddir)
       if nfd le 1 then begin
          select = (*self.view_u2folddir)[0]+1
          value  = ['-','auto','+']
       endif else begin
          select = 0
          value = (['-','auto','+'])[(*self.view_u2folddir)[0]+1]
          for i=1,nfd-1 do value = value+','+(['-','auto','+'])[(*self.view_u2folddir)[i]+1]
       endelse   
       dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU2Folddir'),select=select,set_value=value,sensitive=((self.fold_type eq 0) and self.view_u2fold)
       if extravaxis then begin
          self->my_widget_control,'viewU3Fold',self.projBas,set_button=self.view_u3fold,sensitive=(self.fold_type eq 0)
          self->my_widget_control,'viewU3Foldcenter',self.projBas,set_value=dm_to_string(*(self.view_u3foldcenter),separator=','),sensitive=((self.fold_type eq 0) and self.view_u3fold)
          nfd = n_elements(*self.view_u3folddir)
          if nfd le 1 then begin
             select = (*self.view_u3folddir)[0]+1
             value  = ['-','auto','+']
          endif else begin
             select = 0
             value = (['-','auto','+'])[(*self.view_u3folddir)[0]+1]
             for i=1,nfd-1 do value = value+','+(['-','auto','+'])[(*self.view_u3folddir)[i]+1]
          endelse   
          dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU3Folddir'),select=select,set_value=value,sensitive=((self.fold_type eq 0) and self.view_u3fold)
       endif
       if self.idl_version ge 5.6 then begin  ;tips for fold buttons
          self->my_widget_control,'viewU1Fold',self.projBas,tooltip=(['Specify the folding center and direction to the right.','Specify the n-fold symmetry to the right.'])[self.fold_type]
          self->my_widget_control,'viewU2Fold',self.projBas,tooltip='Specify the folding center and direction to the right.'
          if extravaxis then $
          self->my_widget_control,'viewU3Fold',self.projBas,tooltip='Specify the folding center and direction to the right.'
       endif
    endif
    if keyword_set(foldonly) then return
    switch self.samp_typ of
      0:begin   ;powder
        value = self.view_type[0:3+(self.instrname eq 'dcs'?1:0)]
        if (self.instrname ne 'dcs') and (self.view_u1typ eq 4) then begin
           self.view_u1typ = 1
           self.view_u1lab[0] = self.view_label[self.view_u1typ]
        endif
        if (self.instrname ne 'dcs') and (self.view_u2typ eq 4) then begin
           self.view_u2typ = 0
           self.view_u2lab[0] = self.view_label[self.view_u2typ]
        endif
        dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU1typ'),set_value=value,select=self.view_u1typ
        dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU2typ'),set_value=value,select=self.view_u2typ
        break
        end
      1:        ;single crystal
      2:        ;diffuse scattering single energy
      3:begin   ;diffuse scattering multiple energy   
        self->my_widget_control,'viewAxis',self.projBas,set_value=(self.view_ortharbi?'  Viewing Axes:':'  Orthogonal Viewing Axes:')
        self->my_widget_control,'lattA',self.projBas,set_value=dm_to_string(float(self.latt_parm[0]))
        self->my_widget_control,'lattB',self.projBas,set_value=dm_to_string(float(self.latt_parm[1]))
        self->my_widget_control,'lattC',self.projBas,set_value=dm_to_string(float(self.latt_parm[2]))
        self->my_widget_control,'lattAA',self.projBas,set_value=dm_to_string(float(self.latt_parm[3]))
        self->my_widget_control,'lattBB',self.projBas,set_value=dm_to_string(float(self.latt_parm[4]))
        self->my_widget_control,'lattCC',self.projBas,set_value=dm_to_string(float(self.latt_parm[5]))
        self->my_widget_control,'orieUx',self.projBas,set_value=dm_to_string(self.orie_u[0])
        self->my_widget_control,'orieUy',self.projBas,set_value=dm_to_string(self.orie_u[1])
        self->my_widget_control,'orieUz',self.projBas,set_value=dm_to_string(self.orie_u[2])
        self->my_widget_control,'orieVx',self.projBas,set_value=dm_to_string(self.orie_v[0])
        self->my_widget_control,'orieVy',self.projBas,set_value=dm_to_string(self.orie_v[1])
        self->my_widget_control,'orieVz',self.projBas,set_value=dm_to_string(self.orie_v[2])
        self->my_widget_control,'oriePsi',self.projBas,set_value=dm_to_string(self.orie_psi,resolution=3)
        if (total(abs(self.view_u1)) eq 0) or (total(abs(self.view_u2)) eq 0) then begin
           self.view_u1 = self.orie_u
           self.view_u2 = self.orie_v
           self->adjust_viewlabel,[0,1]
        endif
        if extravaxis  and (total(abs(self.view_u3)) eq 0) then begin
           self.view_u3 = [0,0,1]
           self.view_u3lab = 'L'
        endif
        self->reset_droplist
        self->my_widget_control,'viewU1h',self.projBas,set_value=dm_to_string(self.view_u1[0])
        self->my_widget_control,'viewU1k',self.projBas,set_value=dm_to_string(self.view_u1[1])
        self->my_widget_control,'viewU1l',self.projBas,set_value=dm_to_string(self.view_u1[2])
        self->my_widget_control,'viewU2h',self.projBas,set_value=dm_to_string(self.view_u2[0])
        self->my_widget_control,'viewU2k',self.projBas,set_value=dm_to_string(self.view_u2[1])
        self->my_widget_control,'viewU2l',self.projBas,set_value=dm_to_string(self.view_u2[2])
        if extravaxis then begin
           self->my_widget_control,'viewU3h',self.projBas,set_value=dm_to_string(self.view_u3[0])
           self->my_widget_control,'viewU3k',self.projBas,set_value=dm_to_string(self.view_u3[1])
           self->my_widget_control,'viewU3l',self.projBas,set_value=dm_to_string(self.view_u3[2])
           self->my_widget_control,'viewU3lab',self.projBas,set_value=self.view_u3lab
        endif 
        self->my_widget_control,'oriepsiLab',self.projBas,set_value=(self.instrname eq 'macs')?'   <- A3 offset angle':(((self.samp_typ eq 1)?'   <- ':'   <- first ')+'angle of ki^u')      ; positive if (ki x u)||Qz (up)
        break
        end
      else:
    endswitch
    self->my_widget_control,'viewU1lab',self.projBas,set_value=self.view_u1lab[(self.samp_typ ge 1)]
    self->my_widget_control,'viewU2lab',self.projBas,set_value=self.view_u2lab[(self.samp_typ ge 1)]
end

;reset the data content for self.samp_typ=-1
pro dcs_mslice::reset_data,data=data,author=author,label=label,instrument=instrument,is_uniq=is_uniq,unit=unit,no_copy=no_copy
    if n_elements(author) eq 1 then begin
       nlin = n_elements(*self.info) 
       date = strsplit((*self.info)[nlin-1],' ',/extract) & date = date[n_elements(date)-1]
       (*self.info)[nlin-1] = 'Yiming Qiu(yiming.qiu@nist.gov), '+author+'   '+date 
    endif
    if self.samp_typ ne -1 then return
    reset   = 1b
    n_item  = n_elements((*self.dataPtr).label)
    n_label = n_elements(label)
    n_unit  = n_elements(unit)
    n_uniq  = n_elements(is_uniq)
    n_instr = n_elements(instrument) 
    if n_elements(data) ne 0 then begin
       self.ftypecheckmask = (total(finite(data,/nan)) gt 0)
       o_item = n_item
       n_item = n_elements(data[0,*])
       if n_item lt 4 then begin
          ok = dialog_message('The data must be [ndat,nitem] with nitem>3.',dialog_parent=self.tlb,/center)
          return
       endif
       instrument1 = (*self.dataPtr).instrument
       label1      = (*self.dataPtr).label
       unit1       = (*self.dataPtr).unit
       is_uniq1    = bytarr(n_item)
       if n_item ne o_item then begin
          reset    = 0b   ;need to reconstruct display
          label1   = ['I','dI','x'+dm_to_string(indgen(n_item-2)+1)]
          unit1    = strarr(n_item)
       endif
       ptr_free,self.volsel_t,self.volsel_tft,self.slice_t,self.slice_tft,self.cutsel_t,self.cutsel_tft
       if n_item gt 5 then begin
          self.volsel_t   = ptr_new(indgen(n_item-5)+3)
          self.volsel_tft = ptr_new(fltarr(n_item-5,2))
          (*self.volsel_tft)[*] = !values.f_nan 
       endif
       if n_item gt 4 then begin
          self.slice_t   = ptr_new(indgen(n_item-4)+2)
          self.slice_tft = ptr_new(fltarr(n_item-4,2))
          (*self.slice_tft)[*] = !values.f_nan 
          if self.volsel_x[0] gt n_item-3 then self.volsel_x[0] = 0
          if self.volsel_y[0] gt n_item-3 then self.volsel_y[0] = 1
          if self.volsel_z[0] gt n_item-3 then self.volsel_z[0] = 2
       endif
       if n_item gt 3 then begin
          self.cutsel_t    = ptr_new(indgen(n_item-3)+1)
          self.cutsel_tft  = ptr_new(fltarr(n_item-3,2))
          (*self.cutsel_tft)[*] = !values.f_nan 
          if self.slice_x[0] gt n_item-3 then self.slice_x[0] = 0
          if self.slice_y[0] gt n_item-3 then self.slice_y[0] = 1
          if self.cut_x[0]   gt n_item-3 then self.cut_x[0]   = 0
          if self.cut_y[0]   gt n_item-2 then self.cut_y[0]   = 0
       endif
       (*self.dataPtr) = create_struct('data',keyword_set(no_copy)?temporary(data):data,'label',label1,'unit',unit1,'is_uniq',is_uniq1,'instrument',instrument1)
       if finite(self.mask_filler) then begin
          ptr_free,self.dataStrPtr
          index = where((*self.dataPtr).data[*,0] gt self.mask_filler,count)
          if count ne 0 then begin
             ok = dialog_message('The existing mask value of '+dm_to_string(self.mask_filler)+' has been used.',dialog_parent=self.tlb,/info,/center)
             self.dataStrPtr = ptr_new((*self.dataPtr))
             (*self.dataPtr) = {data:(*self.dataStrPtr).data[index,*],label:(*self.dataStrPtr).label,$
                                unit:(*self.dataStrPtr).unit,is_uniq:(*self.dataStrPtr).is_uniq,instrument:(*self.dataStrPtr).instrument}
          endif else begin
             self.mask_filler = !values.f_nan
             self->my_widget_control,'maskFiller',self.tlb,set_button=0b
          endelse
       endif
    endif
       
    if n_label ne 0 then begin
       (*self.dataPtr).label[0:(n_item<n_label)-1] = label[0:(n_item<n_label)-1]
    endif
    
    if n_unit ne 0 then begin
       (*self.dataPtr).unit[0:(n_item<n_unit)-1] = unit[0:(n_item<n_unit)-1]
    endif

    if n_uniq ne 0 then begin
       (*self.dataPtr).is_uniq[0:(n_item<n_uniq)-1] = is_uniq[0:(n_item<n_uniq)-1]
    endif

    if n_instr ne 0 then begin
       (*self.info)[0] = '--Mslice'+(strlen(instrument[0]) eq 0?'':' for ')+instrument[0]+' --'       
       widget_control,self.tlb,tlb_set_title=(*self.info)[0]
       (*self.dataPtr).instrument = instrument[0]
    endif
    
    if (~ keyword_set(reset)) then begin
       geom_old = widget_info(self.dispBas,/geometry)
       widget_control,self.tlb,update=0
       widget_control,self.dispBas,/destroy
    endif
    self->reset_display,reset=reset
    self->my_widget_control,'savevolumeMenu',sensitive=(n_item ge 5)
    if (~ keyword_set(reset)) then begin
       geom = widget_info(self.dispBas,/geometry)
       if self.idl_version ge 5.5 then begin
          self.geom[1] = self.geom[1]+geom.ysize-geom_old.ysize
          widget_control,self.tlb,ysize=self.geom[1]
       endif else begin
          self.geom[1] = self.geom[1]+geom.scr_ysize-geom_old.scr_ysize
          widget_control,self.tlb,scr_ysize=self.geom[1]
       endelse
       widget_control,self.dispBas,/update,/map
       widget_control,self.tlb,/update
    endif
end

;(re)create and set values of all the widgets in self.dispBas
;keyword:
;   reset:    if set, only set the existing widget values, otherwise regenerate all the widgets
pro dcs_mslice::reset_display,reset=reset
    z2_yn = (self.extravaxis_yn and (self.instrname ne 'macs') and (self.samp_typ eq 1 or self.samp_typ eq 3))  ;flag for z2 axis in trajectory
    z1_yn = ((self.extravaxis_yn and (self.instrname ne 'wand')) or (self.samp_typ ne 2))                       ;flag for z1 axis in trajectory
    if (~ keyword_set(reset)) then begin   ;(re)generate all the widgets
       if ~widget_info(self.dispBas,/valid_id) then begin
          if self.samp_typ eq -1 then $ ;non-tab window
             self.dispBas = widget_base(self.tlb,/frame,xpad=0,ypad=0,map=0,space=0) $
          else $
             self.dispBas = widget_base(self.tlb,/frame,xpad=0,ypad=0,map=0,space=0,xsize=self.xsize[0])
       endif
       widget_control,self.dispBas,update=0
       if (~self.view[1]) and (self.samp_typ ne -1) then return
       xsize = fix((self.xsize[0]-3.*self.xsize[1]-self.xsize[2])/4.,type=3)
       self.xsize[4] = xsize
       oldxsize1 = self.xsize[1]
       if self.samp_typ eq -1 then begin ;non-tab window
          extra = widget_base(self.dispBas,/col,xpad=0,ypad=1) 
          nitem = n_elements((*self.dataPtr).label)
          sel_list = (*self.dataPtr).label[2:nitem-1]
          row   = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label = widget_label(row,value='sha ding',/align_center) 
          geom  = widget_info(label,/geometry)
          self.xsize[1] = geom.xsize
          if nitem ge 5 then begin  
             widget_control,label,set_value='  Volume:'
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='x',xsize=self.xsize[1],/align_center)
             volselx   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volselX',select=self.volsel_x[0])
             label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
             volxfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volxfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volxto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volxto')
             label     = widget_label(row,value='step',xsize=xsize,/align_center)
             volxstep  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volxstep')   
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
             volsely   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volselY',select=self.volsel_y[0])
             label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
             volyfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volyfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volyto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volyto')
             label     = widget_label(row,value='step',xsize=xsize,/align_center)
             volystep  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volystep')   
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='z',xsize=self.xsize[1],/align_center)
             volselz   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volselZ',select=self.volsel_z[0])
             label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
             volzfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volzfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volzto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volzto')
             label     = widget_label(row,value='step',xsize=xsize,/align_center)
             volzstep  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volzstep')   
             for i=0,nitem-6 do begin
                 row      = widget_base(extra,/row,xpad=0,ypad=0,space=0)
                 label    = widget_label(row,value='',xsize=self.xsize[1],/align_center)
                 voltsel  = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volsel_T'+dm_to_string(i),select=(*self.volsel_t)[i])
                 label    = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
                 voltfrom = self->my_widget_text(row,value=dm_to_string((*self.volsel_tft)[i,0]),scr_xsize=xsize,uname='volselfrom_T'+dm_to_string(i))
                 label    = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
                 voltto   = self->my_widget_text(row,value=dm_to_string((*self.volsel_tft)[i,1]),scr_xsize=xsize,uname='volselto_T'+dm_to_string(i))
             endfor
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             label     = widget_label(row,value='intensity range',xsize=self.xsize[1]+self.xsize[2],/align_center)
             volIfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volIfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volIto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volIto')
             label     = widget_label(row,value='color    table ',xsize=xsize*1.5,/align_center)
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             label     = widget_label(row,value='smoothing level',xsize=self.xsize[1]+self.xsize[2],/align_center)
             volSmooth = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volSmooth')
             label     = widget_label(row,value='interp.',xsize=self.xsize[1],/align_center)
             volInterp = dm_widget_droplist(row,value=['Nearest Neighb','Trilinear'],xsize=xsize,uname='volInterp')
             volColor  = dm_widget_droplist(row,value=(*self.slice_cmap),xsize=xsize*2,uname='volColor')
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=3*self.xsize[1]+self.xsize[2]+xsize*2,/align_center)
             volPlot   = widget_button(row,/pushbutton_events,value='Plot Volume',xsize=xsize,uname='volPlot')
             voloPlot  = widget_button(row,/pushbutton_events,value='Oplot Volume',xsize=xsize,uname='voloPlot')
             row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='  Slice:',/align_center) 
          endif else $
             widget_control,label,set_value='  Slice:'
          row          = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label        = widget_label(row,value='x',xsize=self.xsize[1],/align_center)
          dispselH     = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='slicesel_x',select=self.slice_x[0])
          label        = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          dispHfrom    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispxfrom')
          label        = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          dispHto      = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispxto')
          label        = widget_label(row,value='step',xsize=xsize,/align_center)
          dispHstep    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispxstep')   
          row          = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label        = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
          dispselV     = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='slicesel_y',select=self.slice_y[0])
          label        = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          dispVfrom    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispyfrom')
          label        = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          dispVto      = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispyto')
          label        = widget_label(row,value='step',xsize=xsize,/align_center)
          dispVstep    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispystep')
          for i=0,nitem-5 do begin
              row      = widget_base(extra,/row,xpad=0,ypad=0,space=0)
              label    = widget_label(row,value='',xsize=self.xsize[1],/align_center)
              disptsel = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='dispsel_T'+dm_to_string(i),select=(*self.slice_t)[i])
              label    = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
              disptfrom= self->my_widget_text(row,value=dm_to_string((*self.slice_tft)[i,0]),scr_xsize=xsize,uname='dispselfrom_T'+dm_to_string(i))
              label    = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
              disptto  = self->my_widget_text(row,value=dm_to_string((*self.slice_tft)[i,1]),scr_xsize=xsize,uname='dispselto_T'+dm_to_string(i))
          endfor
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
          label     = widget_label(row,value='intensity range',xsize=self.xsize[1]+self.xsize[2],/align_center)
          dispIfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispIfrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          dispIto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispIto')
          label     = widget_label(row,value='color    table ',xsize=xsize*1.5,/align_center)
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
          label     = widget_label(row,value='smoothing level',xsize=self.xsize[1]+self.xsize[2],/align_center)
          dispSmooth = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispSmooth')
          label     = widget_label(row,value='shading',xsize=self.xsize[1],/align_center)
          dispShading = dm_widget_droplist(row,value=['Flat','Gouraud'],xsize=xsize,uname='dispShading')
          dispColor = dm_widget_droplist(row,value=(*self.slice_cmap),xsize=xsize*2,uname='dispColor')
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=2*self.xsize[1]+self.xsize[2]+xsize,/align_center)
          dispFitSlice  = widget_button(row,/pushbutton_events,value='Fit',xsize=self.xsize[1],uname='dispFitSlice')
          dispPlotSlice = widget_button(row,/pushbutton_events,value='Plot Slice',xsize=xsize,uname='dispPlotSlice')
          dispoPltSlice = widget_button(row,/pushbutton_events,value='Oplot Slice',xsize=xsize,uname='dispoPltSlice')
          dispSurfSlice = widget_button(row,/pushbutton_events,value='Surf Slice',xsize=xsize,uname='dispSurfSlice')
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0,/align_left)
          label     = widget_label(row,value='  Cut:',xsize=self.xsize[1],/align_center)
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='along',xsize=self.xsize[1],/align_center)
          cutX      = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='cutX',select=self.cut_x[0])
          label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          cutFrom   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutFrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          cutTo     = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutTo')
          label     = widget_label(row,value='step',xsize=xsize,/align_center)
          cutStep   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutStep')   
          for i=0,nitem-4 do begin
              row      = widget_base(extra,/row,xpad=0,ypad=0,space=0)
              label    = widget_label(row,value='',xsize=self.xsize[1],/align_center)
              cuttsel  = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='cutsel_T'+dm_to_string(i),select=(*self.cutsel_t)[i])
              label    = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
              cuttfrom = self->my_widget_text(row,value=dm_to_string((*self.cutsel_tft)[i,0]),scr_xsize=xsize,uname='cutselfrom_T'+dm_to_string(i))
              label    = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
              cuttto   = self->my_widget_text(row,value=dm_to_string((*self.cutsel_tft)[i,1]),scr_xsize=xsize,uname='cutselto_T'+dm_to_string(i))
          endfor
          label     = widget_label(row,value='symbol',xsize=xsize,/align_center)
          cutColor  = dm_widget_droplist(row,value=(*self.colors),xsize=xsize,uname='cutColor')
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
          cutY      = dm_widget_droplist(row,value=['Intensity',sel_list],xsize=self.xsize[2],uname='cutY',select=self.cut_y[0]) ;volume data
          label     = widget_label(row,value='range',xsize=self.xsize[1],/align_center)
          cutYfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutYfrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          cutYto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutYto')
          cutSymbol = dm_widget_droplist(row,value=(*self.symbols),xsize=xsize,uname='cutSymbol')
          cutLine   = dm_widget_droplist(row,value=(*self.lines),xsize=xsize,uname='cutLine')
          row       = widget_base(extra,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=3*self.xsize[1]+self.xsize[2]+xsize,/align_center)
          cutFit    = widget_button(row,/pushbutton_events,value='Fit',xsize=xsize,uname='cutFit')
          cutPlot   = widget_button(row,/pushbutton_events,value='Plot Cut',xsize=xsize,uname='cutPlot')
          cutoPlot  = widget_button(row,/pushbutton_events,value='Oplot Cut',xsize=xsize,uname='cutoPlot')
       endif else begin ;tab window
          case self.samp_typ of
            0: sel_list = self.view_type[0:3]    ;powder
            1: sel_list = self.traj_type[0:2]    ;single crystal
            2: sel_list = self.traj_type[0:1]    ;diffuse scattering single E
            3: sel_list = self.traj_type[0:2]    ;diffuse scattering multipe E
            else:
          endcase
          ;adjust button size
          if self.samp_typ eq 0 then self->my_widget_control,'calcPrj',self.projBas,scr_xsize=self.xsize[4]*2
          extra  = widget_tab(self.dispBas,tab_mode=0,uname='dispTab')
          sliTab = widget_base(extra,/col,title='Slice',uname='sliceTab',xpad=0,ypad=5)
          cutTab = widget_base(extra,/col,title='Cut',uname='cutTab',xpad=0,ypad=5)
          tmp = widget_info(sliTab,/geometry)
          self.xsize[1] = self.xsize[1]-tmp.space
          ;volume
          if (self.instrname eq 'macs' and self.extravaxis_yn) or (self.samp_typ eq 1) or (self.samp_typ eq 2 and self.extravaxis_yn) or (self.samp_typ eq 3) then begin
             volTab = widget_base(extra,/col,title='Volume',uname='volTab',xpad=0,ypad=5)
             row       = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='x',xsize=self.xsize[1],/align_center)
             volselx   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volselX',select=self.volsel_x[self.samp_typ])
             label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
             volxfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volxfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volxto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volxto')
             label     = widget_label(row,value='step',xsize=xsize,/align_center)
             volxstep  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volxstep')   
             row       = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
             volsely   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volselY',select=self.volsel_y[self.samp_typ])
             label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
             volyfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volyfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volyto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volyto')
             label     = widget_label(row,value='step',xsize=xsize,/align_center)
             volystep  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volystep')   
             row       = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='z',xsize=self.xsize[1],/align_center)
             volselz   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='volselZ',select=self.volsel_z[self.samp_typ])
             label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
             volzfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volzfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volzto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volzto')
             label     = widget_label(row,value='step',xsize=xsize,/align_center)
             volzstep  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volzstep')   
             nitem     = [2,3,2,3]+self.extravaxis_yn
             for i=0,nitem[self.samp_typ]-4 do begin
                 row      = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
                 label    = widget_label(row,value='',xsize=self.xsize[1],/align_center)
                 voltsel  = dm_widget_droplist(row,xsize=self.xsize[2],uname='volsel_T'+dm_to_string(i))
                 label    = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
                 voltfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volselfrom_T'+dm_to_string(i))
                 label    = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
                 voltto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volselto_T'+dm_to_string(i))
             endfor
             row       = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             label     = widget_label(row,value='intensity range',xsize=self.xsize[1]+self.xsize[2],/align_center)
             volIfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volIfrom')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             volIto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volIto')
             label     = widget_label(row,value='color    table ',xsize=xsize*1.5,/align_center)
             row       = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             label     = widget_label(row,value='smoothing level',xsize=self.xsize[1]+self.xsize[2],/align_center)
             volSmooth = self->my_widget_text(row,value='',scr_xsize=xsize,uname='volSmooth')
             label     = widget_label(row,value='interp.',xsize=self.xsize[1],/align_center)
             volInterp = dm_widget_droplist(row,value=['Nearest Neighb','Trilinear'],xsize=xsize,uname='volInterp')
             volColor  = dm_widget_droplist(row,value=(*self.slice_cmap),xsize=xsize*2,uname='volColor')
             row       = widget_base(volTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=3*self.xsize[1]+self.xsize[2]+xsize*2,/align_center)
             volPlot   = widget_button(row,/pushbutton_events,value='Plot Volume',xsize=xsize,uname='volPlot')  
             voloPlot  = widget_button(row,/pushbutton_events,value='Oplot Volume',xsize=xsize,uname='voloPlot')
          endif 
          ;slice
          row          = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
          label        = widget_label(row,value='x',xsize=self.xsize[1],/align_center)
          dispselH     = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='slicesel_x',select=self.slice_x[self.samp_typ])
          label        = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          dispHfrom    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispxfrom')
          label        = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          dispHto      = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispxto')
          label        = widget_label(row,value='step',xsize=xsize,/align_center)
          dispHstep    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispxstep')   
          row          = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
          label        = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
          dispselV     = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='slicesel_y',select=self.slice_y[self.samp_typ])
          label        = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          dispVfrom    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispyfrom')
          label        = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          dispVto      = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispyto')
          label        = widget_label(row,value='step',xsize=xsize,/align_center)
          dispVstep    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispystep')
          if (self.instrname eq 'macs' and self.extravaxis_yn) or (self.samp_typ eq 1) or ((self.samp_typ eq 2) and self.extravaxis_yn) or (self.samp_typ eq 3) then begin
             row       = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             disptsel  = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='dispsel_T0',select=self.slice_t1[self.samp_typ])
             label     = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
             disptfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispselfrom_T0')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             disptto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispselto_T0')
          endif
          if ((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn then begin
             row       = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
             label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             disptsel  = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='dispsel_T1',select=self.slice_t2[self.samp_typ])
             label     = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
             disptfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispselfrom_T1')
             label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             disptto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispselto_T1')
          endif
          row       = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
          label     = widget_label(row,value='intensity range',xsize=self.xsize[1]+self.xsize[2],/align_center)
          dispIfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispIfrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          dispIto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispIto')
          label     = widget_label(row,value='color    table ',xsize=xsize*1.5,/align_center)
          row       = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
          label     = widget_label(row,value='smoothing level',xsize=self.xsize[1]+self.xsize[2],/align_center)
          dispSmooth = self->my_widget_text(row,value='',scr_xsize=xsize,uname='dispSmooth')
          label     = widget_label(row,value='shading',xsize=self.xsize[1],/align_center)
          dispShading = dm_widget_droplist(row,value=['Flat','Gouraud'],xsize=xsize,uname='dispShading')
          dispColor = dm_widget_droplist(row,value=(*self.slice_cmap),xsize=xsize*2,uname='dispColor')
          row       = widget_base(sliTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=2*self.xsize[1]+self.xsize[2]+xsize,/align_center)
          dispFitSlice  = widget_button(row,/pushbutton_events,value='Fit',xsize=self.xsize[1],uname='dispFitSlice')
          dispPlotSlice = widget_button(row,/pushbutton_events,value='Plot Slice',xsize=xsize,uname='dispPlotSlice')
          dispoPltSlice = widget_button(row,/pushbutton_events,value='Oplot Slice',xsize=xsize,uname='dispoPltSlice')
          dispSurfSlice = widget_button(row,/pushbutton_events,value='Surf Slice',xsize=xsize,uname='dispSurfSlice')
          ;cut
          row       = widget_base(cutTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='along',xsize=self.xsize[1],/align_center)
          cutX      = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='cutX',select=self.cut_x[self.samp_typ])
          label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          cutFrom   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutFrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          cutTo     = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutTo')
          label     = widget_label(row,value='step',xsize=xsize,/align_center)
          cutStep   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutStep')
          row       = widget_base(cutTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=self.xsize[1],/align_center)
          cuttsel   = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='cutsel_T0',select=self.cut_t1[self.samp_typ])
          label     = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
          cuttfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutselfrom_T0')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          cuttto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutselto_T0')
          if (self.instrname eq 'macs' and self.extravaxis_yn) or (self.samp_typ eq 1) or ((self.samp_typ eq 2) and self.extravaxis_yn) or (self.samp_typ eq 3) then begin
             row      = widget_base(cutTab,/row,xpad=0,ypad=0,space=0)
             label    = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             cuttsel  = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='cutsel_T1',select=self.cut_t2[self.samp_typ])
             label    = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
             cuttfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutselfrom_T1')
             label    = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             cuttto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutselto_T1')
          endif
          if (self.samp_typ eq 1 or self.samp_typ eq 3) and self.extravaxis_yn then begin
             row      = widget_base(cutTab,/row,xpad=0,ypad=0,space=0)
             label    = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             cuttsel  = dm_widget_droplist(row,value=sel_list,xsize=self.xsize[2],uname='cutsel_T2',select=self.cut_t3[self.samp_typ])
             label    = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
             cuttfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutselfrom_T2')
             label    = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             cuttto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutselto_T2')
          endif
          label     = widget_label(row,value='symbol',xsize=xsize,/align_center)
          cutColor  = dm_widget_droplist(row,value=(*self.colors),xsize=xsize,uname='cutColor')
          row       = widget_base(cutTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
          cutY      = dm_widget_droplist(row,value=['Intensity',sel_list],xsize=self.xsize[2],uname='cutY',select=self.cut_y[self.samp_typ])
          label     = widget_label(row,value='range',xsize=self.xsize[1],/align_center)
          cutYfrom  = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutYfrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          cutYto    = self->my_widget_text(row,value='',scr_xsize=xsize,uname='cutYto')
          cutSymbol = dm_widget_droplist(row,value=(*self.symbols),xsize=xsize,uname='cutSymbol')
          cutLine   = dm_widget_droplist(row,value=(*self.lines),xsize=xsize,uname='cutLine')
          row       = widget_base(cutTab,/row,xpad=0,ypad=0,space=0)
          label     = widget_label(row,value='',xsize=3*self.xsize[1]+self.xsize[2]+xsize,/align_center)
          cutFit    = widget_button(row,/pushbutton_events,value='Fit',xsize=xsize,uname='cutFit')
          cutPlot   = widget_button(row,/pushbutton_events,value='Plot Cut',xsize=xsize,uname='cutPlot')
          cutoPlot  = widget_button(row,/pushbutton_events,value='Oplot Cut',xsize=xsize,uname='cutoPlot')
          ;trajectory
          trajTab   = widget_base(extra,/row,title='Detector Trajectory',uname='trajTab',xpad=0,ypad=5)
          left      = widget_base(trajTab,/col,xpad=0,ypad=0)
          row       = widget_base(left,/row,ypad=0,xpad=0,space=0)
          label     = widget_label(row,value='x',xsize=self.xsize[1],/align_center)
          trajX     = dm_widget_droplist(row,value=self.traj_type,xsize=self.xsize[2],uname='trajX')
          label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          trajXfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajXfrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          trajXto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajXto')
          row       = widget_base(left,/row,ypad=0,xpad=0,space=0)
          label     = widget_label(row,value='y',xsize=self.xsize[1],/align_center)
          trajY     = dm_widget_droplist(row,value=self.traj_type,xsize=self.xsize[2],uname='trajY')
          label     = widget_label(row,value='from:',xsize=self.xsize[1],/align_center)
          trajYfrom = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajYfrom')
          label     = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
          trajYto   = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajYto')
          if z1_yn then begin
             row    = widget_base(left,/row,ypad=0,xpad=0,space=0)
             label  = widget_label(row,value='',xsize=self.xsize[1],/align_center)
             trajZ1 = dm_widget_droplist(row,value=self.traj_type,xsize=self.xsize[2],uname='trajZ1')
             label  = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
             trajZ1from = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajZ1from')
             label  = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             trajZ1to = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajZ1to')
          endif 
          if z2_yn then begin
             row    = widget_base(left,/row,ypad=0,xpad=0,space=0)
             label  = widget_label(row,value='',xsize=self.xsize[1]-1,/align_center)
             trajZ2 = dm_widget_droplist(row,value=self.traj_type,xsize=self.xsize[2],uname='trajZ2')
             label  = widget_label(row,value='thick',xsize=self.xsize[1],/align_center)
             trajZ2from = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajZ2from')
             label  = widget_label(row,value='to:',xsize=self.xsize[1],/align_center)
             trajZ2to = self->my_widget_text(row,value='',scr_xsize=xsize,uname='trajZ2to')
          endif
          geom     = widget_info(left,/geom)
          right    = widget_base(trajTab,/col,/align_bottom,xpad=0,xsize=self.xsize[0]-geom.xsize-8,ypad=0)
          plotTra  = widget_button(right,/pushbutton_events,value='Plot Trajectories',uname='plotTra',sensitive=0)
          oplotTra = widget_button(right,/pushbutton_events,value='Oplot Trajectories',uname='oplotTra',sensitive=0)
          if self.idl_version ge 5.6 then widget_control,oplotTra,tooltip='Overplot the trajectories in the current plot window.' 
          ;calculator
          if ~lmgr(/vm) then begin
             calcTab = widget_base(extra,/row,title='Calculator',uname='calcTab',xpad=0,ypad=5)
             tmp1 = widget_info(sliTab,/geometry)
             tmp2 = widget_info(plotTra,/geometry)
             ysize = tmp1.scr_ysize-tmp2.scr_ysize & line = 4
             repeat begin
                line = line+1
                void = widget_text(calcTab,uname='calcInput',ysize=line,/editable,scr_xsize=self.xsize[2]+3*self.xsize[1]+2*xsize) 
                tmp  = widget_info(void,/geometry) 
                widget_control,void,/destroy
             endrep until tmp.scr_ysize gt ysize
             void   = widget_text(calcTab,uname='calcInput',ysize=line-1,/editable,scr_xsize=self.xsize[2]+3*self.xsize[1]+2*xsize,/scroll) 
             right  = widget_base(calcTab,/col,/align_bottom,xpad=0,xsize=self.xsize[0]-geom.xsize-8,ypad=0)
             clcCal = widget_button(right,/pushbutton_events,value='Clear',uname='clearCal')
             calCal = widget_button(right,/pushbutton_events,value='Calculate',uname='calcCal')   
          endif
       endelse
       self.xsize[1] = oldxsize1
       if self.idl_version ge 5.6 then begin  ;tips for some widget buttons
          if n_elements(voloPlot) ne 0 then $
          widget_control,voloPlot,tooltip='Overplot the volume in the current plot window.'
          widget_control,dispFitSlice,tooltip='Fit the grouped slice data in PAN(Peak Analysis).'
          widget_control,dispPlotSlice,tooltip='Contour plot the slice.'
          widget_control,dispoPltSlice,tooltip='Overplot the slice in the current plot window.'
          widget_control,dispSurfSlice,tooltip='Surface plot the slice.'
          widget_control,cutoPlot,tooltip='Overplot the cut in the current plot window.'
          widget_control,cutFit,tooltip='Fit the cut data in PAN(Peak Analysis).'
       endif
    endif
    if ~widget_info(self.dispBas,/valid_id) then return
    ;set all the values
    if (~ ptr_valid(self.dataPtr)) then begin ;tab window
       extravaxis = ((self.extravaxis_yn) and (self.instrname ne 'macs') and (self.instrname ne 'wand'))  ;excluding T-axis
       self->set_display_button    ;include some menu buttons and all display buttons
       case self.samp_typ of
         0:begin  ;powder
           sel_list = self.view_type[0:3]   ;Not including DCS time channel yet
           end
         1:begin  ;single crystal
           if extravaxis then sel_list = self.traj_type[[0,1,2,3]] else sel_list = self.traj_type[[0,1,3]]
           end
         2:begin  ;diffuse scattering single E
           if extravaxis then sel_list = self.traj_type[[0,1,2]] else sel_list = self.traj_type[[0,1]]
           end
         3:begin  ;diffuse scattering multiple E
           if extravaxis then sel_list = self.traj_type[[0,1,2,3]] else sel_list = self.traj_type[[0,1,3]]
           end
         else:
       endcase
       ;"intelligent" cut step
       if ptr_valid(self.dataStrPtr) then begin
          if self.instrname eq 'macs' then begin
             en = (*self.dataStrPtr).Ei-(*self.dataStrPtr).Ef
             en = en[uniq(en,sort(en))] 
          endif else begin
             en = (*self.dataStrPtr).e
          endelse
          emax = max(en,min=emin)
       endif
       notE = [([self.cut_x[0] ne 0,(self.cut_x[0] ne 0) and (self.cut_x[0] ne 4)])[self.instrname eq 'dcs'],self.cut_x[1] ne 2+extravaxis,1b,self.cut_x[3] ne 2+extravaxis] 
       if notE[self.samp_typ] then begin
          if (~finite(self.cut_step)) then self.cut_step=0.01
       endif else begin
          if finite(self.cut_step) and ptr_valid(self.dataStrPtr) and ((self.instrname eq 'dcs') or (self.instrname eq 'macs')) then begin
             nen = ((finite(self.cut_ran[1])?(self.cut_ran[1]):(emax))-(finite(self.cut_ran[0])?(self.cut_ran[0]):(emin)))/(1e-7>(self.cut_step))
             tmp = where((en le (finite(self.cut_ran[1])?(self.cut_ran[1]):(emax))) and (en ge (finite(self.cut_ran[0])?(self.cut_ran[0]):(emin))), nen1)
             if nen gt nen1 then self.cut_step = !values.f_nan
          endif
       endelse
       ;"intelligent" slice step
       notE = [([self.slice_x[0] ne 0,(self.slice_x[0] ne 0) and (self.slice_x[0] ne 4)])[self.instrname eq 'dcs'],self.slice_x[1] ne 2+extravaxis,1b,self.slice_x[3] ne 2+extravaxis] 
       if notE[self.samp_typ] then begin
          if ~finite(self.slice_xstep) then self.slice_xstep = 0.035
       endif else if ptr_valid(self.dataStrPtr) and ((self.instrname eq 'dcs') or (self.instrname eq 'macs')) and finite(self.slice_xstep) then begin
          nen = ((finite(self.slice_xran[1])?(self.slice_xran[1]):(emax))-(finite(self.slice_xran[0])?(self.slice_xran[0]):(emin)))/(1e-7>(self.slice_xstep))
          tmp = where((en le (finite(self.slice_xran[1])?(self.slice_xran[1]):(emax))) and (en ge (finite(self.slice_xran[0])?(self.slice_xran[0]):(emin))), nen1)
          if nen gt nen1 then self.slice_xstep = !values.f_nan
       endif
       notE = [([self.slice_y[0] ne 0,(self.slice_y[0] ne 0) and (self.slice_y[0] ne 4)])[self.instrname eq 'dcs'],self.slice_y[1] ne 2+extravaxis,1b,self.slice_y[3] ne 2+extravaxis] 
       if notE[self.samp_typ] then begin
          if ~finite(self.slice_ystep) then self.slice_ystep = 0.035
       endif else if ptr_valid(self.dataStrPtr) and ((self.instrname eq 'dcs') or (self.instrname eq 'macs')) and finite(self.slice_ystep) then begin
          nen = ((finite(self.slice_yran[1])?(self.slice_yran[1]):(emax))-(finite(self.slice_yran[0])?(self.slice_yran[0]):(emin)))/(1e-7>(self.slice_ystep))
          tmp = where((en le (finite(self.slice_yran[1])?(self.slice_yran[1]):(emax))) and (en ge (finite(self.slice_yran[0])?(self.slice_yran[0]):(emin))), nen1)
          if nen gt nen1 then self.slice_ystep = !values.f_nan
       endif
       ;"intelligent" volume step
       if (self.instrname eq 'macs' and self.extravaxis_yn) or (self.samp_typ eq 1) or (self.samp_typ eq 2 and self.extravaxis_yn) or (self.samp_typ eq 3) then begin
          if self.extravaxis_yn and (self.instrname ne 'macs') then index = [0b,0b,0b,1b,0b] else index = [0b,0b,1b,0b,0b]
          if self.samp_typ eq 0 then index = [1b,0b,0b,0b,0b]
          isE = [index[self.volsel_x[0]],index[self.volsel_x[1]],0b,index[self.volsel_x[3]]]
          if (~ finite(self.vol_xstep)) then begin
             self.vol_xstep = 0.05
             if isE[self.samp_typ] and ptr_valid(self.dataStrPtr) then begin
                self.vol_xstep = ((finite(self.vol_xran[1])?(self.vol_xran[1]):(emax))-(finite(self.vol_xran[0])?(self.vol_xran[0]):(emin)))/150.
                self.vol_xstep = 0.005>(round(self.vol_xstep*100.)/100.)
             endif
          endif
          isE = [index[self.volsel_y[0]],index[self.volsel_y[1]],0b,index[self.volsel_y[3]]]
          if (~ finite(self.vol_ystep)) then begin
             self.vol_ystep = 0.05
             if isE[self.samp_typ] and ptr_valid(self.dataStrPtr) then begin
                self.vol_ystep = ((finite(self.vol_yran[1])?(self.vol_yran[1]):(emax))-(finite(self.vol_yran[0])?(self.vol_yran[0]):(emin)))/150.
                self.vol_ystep = 0.005>(round(self.vol_ystep*100.)/100.)
             endif
          endif
          isE = [index[self.volsel_z[0]],index[self.volsel_z[1]],0b,index[self.volsel_z[3]]]
          if (~ finite(self.vol_zstep)) then begin
             self.vol_zstep = 0.05
             if isE[self.samp_typ] and ptr_valid(self.dataStrPtr) then begin
                self.vol_zstep = ((finite(self.vol_zran[1])?(self.vol_zran[1]):(emax))-(finite(self.vol_zran[0])?(self.vol_zran[0]):(emin)))/150.
                self.vol_zstep = 0.005>(round(self.vol_zstep*100.)/100.)
             endif
          endif
       endif
       ;trajectory, doesn't include T and time channel(DCS powder) in the list, add 'None' at the end of the list     
       n_list = n_elements(sel_list)
       ind_list = indgen(n_list+1)
       if self.traj_x[self.samp_typ] ge n_list  then self.traj_x[self.samp_typ] = ([1,0])[self.samp_typ gt 0]
       if self.traj_y[self.samp_typ] ge n_list  then self.traj_y[self.samp_typ] = ([0,1])[self.samp_typ gt 0]
       if self.traj_x[self.samp_typ] eq self.traj_y[self.samp_typ] then begin
          self.traj_x[self.samp_typ] = ([1,0])[self.samp_typ gt 0]
          self.traj_y[self.samp_typ] = ([0,1])[self.samp_typ gt 0]
       endif
       if z1_yn and self.traj_z1[self.samp_typ] gt n_list then self.traj_z1[self.samp_typ] = n_list
       if z2_yn and self.traj_z2[self.samp_typ] gt n_list then self.traj_z2[self.samp_typ] = n_list
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajX'),set_value=[sel_list,'None'],select=self.traj_x[self.samp_typ]
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajY'),set_value=[sel_list,'None'],select=self.traj_y[self.samp_typ]
       self->my_widget_control,'trajXfrom', self.dispBas,set_value=dm_to_string(self.traj_xran[0])
       self->my_widget_control,'trajXto',   self.dispBas,set_value=dm_to_string(self.traj_xran[1])
       self->my_widget_control,'trajYfrom', self.dispBas,set_value=dm_to_string(self.traj_yran[0])
       self->my_widget_control,'trajYto',   self.dispBas,set_value=dm_to_string(self.traj_yran[1])
       if z1_yn then begin
          list = where((ind_list ne self.traj_x[self.samp_typ]) and (ind_list ne self.traj_y[self.samp_typ]),count)
          if count ne 0 then self.traj_z1[self.samp_typ] = list[0>((where(list eq self.traj_z1[self.samp_typ]))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajZ1'),set_value=[sel_list,'None'],select=self.traj_z1[self.samp_typ]
          self->my_widget_control,'trajZ1from',self.dispBas,set_value=dm_to_string(self.traj_z1ran[0])
          self->my_widget_control,'trajZ1to',  self.dispBas,set_value=dm_to_string(self.traj_z1ran[1])
       endif
       if z2_yn then begin
          list = where((ind_list ne self.traj_x[self.samp_typ]) and (ind_list ne self.traj_y[self.samp_typ]) and (ind_list ne self.traj_z1[self.samp_typ]),count)
          if count ne 0 then self.traj_z2[self.samp_typ] = list[0>((where(list eq self.traj_z2[self.samp_typ]))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='trajZ2'),set_value=[sel_list,'None'],select=self.traj_z2[self.samp_typ]
          self->my_widget_control,'trajZ2from',self.dispBas,set_value=dm_to_string(self.traj_z2ran[0])
          self->my_widget_control,'trajZ2to',  self.dispBas,set_value=dm_to_string(self.traj_z2ran[1])
       endif 
       if self.samp_typ eq 0 then begin
          sel_list = self.view_type[0:3+(self.instrname eq 'dcs'?1:0)]    ;add time channel back to the selection list for DCS powder
          if (self.instrname eq 'macs') and self.extravaxis_yn then sel_list = [sel_list,'T (K)'] 
       endif else if self.extravaxis_yn and ~extravaxis then begin
          sel_list = [sel_list,'T (K)']                                   ;add T to the list for MACS and WAND
       endif
       n_list   = n_elements(sel_list)
       ind_list = indgen(n_list)
       ;slice
       if self.slice_x[self.samp_typ] ge n_list then self.slice_x[self.samp_typ] = ([1,0])[self.samp_typ ne 0]
       if self.slice_y[self.samp_typ] ge n_list then self.slice_y[self.samp_typ] = ([0,1])[self.samp_typ ne 0]
       if self.slice_x[self.samp_typ] eq self.slice_y[self.samp_typ] then begin
          self.slice_x[self.samp_typ] = ([1,0])[self.samp_typ ne 0]
          self.slice_y[self.samp_typ] = ([0,1])[self.samp_typ ne 0]
       endif
       ;cut 
       if self.cut_x[self.samp_typ] ge n_list then self.cut_x[self.samp_typ] = 0
       if (self.cut_T1[self.samp_typ] eq self.cut_x[self.samp_typ]) or (self.cut_T1[self.samp_typ] ge n_list) then self.cut_T1[self.samp_typ] = (where(ind_list ne self.cut_x[self.samp_typ]))[0]
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T0'),set_value=sel_list,select=self.cut_T1[self.samp_typ]
       self->my_widget_control,'cutselfrom_T0',self.dispBas,set_value=dm_to_string(self.cut_t1ran[0])
       self->my_widget_control,'cutselto_T0',  self.dispBas,set_value=dm_to_string(self.cut_t1ran[1])
       if ((self.instrname eq 'macs') and self.extravaxis_yn) or (self.samp_typ eq 1) or ((self.samp_typ eq 2) and self.extravaxis_yn) or (self.samp_typ eq 3) then begin
          ;slice t1
          list = where((ind_list ne self.slice_x[self.samp_typ]) and (ind_list ne self.slice_y[self.samp_typ]),count)
          if count ne 0 then self.slice_t1[self.samp_typ] = list[0>((where(list eq self.slice_t1[self.samp_typ]))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T0'),set_value=sel_list,select=self.slice_t1[self.samp_typ]
          self->my_widget_control,'dispselfrom_T0',self.dispBas,set_value=dm_to_string(self.slice_t1ran[0])
          self->my_widget_control,'dispselto_T0',  self.dispBas,set_value=dm_to_string(self.slice_t1ran[1])
          ;volume
          vol_existed = 1b
          if self.volsel_x[self.samp_typ] ge n_list then self.volsel_x[self.samp_typ] = 0
          if (self.volsel_y[self.samp_typ] ge n_list) or (self.volsel_y[self.samp_typ] eq self.volsel_x[self.samp_typ]) then $
             self.volsel_y[self.samp_typ] = (where(ind_list ne self.volsel_x[self.samp_typ]))[0]
          if (self.volsel_z[self.samp_typ] ge n_list) or (self.volsel_z[self.samp_typ] eq self.volsel_x[self.samp_typ]) or (self.volsel_z[self.samp_typ] eq self.volsel_y[self.samp_typ]) then $
             self.volsel_z[self.samp_typ] = (where((ind_list ne self.volsel_x[self.samp_typ]) and (ind_list ne self.volsel_y[self.samp_typ])))[0]
          ;cut
          list = where((ind_list ne self.cut_x[self.samp_typ]) and (ind_list ne self.cut_T1[self.samp_typ]),count)
          if count ne 0 then self.cut_T2[self.samp_typ] = list[0>((where(list eq self.cut_T2[self.samp_typ]))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T1'),set_value=sel_list,select=self.cut_T2[self.samp_typ]
          self->my_widget_control,'cutselfrom_T1',self.dispBas,set_value=dm_to_string(self.cut_t2ran[0])
          self->my_widget_control,'cutselto_T1',  self.dispBas,set_value=dm_to_string(self.cut_t2ran[1])
       endif
       if ((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn then begin
          ;slice t2
          list = where((ind_list ne self.slice_x[self.samp_typ]) and (ind_list ne self.slice_y[self.samp_typ]) and (ind_list ne self.slice_t1[self.samp_typ]),count)
          if count ne 0 then self.slice_t2[self.samp_typ] = list[0>((where(list eq self.slice_t2[self.samp_typ]))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T1'),set_value=sel_list,select=self.slice_t2[self.samp_typ]
          self->my_widget_control,'dispselfrom_T1',self.dispBas,set_value=dm_to_string(self.slice_t2ran[0])
          self->my_widget_control,'dispselto_T1',  self.dispBas,set_value=dm_to_string(self.slice_t2ran[1])
          ;volume t1
          list = where((ind_list ne self.volsel_x[self.samp_typ]) and (ind_list ne self.volsel_y[self.samp_typ]) and (ind_list ne self.volsel_z[self.samp_typ]),count)
          if count ne 0 then self.volsel_t1 = list[0>((where(list eq self.volsel_t1))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T0'),set_value=sel_list,select=self.volsel_t1
          self->my_widget_control,'volselfrom_T0',self.dispBas,set_value=dm_to_string(self.volsel_tft1[0])
          self->my_widget_control,'volselto_T0',  self.dispBas,set_value=dm_to_string(self.volsel_tft1[1])
          ;cut
          list = where((ind_list ne self.cut_x[self.samp_typ]) and (ind_list ne self.cut_T1[self.samp_typ]) and (ind_list ne self.cut_T2[self.samp_typ]),count)
          if count ne 0 then self.cut_T3[self.samp_typ] = list[0>((where(list eq self.cut_T3[self.samp_typ]))[0])]
          dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T2'),set_value=sel_list,select=self.cut_T3[self.samp_typ]
          self->my_widget_control,'cutselfrom_T2',self.dispBas,set_value=dm_to_string(self.cut_t3ran[0])
          self->my_widget_control,'cutselto_T2',  self.dispBas,set_value=dm_to_string(self.cut_t3ran[1])
       endif
    endif else begin
       nitem    = n_elements((*self.dataPtr).label)
       sel_list = (*self.dataPtr).label[2:nitem-1]
       n_list   = n_elements(sel_list)
       ind_list = indgen(n_list)
       ;volume 
       if nitem ge 5 then begin 
          vol_existed = 1b
          if self.volsel_x[0] ge n_list then self.volsel_x[0] = 0
          if (self.volsel_y[0] ge n_list) or (self.volsel_y[0] eq self.volsel_x[0]) then $
             self.volsel_y[0] = (where(ind_list ne self.volsel_x[0]))[0]
          if (self.volsel_z[0] ge n_list) or (self.volsel_z[0] eq self.volsel_x[0]) or (self.volsel_z[0] eq self.volsel_y[0]) then $
             self.volsel_z[0] = (where((ind_list ne self.volsel_x[0]) and (ind_list ne self.volsel_y[0])))[0]
          for i=0,nitem-6 do begin  
              tmp = bytarr(n_list)+1b
              tmp[0>([self.volsel_x[0],self.volsel_y[0],self.volsel_z[0]])<(n_list-1)] = 0b
              if i gt 0 then tmp[0>((*self.volsel_t)[0:i-1])<(n_list-1)] = 0b
              list = where(tmp,count)
              if count gt 0 then begin
                 list = ind_list[list]
                 (*self.volsel_t)[i] = list[0>((where(list eq (*self.volsel_t)[i]))[0])]
              endif 
              dm_set_droplist,widget_info(self.dispBas,find_by_uname='volsel_T'+dm_to_string(i)),set_value=sel_list,select=(*self.volsel_t)[i]
              self->my_widget_control,'volselfrom_T'+dm_to_string(i),self.dispBas,set_value=dm_to_string((*self.volsel_tft)[i,0])
              self->my_widget_control,'volselto_T'+dm_to_string(i),self.dispBas,set_value=dm_to_string((*self.volsel_tft)[i,1])
          endfor
       endif
       ;slice
       if self.slice_x[0] ge n_list then self.slice_x[0] = 0
       if self.slice_y[0] ge n_list then self.slice_y[0] = 1
       if self.slice_x[0] eq self.slice_y[0] then begin
          self.slice_x[0] = 0
          self.slice_y[0] = 1
       endif
       for i=0,nitem-5 do begin   
           tmp = bytarr(n_list)+1b
           tmp[0>([self.slice_x[0],self.slice_y[0]])<(n_list-1)] = 0b
           if i gt 0 then tmp[0>((*self.slice_t)[0:i-1])<(n_list-1)] = 0b
           list = where(tmp,count)
           if count gt 0 then begin
              list = ind_list[list]
              (*self.slice_t)[i] = list[0>((where(list eq (*self.slice_t)[i]))[0])]
           endif 
           dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispsel_T'+dm_to_string(i)),set_value=sel_list,select=(*self.slice_t)[i]
           self->my_widget_control,'dispselfrom_T'+dm_to_string(i),self.dispBas,set_value=dm_to_string((*self.slice_tft)[i,0])
           self->my_widget_control,'dispselto_T'+dm_to_string(i),  self.dispBas,set_value=dm_to_string((*self.slice_tft)[i,1])
       endfor
       ;cut
       if self.cut_x[0] ge n_list then self.cut_x[0] = 0
       for i=0,nitem-4 do begin  
           tmp = bytarr(n_list)+1b
           tmp[0>(self.cut_x[0])<(n_list-1)] = 0b
           if i gt 0 then tmp[0>((*self.cutsel_t)[0:i-1])<(n_list-1)] = 0b
           list = where(tmp,count)
           if count gt 0 then begin
              list = ind_list[list]
              (*self.cutsel_t)[i] = list[0>((where(list eq (*self.cutsel_t)[i]))[0])]
           endif 
           dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutsel_T'+dm_to_string(i)),set_value=sel_list,select=(*self.cutsel_t)[i]
           self->my_widget_control,'cutselfrom_T'+dm_to_string(i),self.dispBas,set_value=dm_to_string((*self.cutsel_tft)[i,0])
           self->my_widget_control,'cutselto_T'+dm_to_string(i),  self.dispBas,set_value=dm_to_string((*self.cutsel_tft)[i,1])
       endfor
    endelse
    ;cut
    if self.cut_y[0>(self.samp_typ)] gt n_list then self.cut_y[0>(self.samp_typ)] = 0
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutX'),set_value=sel_list,select=self.cut_x[0>(self.samp_typ)]
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutY'),set_value=['Intensity',sel_list],select=self.cut_y[0>(self.samp_typ)]
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutColor'), select=self.cut_color
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutSymbol'),select=self.cut_symbol
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutLine'),  select=self.cut_line
    self->my_widget_control,'cutFrom', self.dispBas,set_value=dm_to_string(self.cut_ran[0])
    self->my_widget_control,'cutTo',   self.dispBas,set_value=dm_to_string(self.cut_ran[1])
    self->my_widget_control,'cutStep', self.dispBas,set_value=dm_to_string(self.cut_step)
    self->my_widget_control,'cutYfrom',self.dispBas,set_value=dm_to_string(self.cut_yran[0])
    self->my_widget_control,'cutYto',  self.dispBas,set_value=dm_to_string(self.cut_yran[1])
    ;slice
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='slicesel_x'),set_value=sel_list,select=self.slice_x[0>(self.samp_typ)]
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='slicesel_y'),set_value=sel_list,select=self.slice_y[0>(self.samp_typ)]
    self->my_widget_control,'dispxfrom', self.dispBas,set_value=dm_to_string(self.slice_xran[0])
    self->my_widget_control,'dispxto',   self.dispBas,set_value=dm_to_string(self.slice_xran[1])
    self->my_widget_control,'dispxstep', self.dispBas,set_value=dm_to_string(self.slice_xstep)
    self->my_widget_control,'dispyfrom', self.dispBas,set_value=dm_to_string(self.slice_yran[0])
    self->my_widget_control,'dispyto',   self.dispBas,set_value=dm_to_string(self.slice_yran[1])
    self->my_widget_control,'dispystep', self.dispBas,set_value=dm_to_string(self.slice_ystep)
    self->my_widget_control,'dispIfrom', self.dispBas,set_value=dm_to_string(self.slice_iran[0])
    self->my_widget_control,'dispIto',   self.dispBas,set_value=dm_to_string(self.slice_iran[1])
    self->my_widget_control,'dispSmooth',self.dispBas,set_value=dm_to_string(self.slice_smooth,/int)
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispColor'),  select=self.slice_color
    dm_set_droplist,widget_info(self.dispBas,find_by_uname='dispShading'),select=self.slice_shading
    ;volume
    if keyword_set(vol_existed) then begin
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselX'),set_value=sel_list,select=self.volsel_x[0>(self.samp_typ)]
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselY'),set_value=sel_list,select=self.volsel_y[0>(self.samp_typ)]
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='volselZ'),set_value=sel_list,select=self.volsel_z[0>(self.samp_typ)]       
       self->my_widget_control,'volxfrom', self.dispBas,set_value=dm_to_string(self.vol_xran[0])
       self->my_widget_control,'volxto',   self.dispBas,set_value=dm_to_string(self.vol_xran[1])
       self->my_widget_control,'volxstep', self.dispBas,set_value=dm_to_string(self.vol_xstep)
       self->my_widget_control,'volyfrom', self.dispBas,set_value=dm_to_string(self.vol_yran[0])
       self->my_widget_control,'volyto',   self.dispBas,set_value=dm_to_string(self.vol_yran[1])
       self->my_widget_control,'volystep', self.dispBas,set_value=dm_to_string(self.vol_ystep)
       self->my_widget_control,'volzfrom', self.dispBas,set_value=dm_to_string(self.vol_zran[0])
       self->my_widget_control,'volzto',   self.dispBas,set_value=dm_to_string(self.vol_zran[1])
       self->my_widget_control,'volzstep', self.dispBas,set_value=dm_to_string(self.vol_zstep)
       self->my_widget_control,'volIfrom', self.dispBas,set_value=dm_to_string(self.vol_iran[0])
       self->my_widget_control,'volIto',   self.dispBas,set_value=dm_to_string(self.vol_iran[1])
       self->my_widget_control,'volSmooth',self.dispBas,set_value=dm_to_string(self.vol_smooth,/int)
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='volColor'), select=self.vol_color
       dm_set_droplist,widget_info(self.dispBas,find_by_uname='volInterp'),select=self.vol_interp
    endif
end

;reset menu item sensitivity and ticks accordingly
;keywords:
;   geomsensitive: if set to 1: instrument geometry menus will be sensitive
;                  if set to 0: instrument geometry menus will be insensitive
;                  if not set, no action
;   backgroundmenuonly: if set, only reset the background menu
pro dcs_mslice::reset_menu,geomsensitive=geomsensitive,backgroundmenuonly=backgroundmenuonly,pdt_reset=pdt_reset
    ;filemenu
    ftoolmenu = widget_info(self.tlb,find_by_uname='ftoolMenu')
    tmp = widget_info(ftoolmenu,/child)
    widget_control,ftoolmenu,sensitive=(tmp ne 0)
    self->my_widget_control,'dm_copyftpBut',ftoolmenu,sensitive=file_test(self.dirs[1],/directory,/write)
    ;background menu and parameter menu
    emptBut  = widget_info(self.tlb,find_by_uname='emptBut')
    emptBut2 = widget_info(self.tlb,find_by_uname='emptBut2')
    macsdBut = widget_info(self.tlb,find_by_uname='macsdeteffBut')
    wandeBut = widget_info(self.tlb,find_by_uname='wandeiefBut')
    if ((self.instrname eq 'macs') and (macsdBut eq 0)) or ((self.instrname ne 'macs') and (macsdBut ne 0)) or $
       ((self.instrname eq 'wand') and (wandeBut eq 0)) or ((self.instrname ne 'wand') and (wandeBut ne 0)) or $
       ((self.extraempcan_yn eq 1) and (emptBut2 eq 0)) or ((self.extraempcan_yn eq 0) and (emptBut2 ne 0)) then begin
       ;rebuild the background menu and parameter menu
       bkgrmenu = widget_info(self.tlb,find_by_uname='bkgrMenu')
       parmmenu = widget_info(self.tlb,find_by_uname='parmMenu')
       self->my_widget_control,['emptBut2','bgflBut','bgtcBut','bgdkBut','bgovBut','bgclBut','vandBut','vdclBut'],bkgrmenu,/destroy
       self->my_widget_control,['emfcBut','detoffsetBut','wandeiefBut','macsdeteffBut','ddirBut','wdirBut'],parmmenu,/destroy
       if self.extraempcan_yn then begin
          self->my_widget_control,'emptBut',bkgrmenu,set_value='Load Empty Can File(s) 1'
          void  = widget_button(bkgrmenu,value='Load Empty Can File(s) 2',uname='emptBut2')
          void  = widget_button(parmmenu,value='Change 2-Set Empty Can Factors...',uname='emfcBut',sensitive=0)
          if self.instrname eq 'wand' then begin
             void = widget_button(parmmenu,value='Change WAND Detector Offset Angle...',uname='detoffsetBut',/separator)
             void = widget_button(parmmenu,value='Change WAND Incident Wavelength...',uname='wandeiefBut')
          endif 
          if self.instrname eq 'macs' then begin
             void = widget_button(parmmenu,value='Change MACS Detector Offset Angle...',uname='detoffsetBut',/separator)
             void = widget_button(parmmenu,value='Change MACS Detector Efficiency...',uname='macsdeteffBut')
          endif    
          void  = widget_button(parmmenu,value='Set Data Directory...',uname='ddirBut',/separator)
          void  = widget_button(parmmenu,value='Set Working Directory...',uname='wdirBut')
       endif else begin
          self->my_widget_control,'emptBut',bkgrmenu,set_value='Load Empty Can File(s)'
          if self.instrname eq 'wand' then begin
           void = widget_button(parmmenu,value='Change WAND Detector Offset Angle...',uname='detoffsetBut',/separator)
           void = widget_button(parmmenu,value='Change WAND Incident Wavelength...',uname='wandeiefBut')
          endif  
          if self.instrname eq 'macs' then begin
             void = widget_button(parmmenu,value='Change MACS Detector Offset Angle...',uname='detoffsetBut',/separator)
             void = widget_button(parmmenu,value='Change MACS Detector Efficiency...',uname='macsdeteffBut')
          endif      
          void  = widget_button(parmmenu,value='Set Data Directory...',uname='ddirBut',/separator)
          void  = widget_button(parmmenu,value='Set Working Directory...',uname='wdirBut')
       endelse
       void     = widget_button(bkgrmenu,value='Load Detector Background File(s)',uname='bgflBut')
       void     = widget_button(bkgrmenu,value='Load Time Channel Background File(s)',uname='bgtcBut')
       void     = widget_button(bkgrmenu,value='Load Dark Count File(s)',uname='bgdkBut')
       void     = widget_button(bkgrmenu,value='Set Overall Background Rate...',uname='bgovBut')
       void     = widget_button(bkgrmenu,value='Clear Background',uname='bgclBut')
       void     = widget_button(bkgrmenu,value='Load Vanadium File(s)',uname='vandBut',/separator)
       void     = widget_button(bkgrmenu,value='Clear Vanadium',uname='vdclBut')
    endif
    if self.extraempcan_yn then begin
       self->my_widget_control,'emptBut2',bkgrmenu,sensitive=ptr_valid(self.bgdata)
       self->my_widget_control,'emfcBut',bkgrmenu,sensitive=ptr_valid(self.bgdata2)
    endif
    self->my_widget_control,'macsdeteffBut',sensitive=((self.instrname eq 'macs') and ptr_valid(self.eff))
    self->my_widget_control,['bgflBut','bgdkBut'],sensitive=((self.instrname ne 'wand') and (self.instrname ne 'macs'))
    self->my_widget_control,'bgovBut',sensitive=(self.instrname eq 'dcs')
    self->my_widget_control,'bgtcBut',sensitive=((self.samp_typ ne 2) and (self.instrname ne 'wand') and (self.instrname ne 'macs'))
    self->my_widget_control,'vdclBut',sensitive=((ptr_valid(self.eff) and (self.instrname ne 'macs')) or (ptr_valid(self.eff1) and (self.instrname eq 'macs')))
    self->my_widget_control,'bgclBut',sensitive=(total(ptr_valid([self.bgtchnPtr,self.bgratePtr,self.bgdetsPtr,self.bgdata])) ne 0)
    if keyword_set(backgroundmenuonly) then return
    ;file menu: add a click to the current file type
    ftypemenu  = widget_info(self.tlb,find_by_uname='ftypeMenu')
    self->my_menu_toggle,'ftype_'+dm_to_string(indgen(n_elements(*self.ftypename)),/int),self.ftype,base=ftypemenu
    self->my_widget_control,['savesliceMenu','savecutMenu','saveprojMenu'],sensitive=(ptr_valid(self.projStrPtr) and ptr_valid(self.dataStrPtr))
    ;mask menu
    maskmenu = widget_info(self.tlb,find_by_uname='maskMenu')
    self->my_widget_control,['mka3But','mkcla3But'],maskmenu,/destroy
    if self.instrname eq 'macs' then begin
       stmkmenu = widget_info(self.tlb,find_by_uname='stmkMenu')
       void     = widget_button(stmkmenu,value='Mask A3 By Angle...',uname='mka3But',/separator)
       void     = widget_button(maskmenu,value='Clear A3 Mask',uname='mkcla3But',sensitive=ptr_valid(self.a3mask))
    endif
    self->my_widget_control,'mkclBut',maskmenu,set_value=(['Clear Mask','Clear Detector Mask'])[(self.instrname eq 'macs')]
    self->my_widget_control,'mkdbMenu',sensitive=(self.instrname eq 'dcs')
    if self.instrname eq 'dcs' then self->dm_mask_detbanks
    self->my_widget_control,'mkclBut',maskmenu,sensitive=ptr_valid(self.mask)
    self->my_widget_control,'mksvBut',maskmenu,sensitive=(ptr_valid(self.a3mask) or ptr_valid(self.mask))
    self->my_widget_control,'mkplBut',maskmenu,sensitive=ptr_valid(self.dataStrPtr)
    self->my_widget_control,'mkbpBut',maskmenu,sensitive=(self.instrname ne 'macs')
    self->my_widget_control,'mkplangl',maskmenu,sensitive=((self.instrname eq 'dcs') or (self.instrname eq 'macs') or (self.instrname eq 'wand'))
    ;option menu
    self->set_instrgeom,geom=self.instrgeom,sensitive=geomsensitive
    ntype = n_elements(self.intn_typ_str)
    if self.intn_typ ge ntype or self.intn_typ lt 0 then self.intn_typ=0
    self->my_menu_toggle,'intnTyp'+dm_to_string(indgen(ntype),/int),self.intn_typ
    self->my_widget_control,'intnTyp'+dm_to_string(indgen(ntype-1)),sensitive=((self.instrname ne 'wand') and (self.instrname ne 'macs'))
    self->my_widget_control,'intnTyp'+dm_to_string([0,1,3]),sensitive=(self.instrname ne 'wand')  ;allow S(q,omega),symmetrized S(q,omega), and Chi(q,omega) for macs
    dm_set_button,widget_info(self.tlb,find_by_uname='intnEint'),self.eint_yn,onstring='E-Integrated Intensity Enabled',offstring='E-Integrated Intensity Disabled',$
       sensitive=((self.instrname ne 'wand') and (self.instrname ne 'macs'))
    ;construct macs intensity type option
    macsintmenu = widget_info(self.tlb,find_by_uname='macsintMenu')
    vaxismenu   = widget_info(self.tlb,find_by_uname='vaxisMenu')
    taxismenu   = widget_info(self.tlb,find_by_uname='taxisMenu')
    if self.instrname eq 'macs' then begin
       if macsintmenu eq 0 then begin
          itypmenu = widget_info(self.tlb,find_by_uname='itypMenu')
          macsintmenu = widget_button(itypmenu,value='MACS Detector Choice',/menu,uname='macsintMenu')
          void = dm_widget_button(macsintmenu,value='SPEC',uname='macsintSPEC')
          void = dm_widget_button(macsintmenu,value='DIFF',uname='macsintDIFF')
          void = dm_widget_button(macsintmenu,value='SPEC+DIFF',uname='macsintSPDF')
       endif
       self->my_menu_toggle,'macsint'+['SPEC','DIFF','SPDF'],self.macsintn_typ,base=macsintmenu
       if taxismenu eq 0 then begin
          taxismenu = widget_button(vaxismenu,value='Temperature Choice',/menu,uname='taxisMenu')
          void = dm_widget_button(taxismenu,value='Sample',uname='taxisSamp')
          void = dm_widget_button(taxismenu,value='Control',uname='taxisCtrl')
          void = dm_widget_button(taxismenu,value='Setpoint',uname='taxisSetp')
          void = dm_widget_button(taxismenu,value='Average Temperature of Each File',uname='avgTemp',/separator)
       endif
       self->my_menu_toggle,'taxis'+['Samp','Ctrl','Setp'],self.macstemp_typ,base=taxismenu
       dm_set_button,widget_info(taxismenu,find_by_uname='avgTemp'),self.macstempavg_yn,onstring='Average Temperature of Each File',offstring='Keep Individaul Temperature'
       widget_control,taxismenu,sensitive=self.extravaxis_yn
    endif else begin
       if macsintmenu ne 0 then widget_control,macsintmenu,/destroy
       if taxismenu   ne 0 then widget_control,taxismenu,/destroy
    endelse
    self->my_menu_toggle,'bin'+['Avg','Sum'],self.bin_avgsum   
    self->my_menu_toggle,'bincs'+['No','Yes'],self.bin_conststep
    self->my_menu_toggle,'binex'+['No','Yes'],self.bin_extlib
    if (self.instrname eq 'dcs') or (self.instrname eq 'macs') then begin ;reconstruct monitor count option menu
       self->my_menu_toggle,'mon'+['Sum','Fit'],self.mon_sumfit
       self->my_widget_control,'monSum',set_value=(['Sum','Automatic'+(finite(self.macs_mon[0])?' ('+dm_to_string(self.macs_mon[0],/exponent)+')':'')])[self.instrname eq 'macs']
       self->my_widget_control,'monFit',set_value=(['Integrated Intensity','Specify ('+dm_to_string(self.macs_mon[1],/exponent)+')'])[self.instrname eq 'macs']
       moncMenu = widget_info(self.tlb,find_by_uname='moncMenu')
       mondMenu = widget_info(moncMenu,find_by_uname='mondMenu')
       monlamb2 = widget_info(moncMenu,find_by_uname='monlamb2')
       widget_control,moncMenu,/sensitive
       if self.instrname eq 'dcs' then begin
          if monlamb2 ne 0 then widget_control,monlamb2,/destroy
          if mondMenu eq 0 then begin
             mondmenu = widget_button(moncmenu,value='Monitor Choice',/menu,/separator,uname='mondMenu')
             void     = dm_widget_button(mondmenu,value='FC0',uname='monFC0')
             void     = dm_widget_button(mondmenu,value='BM1',uname='monBM1')
             void     = dm_widget_button(mondmenu,value='Duration',uname='monTIME')
          endif 
          self->my_menu_toggle,'mon'+['FC0','BM1','TIME'],self.dcs_mon
       endif else begin
          if mondMenu ne 0 then widget_control,mondMenu,/destroy
          if monlamb2 eq 0 then monlamb2 = dm_widget_button(moncmenu,value='Apply Monitor Lambda/2 Correction',/separator,uname='monlamb2')
          dm_set_button,monlamb2,self.macs_lamb2,onstring='Apply Monitor Lambda/2 Correction',offstring='No Monitor Lambda/2 Correction'
       endelse
    endif else $
       self->my_widget_control,'moncMenu',sensitive=0
    if ptr_valid(self.absp_info) then $
       self->my_menu_toggle,'absp'+['None','Cylinder','Annulus','Sphere','Flat'],(*(self.absp_info))[0]+1 $
    else $
       self->my_menu_toggle,'absp'+['None','Cylinder','Annulus','Sphere','Flat'],0
    self->my_menu_toggle,'vaxis'+['Ortho','Arbi'],self.view_ortharbi 
    self->my_menu_toggle,'vfold'+['Mirror','Rotation'],self.fold_type
    self->change_powderline,self.powd_typ,/menuonly
    self->my_menu_toggle,'epad'+['None','Auto','Spec'],self.eadjust 
    self->my_menu_toggle,'empttol_'+['cwt','wtc'],self.empt_tol_optn
    self->my_menu_toggle,'pbg'+['Black','White'],self.pbgcol_bw  
    pdimmenu  = widget_info(self.tlb,find_by_uname='pdimMenu')
    self->my_menu_toggle,'pdchoice_'+dm_to_string(indgen(n_elements((*self.pdim_list)[0,*])),/int),self.pdim_choice,base=pdimmenu
    self->my_widget_control,'userFunction',set_button=(strlen(self.userf_str) ne 0)
    onstr  = (['Allow Extra Viewing Axis','Allow Temperature as Extra Viewing Axis'])[(self.instrname eq 'macs') or (self.instrname eq 'wand')]
    offstr = (['Disallow Extra Viewing Axis','Disallow Temperature as Extra Viewing Axis'])[(self.instrname eq 'macs') or (self.instrname eq 'wand')]
    self->my_widget_control,'extraVaxis',set_value=onstr
    dm_set_button,widget_info(self.tlb,find_by_uname='extraVaxis'),self.extravaxis_yn,onstring=onstr,offstring=offstr
    dm_set_button,widget_info(self.tlb,find_by_uname='epadCheckfirst'),self.eadjust_checkfirst,onstring='Check the First File Only',offstring='Check All Files',sensitive=(self.eadjust ne 0)
    dm_set_button,widget_info(self.tlb,find_by_uname='darkCorr'),self.dcor_yn,onstring='Dark Angle Correction Enabled',offstring='Dark Angle Correction Disabled'
    dm_set_button,widget_info(self.tlb,find_by_uname='darkTraj'),self.datr_yn,onstring='Show Dark Angle Trajectory',offstring='Disable Dark Angle Trajectory'
    dm_set_button,widget_info(self.tlb,find_by_uname='edepEff'),self.edep_yn,onstring='E-Dependent Detector Efficiency Correction Enabled',offstring='E-Dependent Detector Efficiency Correction Disabled'
    dm_set_button,widget_info(self.tlb,find_by_uname='errBar'),1-self.errbar,onstring='Error Bar Disabled',offstring='Error Bar Enabled'
    self->my_widget_control,['eAdjust','edepEff','viewSpurion'],sensitive=((self.instrname ne 'wand') and (self.instrname ne 'macs'))
    self->change_samptyp,self.samp_typ,pdt_reset=pdt_reset
    ;window menu
    dm_set_button,widget_info(self.tlb,find_by_uname='viewProj'),self.view[0],onstring='Hide Projection',offstring='Show Projection'
    dm_set_button,widget_info(self.tlb,find_by_uname='viewDisp'),self.view[1],onstring='Hide Display',offstring='Show Display'
end

;reset projection and display window
pro dcs_mslice::reset_pdt
    WIDGET_CONTROL,/HOURGLASS

    if ~widget_info(self.projBas,/valid_id) then return
    
    widget_control,self.tlb,update=0
    geom1_old = widget_info(self.projBas,/geometry)
    geom2_old = widget_info(self.dispBas,/geometry)
    widget_control,self.dispBas,/destroy
    widget_control,self.projBas,/destroy
    self->reset_projection
    self->reset_display
    geom1  = widget_info(self.projBas,/geometry)
    geom2  = widget_info(self.dispBas,/geometry)
    margin = total(1-self.view)*geom1.margin*2*(!version.os_family eq 'Windows')
    if self.idl_version ge 5.5 then begin
       self.geom[1] = self.geom[1]+geom1.ysize+geom2.ysize-(geom1_old.ysize+geom2_old.ysize)
       widget_control,self.tlb,ysize=self.geom[1]-margin
    endif else begin
       self.geom[1] = self.geom[1]+geom1.scr_ysize+geom2.scr_ysize-(geom1_old.scr_ysize+geom2_old.scr_ysize)
       widget_control,self.tlb,scr_ysize=self.geom[1]-margin
    endelse
    widget_control,self.tlb,/update
    widget_control,self.projBas,/update,/map
    widget_control,self.dispBas,/update,/map
end

;save cuts in files
;keyword:
;   dave:   if set, save cut as dave files
;   dPtr:   if present, will pass the cut data as a dave pointer
;   xye:    if set, save cut as x, y, yerr 3-column file
;   file:   full filename, including the directory, if absent, a dialog window will show up
pro dcs_mslice::save_cut,dave=dave,dPtr=dPtr,xye=xye,file=file
    if (~ self->pd_valid()) then return
    ext = ''
    if keyword_set(dave) then ext = 'dave'
    if keyword_set(xye)  then ext = 'xye'
    if (~ arg_present(dPtr)) then begin
       if strlen(ext) eq 0 then return
       path = self.dirs[1]
       if n_elements(file) eq 0 then begin
          file = dm_choose_file(ext,/write,dialog_parent=self.tlb,title='Save cut as '+ext+' file.',path=path)
          file = file[0]
       endif
       if strlen(file) eq 0 then return
       self->set_dir,workDir=path
       openw,unit,file,/get_lun,error=openerr
       if openerr ne 0 then begin
          ok = dialog_message("Can't write in "+file,/error,dialog_parent=self.tlb,/center)
          return
       endif
    endif

    ;select data
    self->dm_choose_cutdata,xdat=xdat,ydat=ydat,yerr=yerr,xtit=xtit,ytit=ytit
    if (~ self.errbar) and (n_elements(yerr) ne 0) then tmp=temporary(yerr)   ;destroy yerr

    if keyword_set(xye) then begin
       for i=0L,n_elements(xdat)-1 do begin
           if n_elements(yerr) ne 0 then printf,unit,xdat[i],ydat[i],yerr[i] $
           else printf,unit,xdat[i],ydat[i]
       endfor
    endif

    if keyword_set(dave) or arg_present(dPtr) then begin
       if n_elements(yerr) ne 0 then $
          ret = create_dave_pointer(    $
                dPtr,                   $
                qty     = ydat,         $
                qtlabel = ytit,         $
                err     = yerr,         $
                xvals   = xdat,         $
                xtype   = 'POINT',      $
                xlabel  = xtit          $
          ) $
       else $
          ret = create_dave_pointer(    $
                dPtr,                   $
                qty     = ydat,         $
                qtlabel = ytit,         $
                xvals   = xdat,         $
                xtype   = 'POINT',      $
                xlabel  = xtit          $
          )
       if n_elements(unit) ne 0 then begin
          davePtr = dPtr
          save,davePtr,filename=file
          heap_free,dPtr
       endif
    endif

    if n_elements(unit) ne 0 then free_lun,unit
end

;save slice in files
;keywords:
;   dave:   if set, save slice as grouped DAVE file, for PAN fitting
;   dPtr:   if present, will pass the slice data as a dave pointer
;   iexy:   if set, save slice as Int, Err, X, Y 4-column file
;   psutxt: for PSU, see Craig Brown's 08/03/2012 email
;   spe:    if set, save slice as spe file
;   xyie:   if set, save slice as X, Y, Int, Err matrix file
;   file:   full filename, including the directory, if absent, a dialog window will show up
;   smooth: if present, smooth data according to the smooth value 
;   filler: if present, used as the filler value for empty data, useful in script mode
pro dcs_mslice::save_slice,dave=dave,dPtr=dPtr,file=file,filler=filler,iexy=iexy,psutxt=psutxt,spe=spe,xyie=xyie,smooth=smooth
    if (~ self->pd_valid()) then return
    ext = ''
    if keyword_set(dave)   then ext = 'dave'
    if keyword_set(iexy)   then ext = 'iexy'
    if keyword_set(spe)    then ext = 'spe'
    if keyword_set(xyie)   then ext = 'xyie'
    if keyword_set(psutxt) then ext = 'txt'

    dcs_filler = -1e20
    if finite(self.mask_filler) and (self.samp_typ ne -1) then dcs_filler = self.mask_filler 
    
    if (~ arg_present(dPtr)) then begin
       if strlen(ext) eq 0 then return
       path = self.dirs[1]
       if n_elements(file) eq 0 then begin
          file = dm_choose_file(ext,/write,dialog_parent=self.tlb,title='Save slice as '+ext+' file.',path=path)
          file = file[0]
       endif
       if strlen(file) eq 0 then return
       self->set_dir,workDir=path
       if (~ keyword_set(spe)) then begin
          openw,unit,file,/get_lun,error=openerr
          if openerr ne 0 then begin
             ok = dialog_message("Can't write in "+file,/error,dialog_parent=self.tlb,/center)
             return
          endif
       endif
    endif

    ;select data
    tmp = self.slice_smooth
    if n_elements(smooth) eq 0 then begin
       if self.slice_smooth gt 0 then begin
          info = 'Do you want to '+(['save','send'])[arg_present(dPtr)]+' the smoothed data?'
          ok = dialog_message(info,/question,dialog_parent=self.tlb,/center)
          if strlowcase(ok) eq 'no' then self.slice_smooth = 0
       endif
    endif else self.slice_smooth = smooth
    self->dm_choose_slicedata,xdat=xdat,ydat=ydat,zdat=zdat,zerr=zerr,xtit=xtit,ytit=ytit,zran=zran,zmin=zmin
    self.slice_smooth = tmp
    index = where(finite(zdat,/nan),count)
    if count ne 0 then begin
       if n_elements(filler) ne 0 then begin
          dcs_filler = filler[0]
          notok = 0b
       endif else $
          notok = 1b
       while(notok) do begin
          dcs_filler = dm_dialog_input('filler value:',/float,default=dcs_filler,info='This value is used for empty grids that contain no data.',dialog_parent=self.tlb)
          notok = finite(dcs_filler,/nan)
       endwhile
       if self.samp_typ ne -1 then self.mask_filler = dcs_filler
       zdat[index] = dcs_filler
       zerr[index] = -1.0               ;signifies missing data
    endif
    
    widget_control,/hourglass
    
    if keyword_set(iexy)   then self->write_iexy,unit,xdat,ydat,zdat,zerr
    if keyword_set(spe)    then dm_write_spe,file,xdat,ydat,zdat,zerr,xtit=xtit,ytit=ytit,ztit=self.intn_typ_str[self.intn_typ],group_leader=self.tlb
    if keyword_set(xyie)   then self->write_xyie,unit,xdat,ydat,zdat,zerr
    if keyword_set(psutxt) then self->write_psutxt,unit,xdat,ydat,zdat,zerr
    if keyword_set(dave) or arg_present(dPtr) then begin
       nx = n_elements(xdat) & ny = n_elements(ydat)
       if count ne 0 then begin ;creat mask
          mask = bytarr(nx,ny)+1b
          mask[index] = 0b
          mask = bits2bytes(transpose(mask))
       endif
       ;contruct DAVE pointer
       xunit = ''
       yunit = ''
       if self.samp_typ eq 0 then begin
          xtit  = self.view_label[self.view_u1typ]
          xunit = self.view_unit[self.view_u1typ]
          ytit  = self.view_label[self.view_u2typ]
          yunit = self.view_unit[self.view_u2typ]
          ;replace \AA with string('c5'XB)
          pos = strpos(xunit,'\AA') 
          if pos ne -1 then xunit = strmid(xunit,0,pos)+string('c5'XB)+strmid(xunit,pos+3)
          pos = strpos(yunit,'\AA') 
          if pos ne -1 then yunit = strmid(yunit,0,pos)+string('c5'XB)+strmid(yunit,pos+3)
       endif
       dPtr = dm_define_pointer()
       ptr_free,(*(*dPtr).dataStrPtr).commonStr.histPtr
       (*(*dPtr).dataStrPtr).commonStr.histPtr   = ptr_new({qty:transpose(temporary(zdat)),err:transpose(temporary(zerr)),x:ydat,y:xdat})
	     (*(*dPtr).dataStrPtr).commonStr.xunits    = yunit
	     (*(*dPtr).dataStrPtr).commonStr.xlabel    = ytit
	     (*(*dPtr).dataStrPtr).commonStr.yunits    = xunit
	     (*(*dPtr).dataStrPtr).commonStr.ylabel    = xtit
	     (*(*dPtr).dataStrPtr).commonStr.histlabel = self.intn_typ_str[self.intn_typ]
       if count ne 0 then begin ;mask
          ;add PAN_MASK and DCS_FILLER info to specificPtr
          specific = (*(*(*dPtr).dataStrPtr).specificPtr)
          tags = strupcase(tag_names(specific))
          loc = where(tags eq 'PAN_MASK',count)
          if count eq 0 then specific = create_struct('PAN_MASK',ptr_new(mask),specific) $
          else *specific.pan_mask = mask
          loc = where(tags eq 'DCS_FILLER',count)
          if count eq 0 then specific = create_struct('DCS_FILLER',dcs_filler,specific) $
          else specific.dcs_filler = dcs_filler
    	    (*(*(*dPtr).dataStrPtr).specificPtr) = specific
       endif
       if n_elements(unit) ne 0 then begin
          davePtr = dPtr
          save,davePtr,filename=file
          heap_free,dPtr
       endif
    endif

    if n_elements(unit) ne 0 then free_lun,unit
end

;save the projection volume
;keywords:
;   txt:    if set, save as a txt file, the only one implemented
pro dcs_mslice::save_projection,txt=txt
    if self.samp_typ eq - 1 then begin
       title = 'Save all data as a text file.' 
       nitem = n_elements((*self.dataPtr).label)
       ndat  = n_elements((*self.dataPtr).data[*,0])
    endif else begin
       title = 'Save projection as a text file.'
       nen   = n_elements((*self.projStrPtr).en)
       n2th  = n_elements((*self.projStrPtr).two_theta)
    endelse
    
    path = self.dirs[1]
    file = dm_choose_file(ext,/write,dialog_parent=self.tlb,title=title ,path=path)
    file = file[0]
    if strlen(file) eq 0 then return
    self->set_dir,workDir=path
    openw,unit,file,/get_lun,error=openerr
    if openerr ne 0 then begin
       ok = dialog_message("Can't write in "+file,/error,dialog_parent=self.tlb,/center)
       return
    endif
    
    widget_control,/hourglass
    
    if !version.os_family eq 'Windows' then sep=string(9b) else sep='     '
    
    case self.samp_typ of
      -1: begin
          tmp = ''
          for i=2,nitem-1 do tmp = tmp+sep+(*self.dataPtr).label[i]
          tmp = tmp+sep+(*self.dataPtr).label[0]+sep+(*self.dataPtr).label[1]
          printf,unit,tmp
          for i=0L,ndat-1L do $
              printf,unit,reform((*self.dataPtr).data[i,[2+indgen(nitem-2),0,1]])
          end
       0: begin
          if self.instrname eq 'macs' then begin  ;with T axis
             if self.extravaxis_yn then begin
                printf,unit,sep,'|Q|(A^-1)',sep,'E(meV)',sep,'T (K)',sep,(['Ispec','Idiff','Ispec+diff'])[self.macsintn_typ],sep,(['dIspec','dIdiff','dIspec+diff'])[self.macsintn_typ]
                for i=0L,nen-1 do begin
                    for j=0L,n2th-1 do begin
                        if self.macsintn_typ eq 2 then $
                           printf,unit,(*self.projStrPtr).Q[1,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).temperature[i],' ',(*self.projStrPtr).qty[0,i,j]+(*self.projStrPtr).qty[1,i,j],' ',$
                              sqrt(((*self.projStrPtr).err[0,i,j])^2+((*self.projStrPtr).err[1,i,j])^2) $
                        else $
                           printf,unit,(*self.projStrPtr).Q[self.macsintn_typ,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).temperature[i],' ',(*self.projStrPtr).qty[self.macsintn_typ,i,j],' ',$
                              (*self.projStrPtr).err[self.macsintn_typ,i,j]
                    endfor
                endfor
             endif else begin
                printf,unit,sep,'|Q|(A^-1)',sep,'E(meV)',sep,(['Ispec','Idiff','Ispec+diff'])[self.macsintn_typ],sep,(['dIspec','dIdiff','dIspec+diff'])[self.macsintn_typ]
                for i=0L,nen-1 do begin
                    for j=0L,n2th-1 do begin
                        if self.macsintn_typ eq 2 then $
                           printf,unit,(*self.projStrPtr).Q[1,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[0,i,j]+(*self.projStrPtr).qty[1,i,j],' ',$
                              sqrt(((*self.projStrPtr).err[0,i,j])^2+((*self.projStrPtr).err[1,i,j])^2) $
                        else $
                           printf,unit,(*self.projStrPtr).Q[self.macsintn_typ,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[self.macsintn_typ,i,j],' ',$
                              (*self.projStrPtr).err[self.macsintn_typ,i,j]
                    endfor
                endfor
             endelse
          endif else begin
             printf,unit,sep,'|Q|(A^-1)',sep,'E(meV)',sep,'I',sep,'dI'
             for i=0L,nen-1 do begin
                 for j=0L,n2th-1 do begin
                     printf,unit,(*self.projStrPtr).Q[i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[i,j],' ',(*self.projStrPtr).err[i,j]
                 endfor
             endfor
          endelse
          end
       1: begin
          if self.extravaxis_yn then begin
             if self.instrname eq 'macs' then begin ;with T axis
                printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,'T (K)',sep,'E(meV)',sep,(['Ispec','Idiff','Isepc+diff'])[self.macsintn_typ],sep,$
                    (['dIspec','dIdiff','dIspec+diff'])[self.macsintn_typ]
                for i=0L,nen-1 do begin
                    for j=0L,n2th-1 do begin
                        if self.macsintn_typ eq 2 then $
                           printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).temperature[i],' ',$
                              (*self.projStrPtr).qty[0,i,j]+(*self.projStrPtr).qty[1,i,j],' ',sqrt(((*self.projStrPtr).err[0,i,j])^2+((*self.projStrPtr).err[1,i,j])^2) $
                        else $ 
                           printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).temperature[i],' ',(*self.projStrPtr).qty[self.macsintn_typ,i,j],' ',$
                              (*self.projStrPtr).err[self.macsintn_typ,i,j]
                    endfor
                endfor
             endif else begin
                printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,self.traj_type[2],sep,'E(meV)',sep,'I',sep,'dI'
                for i=0L,nen-1 do begin
                    for j=0L,n2th-1 do begin
                        printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).V[2,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[i,j],' ',$
                              (*self.projStrPtr).err[i,j]
                    endfor
                endfor
             endelse
          endif else begin
             if self.instrname eq 'macs' then begin
                printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],'E(meV)',sep,(['Ispec','Idiff','Ispec+diff'])[self.macsintn_typ],sep,(['dIspec','dIdiff','dIspec+diff'])[self.macsintn_typ]
                for i=0L,nen-1 do begin
                    for j=0L,n2th-1 do begin
                        if self.macsintn_typ eq 2 then $
                           printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[0,i,j]+(*self.projStrPtr).qty[1,i,j],' ',$
                              sqrt(((*self.projStrPtr).err[0,i,j])^2+((*self.projStrPtr).err[1,i,j])^2) $
                        else $
                           printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[self.macsintn_typ,i,j],' ',$
                              (*self.projStrPtr).err[self.macsintn_typ,i,j]
                    endfor
                endfor
             endif else begin
                 printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],'E(meV)',sep,'I',sep,'dI'
                for i=0L,nen-1 do begin
                    for j=0L,n2th-1 do begin
                        printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[i,j],' ',(*self.projStrPtr).err[i,j]
                    endfor
                endfor
             endelse
          endelse
          end
       2: begin
          tmp = size((*self.projStrPtr).V)
          if tmp[0] eq 2 then nfile=1L else nfile=tmp[3]
          if self.extravaxis_yn then begin
             if self.instrname eq 'wand' then begin
                printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,'T (K)',sep,'I',sep,'dI'
                for i=0L,n2th-1 do begin
                    for j=0L,nfile-1 do begin
                        printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).temperature[j],' ',(*self.projStrPtr).qty[i,j],' ',(*self.projStrPtr).err[i,j]
                    endfor
                endfor
             endif else begin
                printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,self.traj_type[2],sep,'I',sep,'dI'
                for i=0L,n2th-1 do begin
                    for j=0L,nfile-1 do begin
                        printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],(*self.projStrPtr).V[2,i,j],' ',(*self.projStrPtr).qty[i,j],' ',(*self.projStrPtr).err[i,j]
                    endfor
                endfor
             endelse
          endif else begin
             printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,'I',sep,'dI'
             for i=0L,n2th-1 do begin
                 for j=0L,nfile-1 do begin
                     printf,unit,(*self.projStrPtr).V[0,i,j],(*self.projStrPtr).V[1,i,j],' ',(*self.projStrPtr).qty[i,j],' ',(*self.projStrPtr).err[i,j]
                 endfor
             endfor
          endelse
          end
       3: begin
          tmp = size((*self.projStrPtr).V)
          if tmp[0] eq 3 then nfile=1L else nfile=tmp[4]
          if self.extravaxis_yn then begin
             printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,self.traj_type[2],sep,'E(meV)',sep,'I',sep,'dI'
             for i=0L,nen-1 do begin
                 for j=0L,n2th-1 do begin
                     for k=0L,nfile-1 do begin
                         printf,unit,(*self.projStrPtr).V[0,i,j,k],(*self.projStrPtr).V[1,i,j,k],(*self.projStrPtr).V[2,i,j,k],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[i,j,k],' ',(*self.projStrPtr).err[i,j,k]
                     endfor
                 endfor
             endfor
          endif else begin
             printf,unit,sep,self.traj_type[0],sep,self.traj_type[1],sep,'E(meV)',sep,'I',sep,'dI'
             for i=0L,nen-1 do begin
                 for j=0L,n2th-1 do begin
                     for k=0L,nfile-1 do begin
                         printf,unit,(*self.projStrPtr).V[0,i,j,k],(*self.projStrPtr).V[1,i,j,k],(*self.projStrPtr).en[i],' ',(*self.projStrPtr).qty[i,j,k],' ',(*self.projStrPtr).err[i,j,k]
                     endfor
                 endfor
             endfor
          endelse
          end   
    endcase   
    
    free_lun,unit
end

;save the volume data
;keywords:
;   slice:  if present,save slices(0: x slice, 1: y slice, 2: z slice), otherwise save all volume data in one text file
;   file:   full filename, including the directory, if absent, a dialog window will show up  
;   filler: if present, used as the filler value for empty data, useful in script mode 
;   iexy:   if set, save slice as Int, Err, X, Y 4-column file
;   psutxt: for PSU, see Craig Brown's 08/03/2012 email
;   spe:    if set, save slice as spe file
;   xyie:   if set, save slice as X, Y, Int, Err matrix file
;   smooth: if present, smooth data according to the smooth value 
pro dcs_mslice::save_volume,slice=slice,file=file,filler=filler,iexy=iexy,psutxt=psutxt,spe=spe,xyie=xyie,smooth=smooth
    ext = 'multi-column text'
    if keyword_set(iexy)   then ext = 'iexy'
    if keyword_set(spe)    then ext = 'spe'
    if keyword_set(xyie)   then ext = 'xyie'
    if keyword_set(psutxt) then ext = 'txt'
    if n_elements(slice) ne 0 then begin
       if (slice[0] lt 0) or (slice[0] gt 2) then return
       if ext eq 'multi-column text' then begin
          spe = 1b
          ext = 'spe'   ;default spe file type for slices
       endif
    endif
     
    path = self.dirs[1]
    if n_elements(file) eq 0 then begin
       file = dm_choose_file((ext eq 'multi-column text')?'txt':ext,/write,dialog_parent=self.tlb,title='Save volume data as '+ext+'  file.',path=path)
       file = file[0]
    endif
    if strlen(file) eq 0 then return
    self->set_dir,workDir=path
    
    dcs_filler = -1e20
    if finite(self.mask_filler) and (self.samp_typ ne -1) then dcs_filler = self.mask_filler 
    
    ;select data
    tmp = self.vol_smooth
    if n_elements(smooth) eq 0 then begin
       if self.vol_smooth gt 0 then begin
          info = 'Do you want to save the smoothed data?'
          ok = dialog_message(info,/question,dialog_parent=self.tlb,/center)
          if strlowcase(ok) eq 'no' then self.vol_smooth = 0
       endif  
    endif else self.vol_smooth = smooth
    self->dm_choose_volumedata,xdat=xdat,ydat=ydat,zdat=zdat,idat=idat,ierr=ierr,xtit=xtit,ytit=ytit,ztit=ztit

    self.vol_smooth = tmp
    itmp = where(finite(idat,/nan),count)
    if count ne 0 then begin
       if n_elements(filler) ne 0 then begin
          dcs_filler = filler[0]
          notok = 0b
       endif else $
          notok = 1b
       while(notok) do begin
          dcs_filler = dm_dialog_input('filler value:',/float,default=dcs_filler,info='This value is used for empty grids that contain no data.',dialog_parent=self.tlb)
          notok = finite(dcs_filler,/nan)
       endwhile
       if self.samp_typ ne -1 then self.mask_filler = dcs_filler
       idat[itmp] = dcs_filler
       ierr[itmp] = -1 
       itmp = -1
    endif
    widget_control,/hourglass  ;busy signal
    
    if n_elements(slice) eq 0 then begin  ;single text file
       openw,unit,file,/get_lun,error=openerr
       if openerr ne 0 then begin
          ok = dialog_message("Can't write in "+file,/error,dialog_parent=self.tlb,/center)
          return
       endif
       if !version.os_family eq 'Windows' then sep=string(9b) else sep='     '
       printf,unit,sep,xtit,sep,ytit,sep,ztit,sep,'I',sep,'dI'
       for i=0L,n_elements(xdat)-1 do begin
           for j=0L,n_elements(ydat)-1 do begin
               for k=0L,n_elements(zdat)-1 do begin
                   printf,unit,xdat[i],' ',ydat[j],' ',zdat[k],' ',idat[i,j,k],' ',ierr[i,j,k]
               endfor
           endfor    
       endfor 
       free_lun,unit
    endif else begin
       tmpfile = file_basename(file,'.'+ext,/fold_case)
       nxyz = size(idat,/dim)
       ok = dialog_message(['The volume slices will be saved as '+dm_to_string(nxyz[slice])+' '+tmpfile+'_'+(['x','y','z'])[slice]+'=####.'+ext+' files.',$
               'Do you want to continue?'],/question,dialog_parent=self.tlb,/center,title='Please confirm')
       if ok eq 'No' then return
       file = file_dirname(file)+self.pathsep+tmpfile
       case slice of 
         0: begin
            xx = ydat
            yy = zdat      
            xtit = ytit & ytit = ztit
            end
         1: begin
            xx = xdat
            yy = zdat
            ytit = ztit
            end
         2: begin
            xx = xdat
            yy = ydat    
            end
       endcase 
       for i=0L,nxyz[slice]-1L do begin
           case slice of 
             0: begin
                zz = reform(idat[i,*,*])
                ee = reform(ierr[i,*,*])
                tt = xdat[i]      
                end
             1: begin
                zz = reform(idat[*,i,*])
                ee = reform(ierr[*,i,*])    
                tt = ydat[i]      
                end
             2: begin
                zz = reform(idat[*,*,i])
                ee = reform(ierr[*,*,i]) 
                tt = zdat[i]         
                end
           endcase 
           tmpfile = file+'_'+(['x','y','z'])[slice]+'='+dm_to_string(tt,resolution=4)+'.'+ext
           if (~keyword_set(spe)) then begin
              openw,unit,tmpfile,/get_lun,error=openerr
              if openerr ne 0 then begin
                 ok = dialog_message("Can't write in "+file,/error,dialog_parent=self.tlb,/center)
                 return
              endif
           endif
           if keyword_set(iexy)   then self->write_iexy,unit,xx,yy,zz,ee
           if keyword_set(spe)    then begin
              dm_write_spe,tmpfile,xx,yy,zz,ee,xtit=xtit,ytit=ytit,ztit=self.intn_typ_str[self.intn_typ],group_leader=self.tlb,error=error
              if error ne 0 then return
           endif
           if keyword_set(xyie)   then self->write_xyie,unit,xx,yy,zz,ee
           if keyword_set(psutxt) then self->write_psutxt,unit,xx,yy,zzt,ee
           if n_elements(unit) ne 0 then free_lun,unit
       endfor
    endelse
end

;script commands
; parameter: 
;     command:   a string or string array of commands. Accepted commands are:
;                'load_data' :     filename, directory, and eief keywords need to be present
;                'load_emptycan':  filename and directory keywords need to be present
;                'load_parameter': filename and directory keywords need to be present
;                'calc_projection':temperature keywords need to be present if used, dwu2 keywords are optional
;                'save_cut':       filename and directory keyword need to be present, extra keywords are xye=xye,dave=dave
;                'save_slice':     filename, directory keywords need to be present, smooth and filler keywords are optional,
;                                  extra keywords are dave=dave,iexy=iexy,spe=spe,xyie=xyie,psutxt=psutxt
;                'save_volume':    filename, directory keywords need to be present, smooth and filler keywords are optional
;                                  extra keywords are slice=slice,iexy=iexy,spe=spe,xyie=xyie,psutxt=psutxt
;                'set_parameter':  name and value keywords need to be present
;                'keep_current_plot': keep the current plot window, new plots will be in a new plot window
;                'plot_cut':       plotting cut, legend and title keywords can be used
;                'plot_slice':     plotting slice, title keyword can be used
;                'plot_volume':    plotting volume, title keyword can be used
; keyword:
;     directory: a string
;     dwu2:      u^2 value in the Debye-Waller factor correction,default=0
;     filename:  a string or string array of file names, excluding directory
;     eief:      efixed for loading files that don't have the eief information
;     filler:    the filler value for empty grid data, default=-1e20
;     legend:    a string for the plot legend
;     name:      name of the wid to be set_parameter
;     overplot:  overplot flag of 'plot_cut' command
;     resolution:used when command is 'set_parameter', in dm_to_string()
;     smooth:    smooth level for saving slice data and volume data, default is 0
;     surfplot:  surface plot flag for 'plot_slice' command
;     temperature: sample temperature for intensity types that requires a temperature in projection calculation
;     title:     a string for the plot title
;     value:     vlaue of the set_parameter
pro dcs_mslice::script,command,directory=directory,dwu2=dwu2,eief=eief,filename=filename,filler=filler,legend=legend,name=name,overplot=overplot,resolution=resolution,smooth=smooth,$
    surfplot=surfplot,temperature=temperature,title=title,value=value,_ref_extra=_extra   
    if n_elements(smooth) eq 0 then smooth = 0
    if n_elements(filler) eq 0 then filler = -1e20
    if n_elements(dwu2)   eq 0 then dwu2   = 0.0
    if n_elements(title) ne 0 then begin
       self.titl_lab = strtrim(title[0],2)
       self->my_widget_control,'titlLab',set_value=self.titl_lab
    endif
    if n_elements(legend) ne 0 then begin
       self.legd_lab = strtrim(legend[0],2)
       self->my_widget_control,'legdLab',set_value=self.legd_lab
    endif
    if keyword_set(overplot) then begin ;shift the color to the next one in the list
       self.cut_color = (self.cut_color+1) mod (n_elements(*self.colors))
       if self.view[1] or (self.samp_typ eq -1) then dm_set_droplist,widget_info(self.dispBas,find_by_uname='cutColor'),select=self.cut_color
    endif
    for i=0,n_elements(command)-1 do begin
        case strlowcase(command[i]) of
          'load_data':       ok = self->dm_filetools('loadbut',filename=filename,directory=directory,eief=eief,/scriptmode)
          'load_emptycan':   ok = self->dm_filetools('emptbut',filename=filename,directory=directory,/scriptmode)
          'load_parameter': begin
              if n_elements(filename)  eq 0 then break
              if n_elements(directory) eq 0 then directory=self.dirs[1]
              self->dm_load_parameter,directory[0]+self.pathsep+filename[0]
              end
          'calc_projection': self->dm_calc_projection,temperature=temperature,dwu2=dwu2
          'save_cut': begin
              if n_elements(filename)  eq 0 then break
              if n_elements(directory) eq 0 then directory=self.dirs[1]
              self->save_cut,file=directory[0]+self.pathsep+filename[0],_extra=_extra
              end
          'save_slice': begin
              if n_elements(filename)  eq 0 then break
              if n_elements(directory) eq 0 then directory=self.dirs[1]
              self->save_slice,file=directory[0]+self.pathsep+filename[0],smooth=smooth,filler=filler,_extra=_extra
              end
          'save_volume': begin
              if n_elements(filename)  eq 0 then break
              if n_elements(directory) eq 0 then directory=self.dirs[1]
              self->save_volume,file=directory[0]+self.pathsep+filename[0],smooth=smooth,filler=filler,_extra=_extra
              end
          'set_parameter':begin
              n_name = n_elements(name) & n_value = n_elements(value)
              if (n_name eq 0) or (n_value eq 0) then break
              txtevent = {widget_text_ch} & txtevent.top = self.tlb & txtevent.handler = self.tlb
              ;recognized unames
              ft = ['from','to'] & fts = [ft,'step']
              unames = ['intn','titl','legd']+'Lab' 
              params = ['intn','titl','legd']+'_lab'
              n_fbas = n_elements(unames)                                   ;in self.fileBas
              unames = [unames,'latt'+['A','B','C','AA','BB','CC'],'orie'+['U'+['x','y','z'],'V'+['x','y','z'],'Psi'],$
                        'viewU'+['1'+['h','k','l','lab'],'2'+['h','k','l','lab'],'3'+['h','k','l','lab']]]
              params = [params,'latt_parm['+['0','1','2','3','4','5']+']','orie_'+['u['+['0','1','2'],'v['+['0','1','2']]+']','orie_psi',$
                        'view_u'+['1'+['[0]','[1]','[2]','lab[(self.samp_typ ge 1)]'],'2'+['[0]','[1]','[2]','lab[(self.samp_typ ge 1)]'],'3'+['[0]','[1]','[2]','lab']]]
              n_pbas = n_elements(unames)                                   ;in self.projBas
              unames = [unames,'dispsel'+ft+'_T0','dispsel'+ft+'_T1','dispx'+fts,'dispy'+fts,'dispI'+ft,'dispSmooth','cut'+['From','To','Step'],'cutsel'+ft+'_T0','cutsel'+ft+'_T1',$
                        'cutsel'+ft+'_T2','cutY'+ft,'volx'+fts,'voly'+fts,'volz'+fts,'volsel'+ft+'_T0','volI'+ft,'volSmooth'] 
              params = [params,'slice_t1ran['+['0','1']+']','slice_t2ran['+['0','1']+']','slice_xran['+['0','1']+']','slice_xstep','slice_yran['+['0','1']+']','slice_ystep',$
                        'slice_iran['+['0','1']+']','slice_smooth','cut_ran['+['0','1']+']','cut_step','cut_t1ran['+['0','1']+']','cut_t2ran['+['0','1']+']','cut_t3ran['+['0','1']+']',$
                        'cut_yran['+['0','1']+']','vol_xran['+['0','1']+']','vol_xstep','vol_yran['+['0','1']+']','vol_ystep','vol_zran['+['0','1']+']','vol_zstep','volsel_tft1['+['0','1']+']',$
                        'vol_iran['+['0','1']+']','vol_smooth']
              n_dbas = n_elements(unames)                                   ;in self.dispBas
              unamel = strlowcase(unames) ;lower case version
              bvalid = widget_info([self.fileBas,self.projBas,self.dispBas],/valid_id)
              for i=0L,n_name-1 do begin
                  ind = where(unamel eq strlowcase(name[i]),count)
                  if count eq 0 then continue
                  tmpvalue = value[i<(n_value-1)]
                  txtevent.id = 0
                  if (ind lt n_fbas) and bvalid[0] then begin
                     txtevent.id = widget_info(self.fileBas,find_by_uname=unames[ind[0]])
                  endif else if (ind lt n_pbas) and bvalid[1] then begin
                     txtevent.id = widget_info(self.projBas,find_by_uname=unames[ind[0]])
                  endif else if bvalid[2] then begin
                     txtevent.id = widget_info(self.dispBas,find_by_uname=unames[ind[0]])
                  endif
                  if txtevent.id ne 0 then begin  ;prefered
                     widget_control,txtevent.id,set_value=dm_to_string(tmpvalue,resolution=resolution),_extra=_extra
                     self->event,txtevent
                  endif else begin  ;not desired
                     ok = execute('self.'+params[ind[0]]+'=tmpvalue',1,1)
                     if ~ok then begin
                        ok = dialog_message(!error_state.msg,/error,dialog_parent=self.tlb,/center)
                     endif
                  endelse
              endfor
              end
          'keep_current_plot': if obj_valid(self.current) then begin
              self.current->setproperty,/keep
              self->dm_add_plotwindow,2,object=self.current
              end    
          'plot_cut':        self->dm_plot_cut,overplot=overplot
          'plot_slice':      self->dm_plot_slice,surfplot=surfplot
          'plot_volume':     self->dm_plot_volume
          else:
        endcase
    endfor
end

;set data directory and work directory
;setting work directory will also set the ftp buffer directory if the current one is invalid
pro dcs_mslice::set_dir,dataDir=dataDir,workDir=workDir
    defsysv,'!dave_defaults',exists=exists
    if exists then exists = ptr_valid(!dave_defaults)
    if (n_elements(dataDir) eq 0) and (n_elements(workDir) eq 0) and exists then begin
       if (file_test((*!dave_defaults).datDir,/directory,/read))[0]   then self.dirs[0] = (*!dave_defaults).datDir
       if (file_test((*!dave_defaults).workDir,/directory,/write))[0] then begin
          self.dirs[1] = (*!dave_defaults).workDir
          if ~(file_test(self.dirs[2],/directory,/write)) then self.dirs[2] = self.dirs[1]
       endif
    endif
    if n_elements(dataDir) ne 0 then begin
       if file_test(dataDir[0],/directory,/read)  then begin
          self.dirs[0] = dataDir[0]
          if exists then (*!dave_defaults).datDir = dataDir[0]
       endif
    endif
    if n_elements(workDir) ne 0 then begin
       if file_test(workDir[0],/directory,/write) then begin
          self.dirs[1] = workDir[0]
          if exists then (*!dave_defaults).workDir = workDir[0]
          if ~file_test(self.dirs[2],/directory,/write) then begin
             self.dirs[2] = self.dirs[1] 
             if obj_valid(self.fileSel) then begin
                self.fileSel->getproperty,ncnrftp=ncnrftp
                if ~ncnrftp then self.fileSel->set_ftpbuffer,self.dirs[2]
             endif
          endif
       endif
    endif 
    self->my_widget_control,'dm_copyftpBut',sensitive=file_test(self.dirs[1],/directory,/write) 
end

;set the buttons after projection is changed
;keyword:
;   clearprojection: if set, clear self.projStrPtr pointer
pro dcs_mslice::set_display_button,clearprojection=clearprojection
    if keyword_set(clearprojection) then ptr_free,self.projStrPtr
    pd_valid = self->pd_valid(/nomessage)
    index = ((self.instrname eq 'macs' and self.extravaxis_yn) or (self.samp_typ eq 1) or (self.samp_typ eq 2 and self.extravaxis_yn) or (self.samp_typ eq 3))  
    self->my_widget_control,'savevolumeMenu',sensitive=(pd_valid and index)
    self->my_widget_control,['savesliceMenu','savecutMenu','saveprojMenu'],sensitive=pd_valid
    self->my_widget_control,'viewSpurion',sensitive=ptr_valid(self.dataStrPtr) and ptr_valid(self.projStrPtr) and (self.samp_typ lt 2) and (self.instrgeom eq 0)
    if ~self.view[1] then return
    self->my_widget_control,['dispFitSlice','dispPlotSlice','dispoPltSlice','dispSurfSlice','cutFit','cutPlot','cutoPlot','volPlot','voloPlot'],self.dispBas,sensitive=pd_valid
    self->my_widget_control,'plotTra', self.dispBas,sensitive=ptr_valid(self.projStrPtr)
    self->my_widget_control,'oplotTra',self.dispBas,sensitive=(ptr_valid(self.projStrPtr) and (self.samp_typ ne 0))
    if ~widget_info(self.dispBas,/valid_id) then return
    wid = widget_info(self.dispBas,find_by_uname='dispTab')
    if wid eq 0 then return
    if (self.instrname eq 'macs') and (self.samp_typ eq 0) then begin
       if  self.extravaxis_yn then begin
           if self.macsintn_typ ne 0 then self->my_widget_control,['volPlot','voloPlot'],self.dispBas,sensitive=0
       endif else begin
          if self.macsintn_typ ne 0 then dfcur = 1
          if pd_valid then begin
             minen = min((*self.projStrPtr).en,max=maxen)
             if minen eq maxen then dfcur = 1
          endif
          if n_elements(dfcur) ne 0 then self->my_widget_control,['dispFitSlice','dispPlotSlice','dispoPltSlice','dispSurfSlice'],self.dispBas,sensitive=0
       endelse
    endif 
    if (~ ptr_valid(self.dataStrPtr)) and (ptr_valid(self.projStrPtr)) then begin
       widget_control,wid,set_tab_current=([2,3])[index]
    endif else begin
       current = widget_info(wid,/tab_current)
       if (current eq ([2,3])[index]) or (current eq ([3,4])[index]) and pd_valid then dfcur=0
       if n_elements(dfcur) ne 0 then widget_control,wid,set_tab_current=dfcur
    endelse
end

pro dcs_mslice::set_filehistory,info,dir,files,choose=choose,ftpobj=ftpobj
    if n_elements(choose) eq 1 then begin
       ftype = strmid((*(self.file_hist[choose]))[0],6,strlen((*(self.file_hist[choose]))[0])-6)
       if ftype ne self.ftype then ok=self->dm_filetools((*(self.file_hist[choose]))[0])
       path = (*(self.file_hist[choose]))[2]
       tmp = stregex(path,'ftp://'+self.fileSel->getproperty(/ftpserver)+'(.+)',/subexpr,/extract)
       if strlen(tmp[1]) gt 0 then path=tmp[1]
       self.filesel->set_path,path,ftp=(strlen(tmp[1]) gt 0)
       self.filesel->getproperty,dFiles=dFiles
       for i=3,n_elements(*(self.file_hist[choose]))-1 do begin
           ind = where(dFiles eq (*(self.file_hist[choose]))[i],count)
           if count ne 0 then begin
              if n_elements(selected) eq 0 then selected=ind $
              else selected=[selected,ind]
           endif
       endfor
       if n_elements(selected) ne 0 then self.filesel->set_file,selected,/highlight
       return
    endif
    num = n_elements(self.file_hist)    ;capacity
    if (n_elements(info) ne 0) and (n_elements(dir) ne 0) and (n_elements(files) ne 0) then begin
       if obj_valid(ftpobj) then dir = 'ftp://'+self.fileSel->getproperty(/ftpserver)+dir
       for i=0,self.n_history-1 do begin  ;check if this entry already exists
           if dir ne (*(self.file_hist[i]))[2] then continue
           if n_elements(files) ne n_elements(*(self.file_hist[i]))-3 then continue
           same = 1b
           for j=0, n_elements(files)-1 do $
               if files[j] ne (*(self.file_hist[i]))[3+j] then same=0b
           if same then return ;entry exists
       endfor
       if self.n_history lt num then begin ;empty room available
          if self.n_history ne 0 then self.file_hist[1:self.n_history] = self.file_hist[0:self.n_history-1]
          self.file_hist[0] = ptr_new(['ftype_'+dm_to_string(self.ftype,/int),info,dir,files])
          self.n_history = self.n_history+1
       endif else begin                    ;full
          ptr_free,self.file_hist[num-1]
          self.file_hist[1:num-1] = self.file_hist[0:num-2]
          self.file_hist[0] = ptr_new(['ftype_'+dm_to_string(self.ftype,/int),info,dir,files])
       endelse
    endif
    histmenu = widget_info(self.tlb,find_by_uname='history')
    widget_control,histmenu,sensitive=(self.n_history gt 0)
    self->my_widget_control,'clearHist',sensitive=(self.n_history gt 0)
    for i=0,num-1 do self->my_widget_control,'hist_'+dm_to_string(i,/int),histmenu,/destroy
    for i=0,self.n_history-1 do begin
        void = widget_button(histmenu,value=dm_to_string(i,/int)+' '+(*(self.file_hist[i]))[1],uname='hist_'+dm_to_string(i))
    endfor
end

;set instrument geometry and adjust the proper menu items
pro dcs_mslice::set_instrgeom,geom=geom,sensitive=sensitive
    if n_elements(geom) ne 0 then begin
       self.instrgeom = keyword_set(geom)
       if ~self.instrgeom then begin        ;direct geometry
          self->my_widget_control,'edepEff',/sensitive
          self->my_widget_control,'viewSpurion',sensitive=ptr_valid(self.dataStrPtr) and ptr_valid(self.projStrPtr) and (self.samp_typ lt 2)
       endif else begin                     ;inverse geometry
          self->my_widget_control,['edepEff','viewSpurion'],sensitive=0
       endelse
       self->my_menu_toggle,['direct','inverse']+'Geom',self.instrgeom
    endif
    if n_elements(sensitive) ne 0 then self->my_widget_control,['direct','inverse']+'Geom',sensitive=sensitive
end

;set intensity type
pro dcs_mslice::set_intntype,type
    ntype = n_elements(self.intn_typ_str)
    self.intn_typ = 0>(type)<(ntype-1)
    self->my_menu_toggle,'intnTyp'+dm_to_string(indgen(ntype),/int),self.intn_typ
    self->set_display_button,/clearprojection ;need to recalculate projections
    ;for MACS, chisq needs temperature
    if (self.instrname eq 'macs') then begin
       self->reset_intnlab
       if ((self.intn_typ eq 1) or (self.intn_typ eq 3)) and (~self.extravaxis_yn) then  $
          ok = dialog_message('Please enable the temperature viewing axis to ensure that temperature information is not missing from the data.',dialog_parent=self.tlb,/center)
    endif 
end

;check the current viewing axes to see if they are both on the scattering plane, only the first two, or the two non-T viewing axis for MACS and WAND
function dcs_mslice::viewonscattplane
    if total(finite(self.view_u1,/nan)) ne 0 then return,0b
    if total(finite(self.view_u2,/nan)) ne 0 then return,0b
    if total(abs(self.view_u1)) eq 0         then return,0b
    if total(abs(self.view_u2)) eq 0         then return,0b
    if total(finite(self.orie_u,/nan))  ne 0 then return,1b    ;can't check, assume it is ok
    if total(finite(self.orie_v,/nan))  ne 0 then return,1b    ;can't check, assume it is ok
    tmp = crossp(self.orie_u,self.orie_v)
    if transpose(self.view_u1)#tmp ne 0      then return,0b
    if transpose(self.view_u2)#tmp ne 0      then return,0b 
    return,1b
end

;save a slice as iexy file
pro dcs_mslice::write_iexy,unit,xdat,ydat,zdat,zerr
    nx = n_elements(xdat) & ny = n_elements(ydat)
    for i=0L,nx-1L do begin
        for j=0L,ny-1L do begin
            printf,unit,zdat[i,j],zerr[i,j],xdat[i],ydat[j]
        endfor
    endfor
end
    
;save a slice as xyie matrix file
pro dcs_mslice::write_xyie,unit,xdat,ydat,zdat,zerr
    nx = n_elements(xdat) & ny = n_elements(ydat)
    printf,unit,nx,ny
    printf,unit,' '
    for i=0L,nx-1L do printf,unit,xdat[i]
    printf,unit,' '
    for i=0L,ny-1L do printf,unit,ydat[i]
    printf,unit,' '
    printf,unit,zdat,format='('+dm_to_string(nx)+'(g13.6," "))'
    printf,unit,' '
    printf,unit,zerr,format='('+dm_to_string(nx)+'(g13.6," "))'
end

;save a slice as psutxt file
pro dcs_mslice::write_psutxt,unit,xdat,ydat,zdat,zerr
    nx = n_elements(xdat)    ;# of Q
    ny = n_elements(ydat)    ;# of E
    printf,unit,'# Number of energy transfer values'
    printf,unit,dm_to_string(ny)
    printf,unit,'# Number of Q transfer values'
    printf,unit,dm_to_string(nx)
    printf,unit,'# energy transfer (meV) Values:'
    for i=0L,ny-1 do printf,unit,dm_to_string(ydat[i])
    printf,unit,'# Q transfer (1/Angstroms) Values:'
    for i=0L,nx-1 do printf,unit,dm_to_string(xdat[i])
    for i=0L,nx-1 do begin
        printf,unit,'# Group '+dm_to_string(i)
        for j=0L,ny-1 do printf,unit,dm_to_string(zdat[i,j])+'   '+dm_to_string(zerr[i,j]) 
    endfor
end

;cleanup, life cycle method
pro dcs_mslice::Cleanup
    ;save parameters in default parameter file if allowed
    if strlen(self.parmname) ne 0 then begin
       if file_test(self.dirs[2],/directory,/write) then self->dm_save_parameter,self.dirs[2]+self.pathsep+self.parmname,/quiet
    endif
    if widget_info(self.tlb,/valid_id) then widget_control,self.tlb,/destroy
    ptr_free,self.colors,self.slice_cmap,self.lines,self.symbols,self.pdim_list
    ptr_free,self.ftypename,self.ftypeext,self.bgratePtr,self.bgtchnPtr,self.bgdetsPtr,self.bgdata,self.bgdata2
    ptr_free,self.powd_qvalue,self.dcor_info,self.a3mask,self.mask,self.eff,self.eff1,self.kfactor,self.rebinsize,self.powd_line,self.absp_info,self.file_hist,self.absp_npts,self.absp_nptst
    ptr_free,self.detPosPtr,self.dataStrPtr,self.projStrPtr,self.dataPtr,self.info,self.slice_t,self.slice_tft,self.cutsel_t,self.cutsel_tft,self.volsel_t,self.volsel_tft
    ptr_free,self.view_u1foldcenter,self.view_u1folddir,self.view_u2foldcenter,self.view_u2folddir,self.view_u3foldcenter,self.view_u3folddir,self.whatsnew 
    obj_destroy,self.gr_list
    obj_destroy,self.fileSel
end

;initialization, life cycle method
;usage:
;   void=obj_new('dcs_mslice',group_leader=group_leader)
;parameters:
;   NONE
;keyword:
;   groupleader:    groupleader widget ID
;   dataPtr:        if present, starts it as a volume data viewer
;   instrname:      default starting instrument name
;   no_parmfile:    if set, don't load mslice.prm
function dcs_mslice::Init,dataPtr=dataPtr,group_leader=group_leader,workDir=workDir,dataDir=dataDir,DAVETool=DAVETool,instrname=instrname,no_parmfile=no_parmfile
    if (n_elements(dataPtr) ne 0) then begin
       ;check dataPtr
       if (~ ptr_valid(dataPtr)) then $
          errmesg = 'Invalid data pointer. ' $
       else begin
          field = strupcase(tag_names(*dataPtr))    
          loc   = where(field eq 'DATA',count)
          if count eq 0 then $
             errmesg = "The data pointer doesn't contain a data field. " $
          else begin
             nitem = n_elements((*dataPtr).data[0,*])
             if nitem lt 4 then errmesg = 'The data need at least two independent variables.'
             self.ftypecheckmask = (total(finite((*dataPtr).data,/nan)) gt 0)
          endelse
       endelse
       if n_elements(errmesg) ne 0 then begin
          ok = dialog_message(errmesg,/error,dialog_parent=group_leader,/center)
          return,0
       endif
       loc = where(field eq 'LABEL',count)
       if count eq 0 then begin
          loc = where(field eq 'LABELS',count)
          if count ne 0 then label = (*dataPtr).labels $
          else begin 
             label = ['Intensity','Error']
             for i=0,nitem-3 do label = [label,'x'+dm_to_string(i+1)]
          endelse
          *dataPtr = create_struct(*dataPtr,'label',label)
       endif
       loc = where(field eq 'INSTRUMENT',count)
       if count eq 0 then *dataPtr = create_struct(*dataPtr,'instrument','')
       loc = where(field eq 'UNIT',count)
       if count eq 0 then begin
          loc = where(field eq 'UNITS',count)
          if count ne 0 then unit = (*dataPtr).units $
          else unit = strarr(nitem)
          *dataPtr = create_struct(*dataPtr,'unit',unit)
       endif
       loc = where(field eq 'IS_UNIQ',count)
       if count eq 0 then begin
          is_uniq = bytarr(nitem)
          *dataPtr = create_struct(*dataPtr,'is_uniq',is_uniq)
       endif
       loc = where(field eq 'AUTHOR',count)
       if count ne 0 then author = ', '+(*dataPtr).author[0] else author = ''
    endif
    if n_elements(group_leader) ne 0 then self.group_leader=group_leader
    registerName = (['dcs_mslice','mslice'])[ptr_valid(dataPtr)] 
    if xregistered(registerName) and (~ ptr_valid(dataPtr)) then begin ;only allow one dcs_mslice running
       FORWARD_FUNCTION LookupManagedWidget
       id = LookupManagedWidget(registername)
       widget_control,id,iconify=0
       return,0
    endif
    
    ;error handler
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(!error_state.msg,/error,dialog_parent=group_leader,/center)
       self->Cleanup
       if obj_valid(tmpplotobj) then obj_destroy,tmpplotobj
       return, 0
    end

    tmpplotobj = obj_new('dm_plot')   ;to get symbol and linestyle and color list

    ;initialize the data
    tmp = dm_check_macskidney(0,5,offset=offset)
    self.kidney_offset      = offset
    self.instrgeom          = 0     ;default direct geometry
    self.view               = [1b,1b] 
    self.userfwarning       = 1b
    self.dcor_yn            = 0
    self.edep_yn            = 1
    self.bitflip            = 1
    self.tthcorr            = 0
    self.mask_filler        = -1e20
    self.mbparm             = [0.25,4.0,0.04,3.0]
    self.pbgcol_bw          = 1     ;defalut white
    self.pdim_list          = ptr_new([3,2]#[150,200,250,300])
    self.pdim_choice        = 1     ;default 600x400
    self.eadjust            = 0
    self.eadjust_spec       = !values.f_nan
    self.eadjust_checkfirst = 0
    self.eint_yn            = 0
    self.intn_typ           = 0     ;default S(q,w)
    self.macsintn_typ       = 0     ;'SPEC'
    self.macs_tolerance     = [0.01,0.1,0.02] ;default tolerance for macs
    self.macs_bgefirst      = 0     ;matching order: energy last
    self.macs_CFXdate       = dm_to_number('5/1/2023',/date) ;not known yet
    self.macs_lamb2         = (dm_to_number(systime(),/date) ge self.macs_CFXdate)
    self.macs_effmask       = [-10.,10.]
    self.psi_tolerance      = 0.02             ;default tolerance for others
    self.intn_typ_str       = ['S(Q,omega)','Symmetrized S(Q,omega)','d2(sigma)/d(Omega)d(Ef)','Chi(Q,omega)','GDOS','VDOS','Unadjusted by ki/kf factor']
    self.powd_typ           = 0     ;no powder line
    self.powd_showall       = 1     ;show closed powder line
    self.powd_latta         = !values.f_nan
    self.dcor_eqran         = [!values.f_nan,!values.f_nan,!values.f_nan,!values.f_nan]
    self.datr_psi           = !values.f_nan
    self.datr_yn            = 0b
    self.xsize              = [589L,47L,120L,9L,92L]
    self.ssfac              = 1.0
    self.view_ortharbi      = 1     ;arbitrary
    self.bin_avgsum         = 0     ;average
    self.binthreshold       = 0.0   ;treshold level
    self.bin_conststep      = 0     ;average value
    self.bin_extlib         = 1     ;use external library
    self.mon_sumfit         = 1     ;dcs-fit, macs-specify
    self.dcs_mon            = 1     ;use bm1 as the monitor
    self.macs_mon           = [!values.f_nan,1e6]
    self.temperature        = 300.0
    self.bgeran             = [-20,-10]
    self.intn_lab           = 'Intensity (arb. units)'
    self.samp_typ           = (ptr_valid(dataPtr)?(-1):0)     ;default powder
    self.rebinsize          = ptr_new([1s,2s,4s,5s,8s,10s,20s,25s,40s,50s])
    self.e_bin              = [1,1,1,1,1]
    self.e_range            = [!values.f_nan,!values.f_nan]
    self.latt_parm          = [10d,10d,10d,90d,90d,90d]
    self.orie_u             = [1e,0e,0e]
    self.orie_v             = [0e,1e,0e]
    self.checklattuv        = 1b    ;default checking
    self.orie_offset        = !values.f_nan
    self.view_type          = ['Energy','|Q|','2Theta','Det Group Number','Time Channel']
    self.view_label         = ['E','|Q|','2theta','Detector Group','Time']
    self.view_unit          = ['meV','\AA!U-1!N','\deg','','\mus']
    self.view_u1typ         = 1
    self.view_u2typ         = 0
    self.view_u1            = [1,0,0]
    self.view_u2            = [0,1,0]
    self.view_u3            = [0,0,1]
    self.view_u1lab         = ['|Q|','H']
    self.view_u2lab         = ['E','K']
    self.view_u3lab         = 'L'
    self.view_u1folddir     = ptr_new(0)
    self.view_u2folddir     = ptr_new(0)
    self.view_u3folddir     = ptr_new(0)
    self.view_u1foldcenter  = ptr_new(0)
    self.view_u2foldcenter  = ptr_new(0)
    self.view_u3foldcenter  = ptr_new(0)
    self.vol_smooth         = 1
    self.vol_interp         = 1     ;0:nearest neighbor 1:trilinear
    self.vol_color          = 13    ;rainbow
    self.volsel_x           = [0,0,0,0]
    self.volsel_y           = [1,1,1,1]
    self.volsel_z           = [2,2,2,2]
    self.vol_xran           = [!values.f_nan,!values.f_nan]
    self.vol_xstep          = !values.f_nan
    self.vol_yran           = [!values.f_nan,!values.f_nan]
    self.vol_ystep          = !values.f_nan
    self.vol_zran           = [!values.f_nan,!values.f_nan]
    self.vol_zstep          = !values.f_nan
    self.vol_iran           = [!values.f_nan,!values.f_nan]
    self.volsel_t1          = 3
    self.volsel_tft1        = [!values.f_nan,!values.f_nan]
    self.slice_x            = [0,0,0,0]
    self.slice_y            = [1,1,1,1]
    self.slice_t1           = [2,2,2,2]
    self.slice_t2           = [3,3,3,3]
    self.slice_xran         = [!values.f_nan,!values.f_nan]
    self.slice_yran         = [!values.f_nan,!values.f_nan]
    self.slice_t1ran        = [!values.f_nan,!values.f_nan]
    self.slice_t2ran        = [!values.f_nan,!values.f_nan]
    self.slice_xstep        = 0.035
    self.slice_ystep        = !values.f_nan
    self.slice_iran         = [!values.f_nan,!values.f_nan]
    self.slice_smooth       = 1
    self.slice_sm_kernel    = [[0.1e,0.2e,0.1e],[0.2e,0.8e,0.2e],[0.1e,0.2e,0.1e]]
    self.slice_shading      = 1
    self.slice_cmap         = ptr_new(tmpplotobj->getpalette(/list))
    self.slice_color        = 13    ;rainbow
    self.cut_x              = [0s,0s,0s,0s]
    self.cut_y              = [0s,0s,0s,0s]
    self.cut_t1             = [1s,1s,1s,1s]
    self.cut_t2             = [2s,2s,2s,2s]
    self.cut_t3             = [3s,3s,3s,3s]
    self.cut_ran            = [!values.f_nan,!values.f_nan]
    self.cut_step           = 0.035
    self.cut_t1ran          = [!values.f_nan,!values.f_nan]
    self.cut_t2ran          = [!values.f_nan,!values.f_nan]
    self.cut_t3ran          = [!values.f_nan,!values.f_nan]
    self.cut_yran           = [!values.f_nan,!values.f_nan]
    if self.samp_typ eq 0 then begin  ;MACS powder with temperature axis
       self.volsel_x[0]     = 1
       self.volsel_y[0]     = 0
       self.volsel_z[0]     = 4 
       self.slice_x[0]      = 1
       self.slice_y[0]      = 0
       self.slice_t1[0]     = 4
       self.cut_t2[0]       = 4
    endif
    self.cut_symbol         = 0s
    self.symbols            = ptr_new(tmpplotobj->getpsym(/list))
    self.cut_color          = 0
    self.colors             = ptr_new(tmpplotobj->getcolor(/list))
    (*self.colors)[0]       = (['white','black'])[self.pbgcol_bw]
    self.coldefine          = 255b-[255b,255b,255b]*self.pbgcol_bw
    self.cut_line           = 0
    self.lines              = ptr_new(tmpplotobj->getlinestyle(/list))
    self.traj_type          = ['[Qh,0,0]','[0,Qk,0]','[0,0,Ql]','E (meV)','None']
    self.traj_x             = [1,0,0,0]
    self.traj_xran          = [!values.f_nan,!values.f_nan]
    self.traj_y             = [0,1,1,1]
    self.traj_yran          = [!values.f_nan,!values.f_nan]
    self.traj_z1            = [5,4,3,4]
    self.traj_z1ran         = [!values.f_nan,!values.f_nan]
    self.traj_z2            = [0,4,3,4]
    self.traj_z2ran         = [!values.f_nan,!values.f_nan]
    self.gr_list            = obj_new('IDL_Container')
    self.errbar             = 1b
    self.useitool           = 0b
    self.negative2th        = 0b
    self.empcanfactor       = [1.0,1.0]
    self.eief               = !values.f_nan
    self.estartend          = [!values.f_nan,!values.f_nan,81.]
    self.psi[*]             = !values.f_nan
    self.absp_npts          = [ptr_new([[30,60],[60,120],[90,180]]),$
                               ptr_new([[30,60],[60,120],[90,180]]),$
                               ptr_new([[30,60,60],[60,120,120],[90,180,180]]),$
                               ptr_new([[60,30],[120,60],[180,90]])]
    self.absp_nptst         = [ptr_new(['nr=30, nphi=60','nr=60, nphi=120','nr=90, nphi=180']),$
                               ptr_new(['nr=30, nphi=60','nr=60, nphi=120','nr=90, nphi=180']),$
                               ptr_new(['nr=30, nphi=60, ntheta=60','nr=60, nphi=120, ntheta=120','nr=90, nphi=180, ntheta=180']),$
                               ptr_new(['nw=60, nt=30','nw=120, nt=60','nw=180, nt=90'])]
    mod_date                = '5/2013'
    self.idl_version        = dm_to_number(!version.release)
    self.info               = ptr_new(['--Mslice for DCS --', $
                                       'Based on the NCNR DCS data reduction programs by John Copley',$
                                       'and the matlab version of mslice by Radu Coldea. ','',$
                                       'Please kindly cite the following paper in your publication:',$
                                       '"DAVE: A comprehensive software suite for the reduction, visualization, and analysis of low energy neutron spectroscopic data",', $
                                       'R.T. Azuah, L.R. Kneller, Y. Qiu, P.L.W. Tregenna-Piggott, C.M. Brown, J.R.D. Copley, and R.M. Dimeo,',$
                                       'J. Res. Natl. Inst. Stan. Technol. 114, 341 (2009).','',$
                                       'Yiming Qiu(yiming.qiu@nist.gov)   '+mod_date])
    self.whatsnew           = ptr_new(['4/2013: Allow temperature as extra viewing axis for MACS and WAND. Revamp main GUI.',$
                                       '3/2013: MACS instrument fully supported.',$ 
                                       '3/2013: Add NCNR ftp data server to the file selection drive list in Windows platform, or to the root directory in unix platform.',$
                                       '2/2013: Add setting mask by angle option in "Option->Set Mask->By Detector Angle".',$
                                       '2/2013: Add symmetry rotation folding option in "Option->Folding Type".',$
                                       '12/2012: Add multiple slices output in saving volume data.',$
                                       '11/2012: Add DCS Q&E resolution convolution tool.',$
                                       '9/2012: Add common LaTex commands in the text input of dm_plot and add more font choices.',$
                                       '12/2011: Add cutting and slicing in plot window in IDL 8.0.'+((self.idl_version ge 8.0)?'':(' You are running IDL '+!version.release+'.')),$
                                       '','For more information, please check the Mslice Help manual.'])
    obj_destroy,tmpplotobj
    if self.idl_version ge 5.5 then self.pathsep=call_function("path_sep") $
    else begin
       defsysv,'!delimiter',exists=exists
       if (~exists) then begin
          case !version.os_family of
            'Windows':  defsysv,'!delimiter','\'
            'unix':     defsysv,'!delimiter','/'
            'MacOS':    defsysv,'!delimiter',':'
            else:       defsysv,'!delimiter','/'
          endcase
       endif
       self.pathsep = !delimiter
    endelse
    
    cd,current=current
    self.dirs = [current,current,self->getrootdir()]
    self->set_dir,dataDir=dataDir,workDir=workDir
    self.helpfile = ['dcs_mslice.pdf','dcs_mslice_addftype.pdf']
    defsysv,'!DAVE_PDFHELP_DIR',exists=exists
    if exists then self.helpfile = !DAVE_PDFHELP_DIR+self.pathsep+self.helpfile $
    else begin
       self.helpfile[0] = file_which(self.helpfile[0],/include)
       self.helpfile[1] = file_which(self.helpfile[1],/include)
    endelse
    defsysv,'!dcs_hsq2mn',exists=exists
    if (~exists) then defsysv,'!dcs_hsq2mn',81.8042d ;h^2/2./m_n/e*1e23
    defsysv,'!screensize',exists=exists   ;used in dcs_plotdetmask
    if (~exists) then defsysv,'!screensize',GET_SCREEN_SIZE(),1
    defsysv,'!dave_colors',exists=exists  ;used in dcs_plotdetmask
    if (~exists) then begin
       color = {color, $ 
            red:   [255,0,255,  0,  0,  0,255,255,152,128,128,128,255,178,255],$ ; red values
            green: [255,0,  0,255,  0,255,  0,255,120,128,128,128, 54,192,128],$ ; green values
            blue:  [255,0,  0,  0,255,255,255,  0,103,220, 60,128, 54,220,128] $ ; blue values
        }
       defsysv, '!dave_colors',color,1
    endif

    if n_elements(DAVETool) ne 0 then $
       if obj_valid(DAVETool) then self.DAVETool = DAVETool

    accel_yn = (self.idl_version ge 6.1)

    ;generate widgets
    if ptr_valid(dataPtr) then begin
       self.mask_filler = !values.f_nan
       self.samp_typ = -1
       self.dataPtr  = dataPtr 
       n_item = n_elements((*dataPtr).label)
       if n_item gt 5 then begin
          self.volsel_t   = ptr_new(indgen(n_item-5)+3)
          self.volsel_tft = ptr_new(fltarr(n_item-5,2))
          (*self.volsel_tft)[*] = !values.f_nan 
       endif
       if n_item gt 4 then begin
          self.slice_t   = ptr_new(indgen(n_item-4)+2)
          self.slice_tft = ptr_new(fltarr(n_item-4,2))
          (*self.slice_tft)[*] = !values.f_nan 
       endif
       if n_item gt 3 then begin
          self.cutsel_t   = ptr_new(indgen(n_item-3)+1)
          self.cutsel_tft = ptr_new(fltarr(n_item-3,2))
          (*self.cutsel_tft)[*] = !values.f_nan 
       endif
       *(self.info) = ['--Mslice'+(strlen((*dataPtr).instrument) eq 0?'':' for '+(*dataPtr).instrument)+' --','Yiming Qiu(yiming.qiu@nist.gov)'+author+'   '+mod_date]
       self.tlb = widget_base(title=(*self.info)[0],/column,kill_notify='dcs_mslice_Exit',mbar=bar,group_leader=self.group_leader,ypad=0,xpad=0,uname='tlb',/tlb_size_event,map=0)
       filemenu = widget_button(bar,value='File',/menu)
       parmmenu = widget_button(bar,value='Parameter',/menu)
       maskmenu = widget_button(bar,value='Mask',/menu)
       optnmenu = widget_button(bar,value='Option',uname='optnMenu',/menu)
       viewmenu = widget_button(bar,value='Window',/menu,uname='viewMenu') 
       helpmenu = widget_button(bar,value='Help',/menu) 
       binmmenu = widget_button(optnmenu,value='Binning Method',/menu)  
       pbgcmenu = widget_button(optnmenu,value='Plot Window Background Color',/menu)
       pdimmenu = widget_button(optnmenu,value='Plot Window Default Size',uname='pdimMenu',/menu)
       fvolmenu = widget_button(filemenu,value='Save Volume Data As',uname='savevolumeMenu',/menu,sensitive=(n_item ge 5))
       fslimenu = widget_button(filemenu,value='Save Slice Data As',uname='savesliceMenu',/menu)
       fcutmenu = widget_button(filemenu,value='Save Cut Data As',uname='savecutMenu',/menu)
       fpromenu = widget_button(filemenu,value='Save All Data As',uname='saveprojMenu',/menu)   
       void     = widget_button(filemenu,value='Exit',uname='exitBut',/separator)
       void     = widget_button(fvolmenu,value='Single Text File...',uname='savevolumetxt')
       vslimenu = widget_button(fvolmenu,value='Multiple Slice Files',uname='savevolsliceMenu',/menu)
       vxslmenu = widget_button(vslimenu,value='X Slices',uname='savevolxsliceMenu',/menu)
       vyslmenu = widget_button(vslimenu,value='Y Slices',uname='savevolysliceMenu',/menu)
       vzslmenu = widget_button(vslimenu,value='Z Slices',uname='savevolzsliceMenu',/menu)
       void     = widget_button(vxslmenu,value='SPE File...',uname='savevolxslicespe')
       void     = widget_button(vxslmenu,value='I,Err,X,Y 4-Column File...',uname='savevolxsliceiexy')
       void     = widget_button(vxslmenu,value='X,Y,I,Err Matrix File...',uname='savevolxslicexyie')
       void     = widget_button(vyslmenu,value='SPE File...',uname='savevolyslicespe')
       void     = widget_button(vyslmenu,value='I,Err,X,Y 4-Column File...',uname='savevolysliceiexy')
       void     = widget_button(vyslmenu,value='X,Y,I,Err Matrix File...',uname='savevolyslicexyie')
       void     = widget_button(vzslmenu,value='SPE File...',uname='savevolzslicespe')
       void     = widget_button(vzslmenu,value='I,Err,X,Y 4-Column File...',uname='savevolzsliceiexy')
       void     = widget_button(vzslmenu,value='X,Y,I,Err Matrix File...',uname='savevolzslicexyie')
       void     = widget_button(fslimenu,value='DAVE File...',uname='saveslicedave')
       void     = widget_button(fslimenu,value='SPE File...',uname='saveslicespe')
       void     = widget_button(fslimenu,value='I,Err,X,Y 4-Column File...',uname='savesliceiexy')
       void     = widget_button(fslimenu,value='X,Y,I,Err Matrix File...',uname='saveslicexyie')
       if obj_valid(self.DAVETool) then $
          void  = widget_button(fslimenu,value='Send to DAVE Data Manager',uname='sendslicedave',/separator)
       void     = widget_button(fcutmenu,value='X,Y,Err 3-Column File...',uname='savecutxye')
       void     = widget_button(fcutmenu,value='DAVE File...',uname='savecutdave')
       if obj_valid(self.DAVETool) then $
          void  = widget_button(fcutmenu,value='Send to DAVE Data Manager',uname='sendcutdave',/separator)
       void     = widget_button(fpromenu,value='Text File',uname='saveprojtxt')
       void     = widget_button(parmmenu,value='Load Parameters...',uname='ldpmBut',_extra=accel_yn?{accelerator:'Ctrl+L'}:{no_copy:1})
       void     = widget_button(parmmenu,value='Save Parameters...',uname='svpmBut',_extra=accel_yn?{accelerator:'Ctrl+S'}:{no_copy:1})
       void     = widget_button(parmmenu,value='Change Smoothing Kernel...',uname='knpmBut',/separator)
       void     = widget_button(parmmenu,value='Change Binning Threshold...',uname='bnthBut')
       void     = widget_button(parmmenu,value='Set Working Directory...',uname='wdirBut',/separator)
       void     = dm_widget_button(maskmenu,value='Mask Data by Value',uname='maskFiller')  
       binAvg   = dm_widget_button(binmmenu,value='Average',uname='binAvg')
       binSum   = dm_widget_button(binmmenu,value='Sum',uname='binSum')
       bgBlack  = dm_widget_button(pbgcmenu,value='Black',uname='pbgBlack',_extra=accel_yn?{accelerator:'Ctrl+B'}:{no_copy:1})
       bgWhite  = dm_widget_button(pbgcmenu,value='White',uname='pbgWhite',_extra=accel_yn?{accelerator:'Ctrl+W'}:{no_copy:1})
       for i=0,n_elements((*self.pdim_list)[0,*])-1 do void = dm_widget_button(pdimmenu,value=dm_to_string((*self.pdim_list)[*,i],separator='x'),uname='pdchoice_'+dm_to_string(i,/int))
       bincsmenu= widget_button(binmmenu,value='Constant Intervals',/menu,/separator)
       bincsYes = dm_widget_button(bincsmenu,value='Yes',uname='bincsYes')
       bincsNo  = dm_widget_button(bincsmenu,value='No',uname='bincsNo')
       binexmenu= widget_button(binmmenu,value='Use External Binning Library',/menu)
       binexYes = dm_widget_button(binexmenu,value='Yes',uname='binexYes')
       binexNo  = dm_widget_button(binexmenu,value='No',uname='binexNo')
       if self.idl_version ge 6.1 then $
          void  = widget_button(viewmenu,value='Current Plot Window',uname='viewCurr',sensitive=0,accelerator='Ctrl+Tab')
       void     = widget_button(viewmenu,value='Close All Plot Windows',uname='killAll',sensitive=0)
       arrpmenu = widget_button(viewmenu,value='Arrange Plots',uname='arrPlot',sensitive=0,/menu)
       void     = widget_button(arrpmenu,value='Cascade',uname='arrCascade')
       void     = widget_button(arrpmenu,value='Table',uname='arrTable')
       void     = widget_button(helpmenu,value='About Mslice',uname='selfBut')
       
       ;menu bar separator for windows system
       if !version.os_family eq 'Windows' then $
          mbar_sep = widget_label(self.tlb,value=' ',sensitive=0,/dynamic_resize,/align_top,scr_ysize=5) $
       else $
          self.xsize[2] = self.xsize[2]+20
          
       ;load mslice.prm if existed
       if ~keyword_set(no_parmfile) then begin
          parmfile = self.dirs[2]+self.pathsep+'mslice.prm'
          if file_test(parmfile,/read) then self->dm_load_parameter,parmfile,/init
       endif   
       self.parmname = 'mslice.prm'
       
       dm_toggle_menubut,check=([binAvg,binSum])[self.bin_avgsum],uncheck=([binAvg,binSum])[1-self.bin_avgsum]
       dm_toggle_menubut,check=([bincsNo,bincsYes])[self.bin_conststep],uncheck=([bincsNo,bincsYes])[1-self.bin_conststep]
       dm_toggle_menubut,check=([binexNo,binexYes])[self.bin_extlib],uncheck=([binexNo,binexYes])[1-self.bin_extlib]
       dm_toggle_menubut,check=([bgBlack,bgWhite])[self.pbgcol_bw],uncheck=([bgBlack,bgWhite])[1-self.pbgcol_bw]
       self->my_menu_toggle,'pdchoice_'+dm_to_string(indgen(n_elements((*self.pdim_list)[0,*])),/int),self.pdim_choice,base=pdimmenu
       
       self->reset_display
       widget_control,self.dispBas,/update,/map
    endif else begin
       self.tlb = widget_base(title=(*self.info)[0],/column,kill_notify='dcs_mslice_Exit',mbar=bar,group_leader=self.group_leader,ypad=0,xpad=0,uname='tlb',/tlb_size_event,map=0)
       filemenu = widget_button(bar,value='File',/menu)
       parmmenu = widget_button(bar,value='Parameter',/menu,uname='parmMenu')
       bkgrmenu = widget_button(bar,value='Background',/menu,uname='bkgrMenu')
       maskmenu = widget_button(bar,value='Mask',/menu,uname='maskMenu')
       optnmenu = widget_button(bar,value='Option',uname='optnMenu',/menu)
       viewmenu = widget_button(bar,value='Window',/menu,uname='viewMenu')
       helpmenu = widget_button(bar,value='Help',/menu)
       igeomenu = widget_button(optnmenu,value='Instrument Geometry',uname='igeomMenu',/menu)
       itypmenu = widget_button(optnmenu,value='View Intensity As',/menu,uname='itypMenu')
       binmmenu = widget_button(optnmenu,value='Binning Method',/menu)
       moncmenu = widget_button(optnmenu,value='Monitor Count',uname='moncMenu',/menu)
       abspmenu = widget_button(optnmenu,value='Absorption Correction',/menu)
       vaxsmenu = widget_button(optnmenu,value='Viewing Axis',/menu,uname='vaxisMenu',sensitive=0)
       vfldmenu = widget_button(optnmenu,value='Folding Type',/menu,uname='vfoldMenu',sensitive=0)
       powdmenu = widget_button(optnmenu,value='Show Powder Line',/menu,uname='showPowder')
       epadmenu = widget_button(optnmenu,value='Elastic Peak Position Adjustment',/menu,uname='eAdjust')
       eptlmenu = widget_button(optnmenu,value='Empty Can Subtraction Tolerance Method',/menu,uname='empttolMenu')
       pbgcmenu = widget_button(optnmenu,value='Plot Window Background Color',/menu)
       pdimmenu = widget_button(optnmenu,value='Plot Window Default Size',uname='pdimMenu',/menu)
       void     = widget_button(filemenu,value='File Type',uname='ftypeMenu',/menu)
       void     = widget_button(filemenu,value='File Tools',uname='ftoolMenu',/menu,sensitive=0)
       fvolmenu = widget_button(filemenu,value='Save Volume Data As',uname='savevolumeMenu',/menu,/separator,sensitive=0)
       fslimenu = widget_button(filemenu,value='Save Slice Data As',uname='savesliceMenu',/menu,sensitive=0)
       fcutmenu = widget_button(filemenu,value='Save Cut Data As',uname='savecutMenu',/menu,sensitive=0)
       fpromenu = widget_button(filemenu,value='Save Projection As',uname='saveprojMenu',/menu,sensitive=0)
       if ~lmgr(/vm) then $
          void  = widget_button(filemenu,value='Execute Script File...',uname='runScript',/separator)
       void     = widget_button(filemenu,value='Recent Files',uname='history',sensitive=0,/menu,/separator)
       void     = widget_button(filemenu,value='Clear File History',uname='clearHist',sensitive=0)
       void     = widget_button(filemenu,value='Exit',uname='exitBut',/separator)
       void     = widget_button(fvolmenu,value='Single Text File...',uname='savevolumetxt')
       vslimenu = widget_button(fvolmenu,value='Multiple Slice Files',uname='savevolsliceMenu',/menu)
       vxslmenu = widget_button(vslimenu,value='X Slices',uname='savevolxsliceMenu',/menu)
       vyslmenu = widget_button(vslimenu,value='Y Slices',uname='savevolysliceMenu',/menu)
       vzslmenu = widget_button(vslimenu,value='Z Slices',uname='savevolzsliceMenu',/menu)
       void     = widget_button(vxslmenu,value='SPE File...',uname='savevolxslicespe')
       void     = widget_button(vxslmenu,value='I,Err,X,Y 4-Column File...',uname='savevolxsliceiexy')
       void     = widget_button(vxslmenu,value='X,Y,I,Err Matrix File...',uname='savevolxslicexyie')
       void     = widget_button(vyslmenu,value='SPE File...',uname='savevolyslicespe')
       void     = widget_button(vyslmenu,value='I,Err,X,Y 4-Column File...',uname='savevolysliceiexy')
       void     = widget_button(vyslmenu,value='X,Y,I,Err Matrix File...',uname='savevolyslicexyie')
       void     = widget_button(vzslmenu,value='SPE File...',uname='savevolzslicespe')
       void     = widget_button(vzslmenu,value='I,Err,X,Y 4-Column File...',uname='savevolzsliceiexy')
       void     = widget_button(vzslmenu,value='X,Y,I,Err Matrix File...',uname='savevolzslicexyie')
       void     = widget_button(fslimenu,value='DAVE File...',uname='saveslicedave')
       void     = widget_button(fslimenu,value='SPE File...',uname='saveslicespe')
       void     = widget_button(fslimenu,value='I,Err,X,Y 4-Column File...',uname='savesliceiexy')
       void     = widget_button(fslimenu,value='X,Y,I,Err Matrix File...',uname='saveslicexyie')
       if obj_valid(self.DAVETool) then $
          void  = widget_button(fslimenu,value='Send to DAVE Data Manager',uname='sendslicedave',/separator)
       void     = widget_button(fcutmenu,value='X,Y,Err 3-Column File...',uname='savecutxye')
       void     = widget_button(fcutmenu,value='DAVE File...',uname='savecutdave')
       if obj_valid(self.DAVETool) then $
          void  = widget_button(fcutmenu,value='Send to DAVE Data Manager',uname='sendcutdave',/separator)
       void     = widget_button(fpromenu,value='Text File...',uname='saveprojtxt')
       void     = widget_button(parmmenu,value='Load Parameters...',uname='ldpmBut',_extra=accel_yn?{accelerator:'Ctrl+L'}:{no_copy:1})
       void     = widget_button(parmmenu,value='Save Parameters...',uname='svpmBut',_extra=accel_yn?{accelerator:'Ctrl+S'}:{no_copy:1})
       void     = widget_button(parmmenu,value='Change Self-shielding Factor...',uname='ssfcBut',/separator)
       void     = widget_button(parmmenu,value='Change Smoothing Kernel...',uname='knpmBut')
       void     = widget_button(parmmenu,value='Change Binning Threshold...',uname='bnthBut')
       void     = widget_button(parmmenu,value='Change Empty Can Subtraction Tolerance Value...',uname='bgtolBut')
       if self.extraempcan_yn then begin
          void  = widget_button(parmmenu,value='Change 2-Set Empty Can Factors...',uname='emfcBut',sensitive=0)
       endif
       if self.instrname eq 'wand' then begin
          void  = widget_button(parmmenu,value='Change WAND Detector Offset Angle...',uname='detoffsetBut',/separator)
          void  = widget_button(parmmenu,value='Change WAND Incident Wavelength...',uname='wandeiefBut')
       endif
       if self.instrname eq 'macs' then begin
          void  = widget_button(parmmenu,value='Change MACS Detector Offset Angle...',uname='detoffsetBut',/separator)
          void  = widget_button(parmmenu,value='Change MACS Detector Efficiency...',uname='macsdeteffBut')
       endif
       void     = widget_button(parmmenu,value='Set Data Directory...',uname='ddirBut',/separator)
       void     = widget_button(parmmenu,value='Set Working Directory...',uname='wdirBut')
       if self.extraempcan_yn then begin
          void  = widget_button(bkgrmenu,value='Load Empty Can File(s) 1',uname='emptBut')
          void  = widget_button(bkgrmenu,value='Load Empty Can File(s) 2',uname='emptBut2',sensitive=0)
       endif else $
          void  = widget_button(bkgrmenu,value='Load Empty Can File(s)',uname='emptBut')
       void     = widget_button(bkgrmenu,value='Load Detector Background File(s)...',uname='bgflBut')
       void     = widget_button(bkgrmenu,value='Load Time Channel Background File(s)...',uname='bgtcBut')
       void     = widget_button(bkgrmenu,value='Load Dark Count File(s)',uname='bgdkBut')
       void     = widget_button(bkgrmenu,value='Set Overall Background Rate...',uname='bgovBut')
       void     = widget_button(bkgrmenu,value='Clear Background',uname='bgclBut',sensitive=0)
       void     = widget_button(bkgrmenu,value='Load Vanadium File(s)...',uname='vandBut',/separator)
       void     = widget_button(bkgrmenu,value='Clear Vanadium',uname='vdclBut',sensitive=0)
       mkdbmenu = widget_button(maskmenu,value='DCS Mask',uname='mkdbMenu',/menu)
       void     = dm_widget_button(mkdbmenu,value='Mask Negative Angle Detectors',uname='mkngBut')
       void     = dm_widget_button(mkdbmenu,value='Mask Upper and Lower Banks',uname='mkulBut')
       void     = dm_widget_button(mkdbmenu,value='Mask Lower Bank',uname='mkloBut',/separator)
       void     = dm_widget_button(mkdbmenu,value='Mask Central Bank',uname='mkcnBut')
       void     = dm_widget_button(mkdbmenu,value='Mask Upper Bank',uname='mkupBut')
       stmkmenu = widget_button(maskmenu,value='Set Mask',/menu,uname='stmkMenu')
       void     = widget_button(stmkmenu,value='By Detector Number...',uname='mkstdnum')
       void     = widget_button(stmkmenu,value='By Detector Angle...',uname='mkstangl')
       plmkmenu = widget_button(maskmenu,value='Plot Mask',uname='mkplBut',sensitive=0,/menu)
       void     = widget_button(plmkmenu,value='By Detector Number...',uname='mkpldnum')
       void     = widget_button(plmkmenu,value='By Detector Angle...',uname='mkplangl')
       void     = widget_button(plmkmenu,value='Mask Bragg Peaks...',uname='mkbpBut',sensitive=0)
       void     = widget_button(maskmenu,value='Read Mask File...',uname='mkrdBut',/separator)
       void     = widget_button(maskmenu,value='Save Mask File...',uname='mksvBut',sensitive=0)
       void     = widget_button(maskmenu,value='Clear Mask',uname='mkclBut',sensitive=0,/separator)
       void     = dm_widget_button(igeomenu,value='Direct',uname='directGeom')
       void     = dm_widget_button(igeomenu,value='Inverse',uname='inverseGeom')
       for i=0,n_elements(self.intn_typ_str)-1 do $
           void = dm_widget_button(itypmenu,value=self.intn_typ_str[i],uname='intnTyp'+dm_to_string(i,/int))
       void     = dm_widget_button(itypmenu,value='E-Integrated Intensity',uname='intnEint',set_button=self.eint_yn,onstring='E-Integrated Intensity Enabled',offstring='E-Integrated Intensity Disabled',/separator)
       void     = dm_widget_button(binmmenu,value='Average',uname='binAvg')
       void     = dm_widget_button(binmmenu,value='Sum',uname='binSum')
       bincsmenu= widget_button(binmmenu,value='Constant Intervals',/menu,/separator)
       void     = dm_widget_button(bincsmenu,value='Yes',uname='bincsYes')
       void     = dm_widget_button(bincsmenu,value='No',uname='bincsNo')
       binexmenu= widget_button(binmmenu,value='Use External Binning Library',/menu)
       void     = dm_widget_button(binexmenu,value='Yes',uname='binexYes')
       void     = dm_widget_button(binexmenu,value='No',uname='binexNo')
       void     = dm_widget_button(moncmenu,value='Sum',uname='monSum')
       void     = dm_widget_button(moncmenu,value='Integrated Intensity',uname='monFit')
       mondmenu = widget_button(moncmenu,value='Monitor Choice',/menu,/separator,uname='mondMenu')
       void     = dm_widget_button(mondmenu,value='FC0',uname='monFC0')
       void     = dm_widget_button(mondmenu,value='BM1',uname='monBM1')
       void     = dm_widget_button(mondmenu,value='Duration',uname='monTIME')
       if lmgr(/runtime) then usertmp = 'User Macro...' else usertmp = 'User Macro/Function/Procedure...'
       if ~lmgr(/vm) then $
          void  = dm_widget_button(optnmenu,value=usertmp,uname='userFunction',set_button=strlen(self.userf_str),onstring='Enable '+usertmp,offstring='Disable '+usertmp,/separator)
       void     = dm_widget_button(optnmenu,value='Dark Angle Correction...',uname='darkCorr',set_button=self.dcor_yn,onstring='Dark Angle Correction Enabled',$
                  offstring='Dark Angle Correction Disabled',sensitive=0,separator=lmgr(/vm))
       void     = dm_widget_button(optnmenu,value='Show Dark Angle Trajectory',uname='darkTraj',set_button=self.datr_yn,onstring='Show Dark Angle Trajectory',$
                  offstring='Disable Dark Angle Trajectory')           
       void     = dm_widget_button(optnmenu,value='E-Dependent Detector Efficiency Correction',uname='edepEff',set_button=self.edep_yn,onstring='E-Dependent Detector Efficiency Correction Enabled',$
                  offstring='E-Dependent Detector Efficiency Correction Disabled')
       void     = dm_widget_button(optnmenu,value='Hide Error Bar',uname='errBar',set_button=1-self.errbar,onstring='Error Bar Disabled',offstring='Error Bar Enabled')
       void     = widget_button(optnmenu,value='View Spurion Caused by Detector',uname='viewSpurion',sensitive=0,/separator)
       void     = dm_widget_button(viewmenu,value='Projection',uname='viewProj',set_button=self.view[0],onstring='Hide Projection',offstring='Show Projection',_extra=accel_yn?{accelerator:'Ctrl+P'}:{no_copy:1})
       void     = dm_widget_button(viewmenu,value='Display',uname='viewDisp',set_button=self.view[1],onstring='Hide Display',offstring='Show Display',_extra=accel_yn?{accelerator:'Ctrl+D'}:{no_copy:1})
       if self.idl_version ge 6.1 then $
          void  = widget_button(viewmenu,value='Current Plot Window',uname='viewCurr',sensitive=0,accelerator='Ctrl+Tab',/separator)
       void     = widget_button(viewmenu,value='Close All Plot Windows',uname='killAll',sensitive=0,separator=(self.idl_version lt 6.1))
       arrpmenu = widget_button(viewmenu,value='Arrange Plots',uname='arrPlot',sensitive=0,/menu)
       void     = dm_widget_button(powdmenu,value='None',uname='powderNone')
       void     = dm_widget_button(powdmenu,value='Al',uname='powderAl')
       void     = dm_widget_button(powdmenu,value='Cu',uname='powderCu')
       void     = dm_widget_button(powdmenu,value='Stainless Steel',uname='powderSt')
       void     = dm_widget_button(powdmenu,value='Specify...',uname='powderSp')
       void     = dm_widget_button(powdmenu,value='Show Closed Powder Line',uname='powdershowall',set_button=self.powd_showall,onstring='Show Closed Powder Line',offstring='Show Actual Powder Line',/separator)
       void     = dm_widget_button(vaxsmenu,value='Orthogonal',uname='vaxisOrtho')
       void     = dm_widget_button(vaxsmenu,value='Arbitrary',uname='vaxisArbi')
       void     = dm_widget_button(vaxsmenu,value='Allow Extra Viewing Axis',uname='extraVaxis',/separator,set_button=self.extravaxis_yn,onstring='Allow Extra Viewing Axis',offstring='Disallow Extra Viewing Axis')
       void     = dm_widget_button(vfldmenu,value='Mirror Reflection',uname='vfoldMirror')
       void     = dm_widget_button(vfldmenu,value='Symmetry Rotation',uname='vfoldRotation')
       void     = dm_widget_button(abspmenu,value='None',uname='abspNone')
       void     = dm_widget_button(abspmenu,value='Cylinder...',uname='abspCylinder')
       void     = dm_widget_button(abspmenu,value='Annulus...',uname='abspAnnulus')
       void     = dm_widget_button(abspmenu,value='Sphere...',uname='abspSphere')
       void     = dm_widget_button(abspmenu,value='Flat Plate...',uname='abspFlat')
       void     = dm_widget_button(epadmenu,value='None',uname='epadNone')
       void     = dm_widget_button(epadmenu,value='Automatic',uname='epadAuto')
       void     = dm_widget_button(epadmenu,value='Specify...',uname='epadSpec')
       void     = dm_widget_button(epadmenu,value='Check the First File Only',uname='epadCheckfirst',set_button=self.eadjust_checkfirst,$
                  onstring='Check the First File Only',offstring='Check All Files',/separator)
       void     = dm_widget_button(eptlmenu,value='Closest or Within Tolerance',uname='empttol_cwt')
       void     = dm_widget_button(eptlmenu,value='Within Tolerance of the Closest',uname='empttol_wtc')
       void     = dm_widget_button(pbgcmenu,value='Black',uname='pbgBlack',_extra=accel_yn?{accelerator:'Ctrl+B'}:{no_copy:1})
       void     = dm_widget_button(pbgcmenu,value='White',uname='pbgWhite',_extra=accel_yn?{accelerator:'Ctrl+W'}:{no_copy:1})
       for i=0,n_elements((*self.pdim_list)[0,*])-1 do void = dm_widget_button(pdimmenu,value=dm_to_string((*self.pdim_list)[*,i],separator='x'),uname='pdchoice_'+dm_to_string(i,/int))
       void     = widget_button(arrpmenu,value='Cascade',uname='arrCascade')
       void     = widget_button(arrpmenu,value='Table',uname='arrTable')
       if strlen(self.helpfile[0]) ne 0 then $
          void  = widget_button(helpmenu,value='Mslice Help',uname='hpctBut',_extra=accel_yn?{accelerator:'F1'}:{no_copy:1})
       if strlen(self.helpfile[1]) ne 0 then $
       void     = widget_button(helpmenu,value='How to Add a New File Type',uname='hpftBut')
       void     = widget_button(helpmenu,value="What's New in Mslice",uname='hpnewBut',separator=(total(strlen(self.helpfile)) ne 0))
       void     = widget_button(helpmenu,value='About Mslice for DCS',uname='selfBut',/separator)

       ;menu bar separator for windows system
       if !version.os_family eq 'Windows' then $
          mbar_sep = widget_label(self.tlb,value=' ',sensitive=0,/dynamic_resize,/align_top,scr_ysize=5) $
       else $
          self.xsize[2] = self.xsize[2]+20
       
       self->dm_filemenu,/init ;also set up menu items
       self->reset_file
       
       ;load dcs_mslice.prm if existed
       parmfile = self.dirs[2]+self.pathsep+'dcs_mslice.prm'   
       if file_test(parmfile,/read) then begin
          self->dm_load_parameter,parmfile,/init
       endif else if n_elements(instrname) eq 0 then begin ;only ask if the default parameter file is unavailable and the instrument is not specified
          ftype = dm_dialog_input('file type:',default=self.ftype,droplist_content=ptr_new(*self.ftypename),is_droplist=[1],/return_number,title='Please choose')
       endif
       if n_elements(instrname) ne 0 then begin
          ftype = where(strlowcase(*self.ftypename) eq strlowcase(strtrim(instrname[0],2)),count)
          if count eq 0 then ftype = self.ftype 
       endif
       if n_elements(ftype) ne 0 then begin
          if self.ftype ne ftype[0] then ok = self->dm_filetools('ftype_'+dm_to_string(ftype))
       endif
       self.parmname = 'dcs_mslice.prm'
       
       self->reset_projection
       self->reset_display
       widget_control,self.projBas,/update,/map
       widget_control,self.dispBas,/update,/map
    endelse
    
    dm_center_kid,self.tlb
    widget_control,self.tlb,/realize,/update
    geom  = widget_info(self.tlb,/geometry)
    space = total(1-self.view)*geom.space*(!version.os_family eq 'Windows')
    if self.idl_version ge 5.5 then $
       self.geom = [geom.xsize,geom.ysize]+[0,space] $
    else begin
       self.geom = [geom.scr_xsize,geom.scr_ysize]+[0,space]
       widget_control,self.tlb,scr_xsize=self.geom[0],scr_ysize=self.geom[1]-space
    endelse
    
    widget_control,self.tlb,set_uvalue=self,/map
    
    xmanager,registerName,self.tlb,event_handler='dcs_mslice_event',cleanup='dcs_mslice_Exit',/no_block
    return,1
end

;definition of dcs_mslice class
pro dcs_mslice__define
    void={dcs_mslice,                  $
         group_leader:     0L,         $ ;group leader
         tlb:              0L,         $ ;container base of most of the widgets
         xsize:            lonarr(5),  $ ;to record the xsize
         geom:             [0L,0L],    $ ;xsize,ysize
         view:             [1b,1b],    $ ;flag for viewing the Projection and Display section
         ssfac:            1.0e,       $ ;self shielding factor
         temperature:      300e,       $ ;sample temperature for detailed balance calculation
         instrgeom:        0s,         $ ;instrument geometry  0:direct    1:inverse
         ftype:            0s,         $ ;file types: 0-DCS 1-SPE ...
         ftypename:        ptr_new(),  $ ;store file type names: 0-"DCS" 1-"SPE" ...
         ftypeext:         ptr_new(),  $ ;store file type extensions
         ftypecheckmask:   0b,         $ ;flag for checking mask in data files 0-no, 1-yes
         instrname:        '',         $ ;store the lower case of the current instrment name
         bgratePtr:        ptr_new(),  $ ;a pointer to the overall background rate or detector dark count array [[bgrate],[dbgrate]]
         bgeran:           [0e,0e],    $ ;background energy range
         bgtchnPtr:        ptr_new(),  $ ;pointer which points to {tchan:[start,end] qty:[end-start+1]} background information
         bgdetsPtr:        ptr_new(),  $ ;pointer which points to an [ndet] array of background
         bgdata:           ptr_new(),  $ ;pointer which points to a data structure which contains the background information
                                       $ ;-single crystal,powder
                                       $ ; {qty:[ntchan,ndet], err:[ntchan,ndet]}
                                       $ ;-diffuse scattering
                                       $ ; {qty:[ndet],err:[ndet]} or {qty:[nen,ndet],err[nen,ndet]}
                                       $ ;-macs
                                       $ ; {qty:[2,ndet,ndat],err:[2,ndet,ndat],rotation:[ndat],kidney:[ndat],weight:[ndat],cfx:[ndat],Ei:[ndat],Ef:[ndat]}
         bgdata2:          ptr_new(),  $ ;same as bgdata, used when extraempcan_yn = 1
         intn_typ:         0s,         $ ;intensity type
                                       $ ;0-S(Q,omega) 1-Symmetrized S(Q,omega) 2-d2sigma/dOmegadEf 3-Chi(Q,omega) 4-GDOS 5-VDOS 6-raw data
         macsintn_typ:     0s,         $ ;0-spec 1-diff 2-spec+diff
         macstemp_typ:     0s,         $ ;0-sample 1-control 2-set point
         macstempavg_yn:   0b,         $ ;0-no 1-yes if set, the tempertures in each file will be averaged
         macs_tolerance:   [0e,0e,0e], $ ;macs tolerance values [energy, kidney angle, a3 angle] in empty can subtraction
         macs_bgefirst:    0b,         $ ;macs empty can matching order 0: energy last 1: energy first
         macs_lamb2:       0b,         $ ;flag for MACS monitor lambda/2 correction, if this flag is on, and the CFX filter is none, lamba/2 correction will be done during projection calculation
         macs_CFXdate:     0ll,        $ ;date string when CFX status is included in the file
         macs_effmask:     [0e,0e],    $ ;a4 angle range to be ignored in the detector efficiency calculation
         psi_tolerance:    0e,         $ ;tolerance value for other instruments in empty can subtraction
         empt_tol_optn:    0b,         $ ;empty can subtraction tolerance option 0:closest or within tolerance 1:within the tolerace of the closest
         intn_typ_str:     strarr(7),  $ ;save the intensity type names
         lamcorrnxspe:     0s,         $ ;flag for ki/kf correction in NXSPE files 0: no (const t bins) 1: yes (const t bins)
                                       $ ; 2 no (const E bins) 3 yes (const E bins)
         mon_sumfit:       0s,         $ ;flag for monitor count calculation method 0:sum 1:gauss fit area
         dcs_mon:          1s,         $ ;dcs monitor choice: 0-fc0 1-bm1 2-duration; if 2, overall background rate is disabled
         macs_mon:         [0e,0e],    $ ;macs monitor counts [automatic value, specify value]
         bin_avgsum:       0b,         $ ;flag for binning method 0:average 1:sum
         binthreshold:     0e,         $ ;treshold level for binning
         bin_conststep:    0b,         $ ;flag for constant binning intervals 0:no  1:yes
         bin_extlib:       0b,         $ ;flag for using external binning libraries 0:no 1:yes
         view_ortharbi:    0s,         $ ;flag for ortho or arbitrary viewing axis 0:orthogonal 1:arbitrary
         fold_type:        0s,         $ ;flag for folding type 0:mirror 1:symmetry rotation
         powd_typ:         0s,         $ ;0-none 1-Al 2-Cu 3-Stainless Steel 4-specify
         powd_qvalue:      ptr_new(),  $ ;save the q values for the powder line
         powd_latta:       0e,         $ ;save input lattice a value for powder line
         powd_bcc:         0b,         $ ;save stainless steel structure choice of BCC
         powd_showall:     0b,         $ ;flag for showing the powder line in full circle
         eint_yn:          0b,         $ ;flag for E-Integrated intensity
                                       $ ;if set, then in Q-cut, I=sum(I[i,j].dE[i,j])/sum(dE[i,j])*E_width
         negative2th:      0b,         $ ;flag for perserving negative 2th in powder mode
         extraempcan_yn:   0b,         $ ;flag for allowing 2 sets of empty can files
         empcanfactor:[0e,0e],         $ ;factors to be applied to the empty can files, used only when two sets are available
         extravaxis_yn:    0b,         $ ;flag for allowing extra viewing axis in single crystal modes
         pbgcol_bw:        0b,         $ ;0:black 1:white for the plotting window background color
         pdim_list:        ptr_new(),  $ ;list of plot window size options
         pdim_choice:      0s,         $ ;default plot window size choice
         pnotooltip:       0b,         $ ;notooltip flag for dm_plot 0: first time starting mslice 1: after the parameter file is loaded 
         eadjust:          0s,         $ ;flag for elastic peak adjustment 0:no 1:auto, 2:specify
         eadjust_spec:     0e,         $ ;specified elastic peak position
         eadjust_checkfirst:0b,        $ ;flag for using the first file shift for all, default is no(check all files)
         userf_str:        '',         $ ;save user function strings:[macro string, user function string, user procedure string] 
         userf_before:     0b,         $ ;flag for executing the user function before the projection 0-after projection 1-before projection 2-before empty can subtraction
         userfwarning:     0b,         $ ;warning for userfunction
         errbar:           1b,         $ ;flag for showing error bar
         useitool:         0b,         $ ;flag for using itools
         datr_yn:          0b,         $ ;flag for showing dark angle trajactory
         datr_psi:         0e,         $ ;position of the dark angle with respect to u
         dcor_yn:          0b,         $ ;flag for dark angle correction in diffuse scattering modes
         dcor_eqran:       fltarr(4),  $ ;E and 2theta range for the dark angle correction or tchan background
         dcor_info:        ptr_new(),  $ ;a pointer points to {factor:[n_psi], psi:[n_psi]}
         edep_yn:          0b,         $ ;flag for calculating energy dep. det. efficiency 0:no 1:yes
         edep_det:         [0e,0e],    $ ;[thickness, he3 pressure] of the detector for e-dep det efficiency calculation
         bitflip:          0b,         $ ;flag for checking bitflip errors 0:no 1:yes for DCS type only
         tthcorr:          0b,         $ ;flag for correcting dcs 2theta   0:no 1:yes for DCS type only
         mask_filler:      0e,         $ ;a filler to indicate empty data in grid data such as slices
         mbparm:           fltarr(4),  $ ;automatic bragg peak masking parameter [width lower limit, width upper limit, intensity lower limit, intensity lower limit(sigma)], default [0.25,4.0,0.04,3.0]
         dcsdetmask:       bytarr(5),  $ ;flag for dcs detector bank mask [negative, lower bank, central bank, upper bank,upper and lower]
         a3mask:           ptr_new(),  $ ;macs only, masking a3, an angulra range string, similar to mask when masktype = 2
         mask:             ptr_new(),  $ ;detector mask pointer
         masktype:         0b,         $ ;0-by det number 1-by angle 2-by angle range  0,1-self.mask save index, 2-self.mask save angular range
         eff:              ptr_new(),  $ ;detector efficiency pointer
         eff1:             ptr_new(),  $ ;save the default macs detector efficiency originally saved in data file, only used when Load Vandadium button is pressed
                                       $ ;clear vanadium will restore its content to eff pointer
         fileSel:          obj_new(),  $ ;data file selection
         fileBas:          0L,         $ ;file selection base
         intn_lab:         '',         $ ;intensity label
         legd_lab:         '',         $ ;legend label for 1d plot
         titl_lab:         '',         $ ;title label
         monrate:          0e,         $ ;monitor rate from dcs data file
         kfactor:          ptr_new(),  $ ;(ki/kf)
                                       $ ;-powder,single crystal,diffuse scattering multiple E
                                       $ ; [ntchan] not rebinned
                                       $ ;-diffuse scattering sinegle E
                                       $ ; 1 for elastic and all time channels ki/kf_ave for the others
         samp_typ:         1s,         $ ;0-powder, 1-single crystal, 2-diffuse scattering single E, 3-diffuse multiple E
         rebinsize:        ptr_new(),  $ ;allowed rebin size
         e_bin:            intarr(5),  $ ;ebin size/erange choice
                                       $ ; [0:1] e_bin size [2:3] erange choice [4] e_bin size for diffuse scattering mult. E
         e_range:          [0e,0e],    $ ;diffuse scattering energy range
         projBas:          0L,         $ ;projection base
         dispBas:          0L,         $ ;display base
         latt_parm:        dblarr(6),  $ ;lattice parameters [a,b,c,alpha,beta,gamma]
         orie_u:           [1e,0e,0e], $ ;[ux,uy,uz]
         orie_v:           [0e,1e,0e], $ ;[vx,vy,vz]
         locklattuv:       0b,         $ ;flag for keeping lattice parameters and u&v from changed by loading macs files
         checklattuv:      0b,         $ ;flag for checking lattice parameters and u&v in files for consistency
         orie_psi:         0e,         $ ;psi
         orie_offset:      0e,         $ ;save the offset angle of psi
         view_type:        strarr(5),  $ ;view  type ['Energy','|Q|','2Theta','Det Group Number','Time Channel']
         view_label:       strarr(5),  $ ;labels     ['E','|Q|','2Theta','Det Group', 'T']
         view_unit:        strarr(5),  $ ;units      ['(meV)','(A^-1)','(^o)','','(usec)']
         view_u1typ:       1s,         $ ;powder u1 type 0-E, 1-Q, 2-2theta, 3-Det Group
         view_u1:          fltarr(3),  $ ;[u1h,u1k,u1l]
         view_u1lab:       strarr(2),  $ ;['|Q|','Qh'] -u1 label
         view_u1fold:      0b,         $ ;flag for folding u1 axis, single crystal only
         view_u1folddir:   ptr_new(),  $ ;folding direction, -1:minus, 0:auto, 1:plus
         view_u1foldcenter:ptr_new(),  $ ;folding center for u1, default is 0
         view_u2typ:       0s,         $ ;powder u2 type 0-E, 1-Q, 2-2theta, 3-Det Group
         view_u2:          fltarr(3),  $ ;[u2h,u2k,u2l]
         view_u2lab:       strarr(2),  $ ;['E','Qk']   -u2 label
         view_u2fold:      0b,         $ ;flag for folding u2 axis, single crystal only
         view_u2folddir:   ptr_new(),  $ ;folding direction, -1:minus, 0:auto, 1:plus
         view_u2foldcenter:ptr_new(),  $ ;folding center for u2, default is 0
         view_u3:          fltarr(3),  $ ;[u3h,u3k,u3l]
         view_u3lab:       '',         $ ;u3 label
         view_u3fold:      0b,         $ ;flag for folding u3 axis, single crystal only
         view_u3folddir:   ptr_new(),  $ ;folding direction, -1:minus, 0:auto, 1:plus
         view_u3foldcenter:ptr_new(),  $ ;folding center for u3, default is 0
         volsel_x:         intarr(4),  $ ;x-axis selection for volume plot
         volsel_y:         intarr(4),  $ ;y-axis selection for volume plot
         volsel_z:         intarr(4),  $ ;z-axis selection for volume plot
         volsel_t:         ptr_new(),  $ ;thickness selection for volume plot, for samp_typ=-1
         volsel_tft:       ptr_new(),  $ ;thickness range for volume plot,for samp_typ=-1
         volsel_t1:        0s,         $ ;thickness selection for volume tab
         volsel_tft1:      fltarr(2),  $ ;thickness range for volume tab
         vol_xran:         fltarr(2),  $ ;x-ran
         vol_xstep:        0e,         $ ;x-step
         vol_yran:         fltarr(2),  $ ;y-ran
         vol_ystep:        0e,         $ ;y-step
         vol_zran:         fltarr(2),  $ ;z-ran
         vol_zstep:        0e,         $ ;z-step
         vol_iran:         fltarr(2),  $ ;i-ran
         vol_smooth:       1s,         $ ;smoothing level
         vol_interp:       0s,         $ ;interpolation 0: nearest neighbor 1:trilinear
         vol_color:        4s,         $ ;color map number 4-Blue/Green/Red/Yellow
         slice_x:          intarr(4),  $ ;x-axis selection of contour plot
         slice_y:          intarr(4),  $ ;y-axis selection of contour plot
         slice_t:          ptr_new(),  $ ;thickness selection of the slice for volume data
         slice_tft:        ptr_new(),  $ ;thickness range of the slice for volume data
         slice_xran:       fltarr(2),  $ ;x range [from,to]
         slice_xstep:      0e,         $ ;x step
         slice_yran:       fltarr(2),  $ ;y range [from,to]
         slice_ystep:      0e,         $ ;y step
         slice_t1:         intarr(4),  $ ;slice thickness 1 slection for tab
         slice_t2:         intarr(4),  $ ;slice thickness 2 slection for tab
         slice_t1ran:      fltarr(2),  $ ;thickness 1 range[from,to]
         slice_t2ran:      fltarr(2),  $ ;thickness 2 range[from,to]
         slice_iran:       fltarr(2),  $ ;intensity range[from,to]
         slice_smooth:     1s,         $ ;smooth level
         slice_sm_kernel:  fltarr(3,3),$ ;default smooth kernel
         slice_shading:    1s,         $ ;shading 0-flat 1-Gouraud
         slice_cmap:       ptr_new(),  $ ;color map names
         slice_color:      13s,        $ ;color map number 13-RAINBOW
         cut_x:            intarr(4),  $ ;cut along axis number
         cut_ran:          fltarr(2),  $ ;cut range
         cut_step:         0e,         $ ;cut step
         cutsel_t:         ptr_new(),  $ ;thickness selection of the cut for volume data
         cutsel_tft:       ptr_new(),  $ ;thickness range of the cut for volume data
         cut_t1:           intarr(4),  $ ;thickness 1 selection
         cut_t2:           intarr(4),  $ ;thickness 2 selection
         cut_t3:           intarr(4),  $ ;thickness 3 selection
         cut_t1ran:        fltarr(2),  $ ;thickness 1 range
         cut_t2ran:        fltarr(2),  $ ;thickness 2 range
         cut_t3ran:        fltarr(2),  $ ;thickness 3 range
         cut_y:            intarr(4),  $ ;y axis number
         cut_yran:         fltarr(2),  $ ;y range
         cut_symbol:       0s,         $ ;plot symbol number
         symbols:          ptr_new(),  $ ;symbol names
         cut_color:        0s,         $ ;color number
         colors:           ptr_new(),  $ ;color names
         coldefine:        bytarr(3),  $ ;user defined color
         cut_line:         0s,         $ ;linestyle number,default solid line
         lines:            ptr_new(),  $ ;linestyle names
         traj_type:        strarr(5),  $ ;traj type names
         traj_x:           intarr(4),  $ ;trajectory x type
         traj_xran:        fltarr(2),  $ ;[xfrom,xto]
         traj_y:           intarr(4),  $ ;trajectory y type
         traj_yran:        fltarr(2),  $ ;[yfrom,yto]
         traj_z1:          intarr(4),  $ ;trajectory z type
         traj_z1ran:       fltarr(2),  $ ;[zfrom,zto]
         traj_z2:          intarr(4),  $ ;trajectory z type
         traj_z2ran:       fltarr(2),  $ ;[zfrom,zto]
         traj_layer:       0s,         $ ;for trajectory overplot coloring
         psi_spe:          0b,         $ ;flag indicating the definition of psi in the detPosPtr is SPE type or not, 1: yes 0: no
         detPosPtr:        ptr_new(),  $ ;pointer which points to a structure
                                       $ ;{two_theta:[ndet], dtwo_theta:[ndet], psi:[ndet], dpsi:[ndet]}
         dataStrPtr:       ptr_new(),  $ ;pointer which points to a data structure
                                       $ ;-single crystal,powder
                                       $ ; {qty:[ntchan,ndet], err:[ntchan,ndet], e:[ntchan],weight:weight,eief:eief,ewid:ewid[ntchan],$
                                       $ ;  info:{ch_wl:ch_wl,ch_ms:ch_ms,ch_srdenom:ch_srdenom,ch_res:ch_res,tsdmin:tsdmin,$
                                       $ ;  temperature:temperature,time:[ntchan]}}    time is only for dcs data
                                       $ ;-diffuse scattering
                                       $ ; {qty:[ndet,nfiles],err:[ndet,nfiles],rotation:[nfiles],weight:[nfiles],eief:eief,ewid:ewid[ntchan],$
                                       $ ;  info:{ch_wl:ch_wl,ch_ms:ch_ms,ch_srdenom:ch_srdenom,ch_res:ch_res,tsdmin:tsdmin,$
                                       $ ;  temperature:temperature},filename:open_file,filepath:open_path}
                                       $ ;-MACS
                                       $ ; {qty:[2,ndet,ndat],err:[2,ndet,ndat],rotation:[ndat],kidney:[ndat],weight:[ndat],cfx:[ndat],Ei:[ndat],Ef:[ndat],temperature:[ndat],$
                                       $ ;  info:{latticeparm:latticeparm,latticeori:latticeori}}
         projStrPtr:       ptr_new(),  $ ;pointer which points to a structure
                                       $ ;-single crystal
                                       $ ; {V:[3,nen,ndet],en:[nen],two_theta:[ndet],qty:[nen,ndet],err:[nen,ndet],ewid:[nen],temperature:[nen]} 
                                       $ ; temperature is only for MACS with extra view axis
                                       $ ;-powder
                                       $ ; {Q:[nen,ndet],en:[nen],two_theta:[ndet],qty:[nen,ndet],err:[nen,ndet],ewid:[nen],time:[nen]}
                                       $ ;-diffuse scattering
                                       $ ; single E
                                       $ ; {V:[3,ndet,nfile],en:[1],two_theta:[ndet],qty:[ndet,nfile],err:[ndet,nfile],ewid:[1],temperature:[nfile],<daQV:[nproj,npt]>}
                                       $ ; temperature is only for WAND with extra view axis
                                       $ ; multiple E
                                       $ ; {V:[3,nen,ndet,nfile],en:[nen],two_theta:[ndet],qty:[nen,ndet,nfile],err:[nen,ndet,nfile],ewid:[nen],<daQV:[nproj,npt]>}
         dataPtr:          ptr_new(),  $ ;general volume data pointer
                                       $ ;{data:[ndat,nitem],label:[nitem],unit:[nitem],is_uniq:[nitem],instrument:''} ; the first two item needs to be I and dI
         powd_line:        ptr_new(),  $ ;powder line for diffuse scattering mode
         absp_info:        ptr_new(),  $ ;absorption information [type,sampling pts,interoplate,l0,l1,Rin/width,Rout/thickness,angle]
                                       $ ; type: 0-cylinder 1-annulus 2-sphere 3-flat plate
         absp_npts:        ptrarr(4),  $ ;saving npts choice option numbers
         absp_nptst:       ptrarr(4),  $ ;saving npts choice option strings
         file_hist:        ptrarr(10), $ ;save the latest 10 file selections
         n_history:        0s,         $ ;number of histories
         gr_list:          obj_new(),  $ ;container of all the graphic objects
         current:          obj_new(),  $ ;current plot object
         error:            0b,         $ ;error flag
         helpfile:         ['',''],    $ ;help file
         dirs:             ['','',''], $ ;[data directory, work directory, parameter file directory]
         parmname:         '',         $ ;default parameter file name
         pathsep:          '',         $ ;path separator
         symfont:          '',         $ ;symbol font name
         eief:             0e,         $ ;save fixed energy, Ei for direct geometry, Ef for inverse geometry
         estartend:        fltarr(3),  $ ;save [e_start, e_end, n_en] for projection calculation, only used when user function is specified 
         phxfile:          '',         $ ;save spe detector info phx file name
         psi:              fltarr(4),  $ ;save psi start,end,step info, psi[3] saves the first file angle
         kidney:           fltarr(3),  $ ;save macs kidney start,end,step info for projection calculation
         kidney_offset:    0e,         $ ;save macs kidney offset angle
         whatsnew:         ptr_new(),  $ ;what's new infomation
         info:             ptr_new(),  $ ;self information
         idl_version:      0e,         $ ;IDL version number
         ftoolflag:        [0b,0b],    $ ;flag for filetools: [resolution convolution tool 0-never started 1-started, macs deteff changed]
         DAVETool:         obj_new()   $ ;from DAVE
    }
end

pro dcs_mslice,group_leader=group_leader,workDir=workDir,dataDir=dataDir,DAVETool=DAVETool,dataPtr=dataPtr,instrname=instrname,no_parmfile=no_parmfile,obj_mslice=obj_mslice,_extra=extra
    obj_mslice = obj_new('dcs_mslice',group_leader=group_leader,workDir=workDir,dataDir=dataDir,DAVETool=DAVETool,dataPtr=dataPtr,instrname=instrname,no_parmfile=no_parmfile)
end